@@ -1,14 +1,21 @@
-/* 
+/*
  *  Copyright (C) 1999-2002 Bernd Gehrmann
  *                          bernd@mail.berlios.de
+ *  Copyright (c) 2002-2005 Christian Loose <christian.loose@kdemail.net>
  *
- * This program may be distributed under the terms of the Q Public
- * License as defined by Trolltech AS of Norway and appearing in the
- * file LICENSE.QPL included in the packaging of this file.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 
@@ -18,19 +25,26 @@
 #include <kparts/part.h>
 #include <kparts/browserextension.h>
 #include <kparts/genericfactory.h>
+#include <kparts/statusbarextension.h>
 
+#include "addremovedlg.h"
 #include "commitdlg.h"
 #include "checkoutdlg.h"
 #include "watchdlg.h"
 #include "tagdlg.h"
 
+namespace Cervisia { class EditWithMenu; }
 class QLabel;
+class QListViewItem;
 class QSplitter;
 class QTimer;
 class UpdateView;
 class ProtocolView;
 class KAboutData;
+class KListView;
 class KRecentFilesAction;
+class CvsService_stub;
+class CervisiaBrowserExtension;
 
 
 /**
@@ -52,31 +66,20 @@ public:
 
     QString sandBox() const { return sandbox; }
 
-    virtual void readProperties(KConfig *config);
-    virtual void saveProperties(KConfig *config);
-
-    void readDialogProperties( KConfig *config );
-    void saveDialogProperties( KConfig *config );
-    
     static KAboutData* createAboutData();
 
-signals:
-    void filterStatusChanged(QString status);
-
 public slots:
+    // unused because we overwrite the default behaviour of openURL()
     virtual bool openFile() { return true; }
     virtual bool openURL( const KURL & );
-    void openSandbox(const QString &dirname);
 
     void openFile(QString filename);
     void openFiles(const QStringList &filenames);
-    void popupRequested();
+    void popupRequested(KListView*, QListViewItem*, const QPoint&);
     void updateActions();
-    
-    void aboutCervisia();
-    void slotOpenSandbox( const KURL &url );
 
-    void slotOpenSandbox();
+    void aboutCervisia();
+
     void slotOpen();
     void slotResolve();
     void slotStatus();
@@ -87,13 +90,16 @@ public slots:
     void slotAddBinary();
 
     void slotRemove();
+    void slotFileProperties();
     void slotRevert();
     void slotBrowseLog();
     //    void slotBrowseMultiLog();
     void slotAnnotate();
-    void slotDiff();
+    void slotDiffBase();
+    void slotDiffHead();
     void slotLastChange();
     void slotHistory();
+    void slotCreateRepository();
     void slotCheckout();
     void slotImport();
     void slotRepositories();
@@ -116,7 +122,9 @@ public slots:
     void slotHideFiles();
     void slotHideUpToDate();
     void slotHideRemoved();
+
     void slotHideNotInCVS();
+    void slotHideEmptyDirectories();
 
     void slotFoldTree();
     void slotUnfoldTree();
@@ -129,17 +137,31 @@ public slots:
     void slotCVSInfo();
 
 protected slots:
-    void slotJobFinished(bool /*success*/);
+    void slotJobFinished();
+
+private slots:
+    // called by menu action "Open Sandbox..."
+    void slotOpenSandbox();
+    void slotSetupStatusBar();
+
+protected:
+    virtual void guiActivateEvent(KParts::GUIActivateEvent* event);
+
+private:
+    enum JobType { Unknown, Commit };
 
-private:    
     void setupActions();
 
-    void updateOrStatus(bool noact, const QString &extraopt);
-    void commitOrAddOrRemove(CommitDialog::ActionType action);
+    void readSettings();
+    void writeSettings();
+
+    bool openSandbox(const QString &dirname);
+    void updateSandbox(const QString &extraopt = QString::null);
+    void addOrRemove(AddRemoveDialog::ActionType action);
     void addOrRemoveWatch(WatchDialog::ActionType action);
-    void importOrCheckout(CheckoutDialog::ActionType action);
-    void createOrDeleteTag(TagDialog::ActionType action);
+    void createOrDeleteTag(Cervisia::TagDialog::ActionType action);
     void showJobStart(const QString &command);
+    void showDiff(const QString& revision);
     void setFilter();
 
     UpdateView *update;
@@ -147,19 +169,26 @@ private:
     bool hasRunningJob;
     QSplitter *splitter;
 
-    // TODO: Find a new way to handle the status items as you can't do this with KParts yet
-    //    QLabel *filterLabel;
     QString sandbox;
     QString repository;
 
     QString changelogstr;
     QStringList recentCommits;
-    bool opt_hideFiles, opt_hideUpToDate, opt_hideRemoved, opt_hideNotInCVS;
+    bool opt_hideFiles, opt_hideUpToDate, opt_hideRemoved, opt_hideNotInCVS, opt_hideEmptyDirectories;
     bool opt_createDirs, opt_pruneDirs;
     bool opt_updateRecursive, opt_commitRecursive, opt_doCVSEdit;
 
     //for the Open Recent directories
     KRecentFilesAction *recent;
+
+    CvsService_stub*            cvsService;
+    KParts::StatusBarExtension* m_statusBar;
+    CervisiaBrowserExtension*   m_browserExt;
+    QLabel*                     filterLabel;
+
+    int                         m_editWithId;
+    Cervisia::EditWithMenu*     m_currentEditMenu;
+    JobType                     m_jobType;
 };
 
 typedef KParts::GenericFactory<CervisiaPart> CervisiaFactory;
@@ -174,6 +203,11 @@ class CervisiaBrowserExtension : public KParts::BrowserExtension
 public:
     CervisiaBrowserExtension( CervisiaPart * );
     ~CervisiaBrowserExtension();
+
+    void setPropertiesActionEnabled(bool enabled);
+
+public slots:
+    void properties();
 };
 
 #endif // CERVISIAPART_H
