@@ -1,29 +1,35 @@
 /*
  *  Copyright (C) 1999-2002 Bernd Gehrmann
  *                          bernd@mail.berlios.de
+ *  Copyright (c) 2004 Christian Loose <christian.loose@hamburg.de>
  *
- * This program may be distributed under the terms of the Q Public
- * License as defined by Trolltech AS of Norway and appearing in the
- * file LICENSE.QPL included in the packaging of this file.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 
 #ifndef LOGTREE_H
 #define LOGTREE_H
 
-#include <qmemarray.h>
+
 #include <qptrlist.h>
 
-#include "qttableview.h"
+#include <qtable.h>
 
 
 class LogTreeItem;
 class LogTreeConnection;
-class TipLabel;
 
 namespace Cervisia
 {
@@ -35,47 +41,43 @@ typedef QPtrList<LogTreeItem> LogTreeItemList;
 typedef QPtrList<LogTreeConnection> LogTreeConnectionList;
 
 
-class LogTreeView : public QtTableView
+class LogTreeView : public QTable
 {
     Q_OBJECT
 
 public:
     explicit LogTreeView( QWidget *parent=0, const char *name=0 );
-    virtual ~LogTreeView();
-    
+
     void addRevision(const Cervisia::LogInfo& logInfo);
     void setSelectedPair(QString selectionA, QString selectionB);
     void collectConnections();
     void recomputeCellSizes();
+    virtual void paintCell(QPainter *p, int row, int col, const QRect& cr,
+                           bool selected, const QColorGroup& cg);
 
     virtual QSize sizeHint() const;
+    
+    virtual QString text(int row, int col) const;
 
 signals:
     void revisionClicked(QString rev, bool rmb);
 
 protected:
-    //    virtual bool eventFilter(QObject *o, QEvent *e);
-    virtual void mousePressEvent(QMouseEvent *e);
-    virtual void mouseMoveEvent(QMouseEvent *e);
-    virtual void windowActivationChange(bool oldActive);
-    virtual void leaveEvent(QEvent *e);
-    virtual void setupPainter(QPainter *p);
-    virtual void paintCell(QPainter *p, int row, int col);
-    virtual int cellWidth(int col);
-    virtual int cellHeight(int row);
+    virtual void contentsMousePressEvent(QMouseEvent *e);
+
+private slots:
+
+    void slotQueryToolTip(const QPoint&, QRect&, QString&);
 
 private:
+    QSize computeSize(const Cervisia::LogInfo&, int* = 0, int* = 0) const;
     void paintRevisionCell(QPainter *p, int row, int col, const Cervisia::LogInfo& logInfo,
-			   bool followed, bool branched, bool selected);
+                           bool followed, bool branched, bool selected);
     void paintConnector(QPainter *p, int row, int col, bool followed, bool branched);
-    void hideLabel();
 
     LogTreeItemList items;
     LogTreeConnectionList connections;
     int currentRow, currentCol;
-    TipLabel *currentLabel;
-    QMemArray<int> colWidths;
-    QMemArray<int> rowHeights;
 
     static const int BORDER;
     static const int INSPACE;
