@@ -3,13 +3,19 @@
  *                          bernd@mail.berlios.de
  *  Copyright (c) 2002-2004 Christian Loose <christian.loose@kdemail.net>
  *
- * This program may be distributed under the terms of the Q Public
- * License as defined by Trolltech AS of Norway and appearing in the
- * file LICENSE.QPL included in the packaging of this file.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 
@@ -323,8 +329,7 @@ void RepositoryDialog::slotAddClicked()
         for( ; item; item = item->nextSibling() )
             if( item->text(0) == repo )
             {
-                KMessageBox::information(this, "Cervisia",
-                                         i18n("This repository is already known."));
+                KMessageBox::information(this, i18n("This repository is already known."));
                 return;
             }
 
