@@ -3,16 +3,21 @@
  *                          bernd@mail.berlios.de
  *  Copyright (c) 2002-2004 Christian Loose <christian.loose@kdemail.net>
  *
- * This program may be distributed under the terms of the Q Public
- * License as defined by Trolltech AS of Norway and appearing in the
- * file LICENSE.QPL included in the packaging of this file.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-
 #ifndef SETTINGSDLG_H
 #define SETTINGSDLG_H
 
@@ -27,6 +32,7 @@ class KLineEdit;
 class KConfig;
 class KColorButton;
 class KURLRequester;
+class AdvancedPage;
 
 
 class FontButton : public QPushButton
@@ -63,12 +69,9 @@ private:
     void addLookAndFeelPage();
 
     KConfig *config;
-    KIntNumInput *timeoutedit;
     KIntNumInput *contextedit;
     KIntNumInput *tabwidthedit;
     KURLRequester *cvspathedit;
-    KIntNumInput* m_defaultCompression;
-    QCheckBox *usesshagent;
     KLineEdit *usernameedit;
     KLineEdit *diffoptedit;
     KURLRequester *extdiffedit;
@@ -82,11 +85,13 @@ private:
     KColorButton* m_conflictButton;
     KColorButton* m_localChangeButton;
     KColorButton* m_remoteChangeButton;
+    KColorButton* m_notInCvsButton;
     KColorButton* m_diffChangeButton;
     KColorButton* m_diffInsertButton;
     KColorButton* m_diffDeleteButton;
 
     QCheckBox*    m_splitterBox;
+    AdvancedPage* m_advancedPage;
 
     KConfig* serviceConfig;
 };
