@@ -598,6 +598,9 @@ void CatalogManager::openFile(QString filename, QString package)
         }
 
         kdDebug(KBABEL_CATMAN) << callfunc << endl;
+	
+	// update the user timestamp for KBabel to get it a focus
+	kapp->updateRemoteUserTimestamp ("kbabel");
 
         if( !client->send("kbabel","KBabelIFace", callfunc, data) )
             KMessageBox::error(this, i18n("Can not send a message to KBabel.\n"
@@ -629,6 +632,9 @@ void CatalogManager::openFile(QString filename, QString package, int msgid)
 
         kdDebug(KBABEL_CATMAN) << callfunc << endl;
 
+	// update the user timestamp for KBabel to get it a focus
+	kapp->updateRemoteUserTimestamp ("kbabel");
+
         if( !client->send("kbabel","KBabelIFace", callfunc, data) )
             KMessageBox::error(this, i18n("Can not send a message to KBabel.\n"
                                           "Please check your installation of KDE."));
@@ -657,6 +663,9 @@ void CatalogManager::openFileInNewWindow(QString filename, QString package)
             callfunc="openURL(QCString, QCString, WId,int,QCString)";
         }
 
+	// update the user timestamp for KBabel to get it a focus
+	kapp->updateRemoteUserTimestamp ("kbabel");
+
         if( !client->send("kbabel","KBabelIFace", callfunc, data) )
             KMessageBox::error(this, i18n("Can not send a message to KBabel.\n"
                                           "Please check your installation of KDE."));
@@ -684,6 +693,9 @@ void CatalogManager::openTemplate(QString openFilename,QString saveFilename,QStr
             callfunc="openTemplate(QCString,QCString,QCString,int,QCString)";
         }
 
+	// update the user timestamp for KBabel to get it a focus
+	kapp->updateRemoteUserTimestamp ("kbabel");
+
         if( !client->send("kbabel","KBabelIFace", callfunc, data) )
             KMessageBox::error(this, i18n("Can not send a message to KBabel.\n"
                                           "Please check your installation of KDE."));
@@ -711,6 +723,9 @@ void CatalogManager::openTemplateInNewWindow(QString openFilename,QString saveFi
             callfunc="openTemplate(QCString,QCString,QCString,int,QCString)";
         }
 
+	// update the user timestamp for KBabel to get it a focus
+	kapp->updateRemoteUserTimestamp ("kbabel");
+
         if( !client->send("kbabel","KBabelIFace", callfunc, data) )
             KMessageBox::error(this, i18n("Can not send a message to KBabel.\n"
                                           "Please check your installation of KDE."));
@@ -727,6 +742,10 @@ void CatalogManager::spellcheck()
         QByteArray data;
         QDataStream arg(data, IO_WriteOnly);
         arg << fileList;
+
+	// update the user timestamp for KBabel to get it a focus
+	kapp->updateRemoteUserTimestamp ("kbabel");
+
         if( !client->send("kbabel","KBabelIFace", "spellcheck(QStringList)", data) )
             KMessageBox::error(this, i18n("Can not send a message to KBabel.\n"
                                           "Please check your installation of KDE."));
@@ -743,6 +762,10 @@ void CatalogManager::markedSpellcheck()
         QByteArray data;
         QDataStream arg(data, IO_WriteOnly);
         arg << fileList;
+
+	// update the user timestamp for KBabel to get it a focus
+	kapp->updateRemoteUserTimestamp ("kbabel");
+
         if( !client->send("kbabel","KBabelIFace", "spellcheck(QStringList)", data) )
             KMessageBox::error(this, i18n("Can not send a message to KBabel.\n"
                                           "Please check your installation of KDE."));
