@@ -19,6 +19,17 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
+
 **************************************************************************** */
 #ifndef CATALOGMANAGERVIEW_H
 #define CATALOGMANAGERVIEW_H
@@ -35,6 +46,7 @@
 #include <kprocess.h>
 #include <qptrlist.h>
 
+#include "kbproject.h"
 #include "projectsettings.h"
 #include "findoptions.h"
 #include "cvsresources.h"
@@ -63,7 +75,7 @@ class CatalogManagerView : public QListView
 {
    Q_OBJECT
 public:
-   CatalogManagerView(QString configFile, QWidget* parent=0, const char* name=0);
+   CatalogManagerView(KBabel::Project::Ptr project, QWidget* parent=0, const char* name=0);
    virtual ~CatalogManagerView();
 
    KBabel::CatManSettings settings() const;
@@ -425,7 +437,7 @@ private:
    bool _markAsFuzzy;
    bool _ignoreFuzzy;
 
-   QString _configFile;
+   KBabel::Project::Ptr _project;
 };
 
 #endif // CATALOGMANAGERVIEW_H
