@@ -20,6 +20,17 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.	    
+	    
 **************************************************************************** */
 #include "catalogsettings.h"
 #include "argextractor.h"
@@ -41,7 +52,7 @@ QStringList *ArgExtractor::regExpList()
 {
     if(!_argList)
     {
-        _argList = sdAL.setObject( new QStringList );
+         sdAL.setObject( _argList, new QStringList );
         //Build the default arg list
         _argList->append("%[ndioxXucsfeEgGp]");
         _argList->append("%([0-9]+(\\$))?[-+'#0]?[0-9]*(.[0-9]+)?[hlL]?[dioxXucsfeEgGp]");  
@@ -53,7 +64,7 @@ QStringList *ArgExtractor::regExpList()
 
 void ArgExtractor::setArgExpressions( QStringList* list )
 {
-    _argList = sdAL.setObject( new QStringList );
+    sdAL.setObject( _argList, new QStringList );
     for( QStringList::Iterator it = list->begin() ; it != list->end() ; ++it )
         _argList->append(*it);
 }
