@@ -152,7 +152,7 @@ int Catalog::indexForMsgid(const QString& id) const
     int i=0;
 	QValueVector<CatalogItem>::ConstIterator it = d->_entries.begin();
 
-    while(it != d->_entries.end() && !((*it).msgid().contains(id)))
+    while(it != d->_entries.end() && !((*it).msgid(true).contains(id)))
     {
         ++it;
         i++;
@@ -2514,6 +2514,7 @@ bool Catalog::findNext(const FindOptions* findOpts, DocPosition& docPos, int& le
 						else
 						{
 							docPos.item++;
+							docPos.form = 0;
 						}
 					}
 					break;
@@ -2545,6 +2546,7 @@ bool Catalog::findNext(const FindOptions* findOpts, DocPosition& docPos, int& le
 						else
 						{
 							docPos.item++;
+							docPos.form = 0;
 						}
 					}
 					break;
@@ -2558,6 +2560,7 @@ bool Catalog::findNext(const FindOptions* findOpts, DocPosition& docPos, int& le
 						{
 							docPos.part = Msgid;
 							docPos.item++;
+							docPos.form = 0;
 						}
 					}
 					else if(findOpts->inMsgstr){
@@ -2580,6 +2583,7 @@ bool Catalog::findNext(const FindOptions* findOpts, DocPosition& docPos, int& le
 						else
 						{
 							docPos.item++;
+							docPos.form = 0;
 						}
 					}
 					break;
