@@ -3,6 +3,8 @@
 
   Copyright (C) 1999-2000 by Matthias Kiefer
                             <matthias.kiefer@gmx.de>
+		2004	  by Stanislav Visnovsky
+			    <visnovsky@kde.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -18,6 +20,16 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
 **************************************************************************** */
 #include "catalogsettings.h"
 #include <kconfig.h>
@@ -38,28 +50,6 @@
 
 using namespace KBabel;
 
-const bool           Defaults::Save::autoUpdate=true;
-const bool           Defaults::Save::updateLastTranslator=true;
-const bool           Defaults::Save::updateRevisionDate=true;
-const bool           Defaults::Save::updateLanguageTeam=true;
-const bool           Defaults::Save::updateCharset=true;
-const bool           Defaults::Save::updateEncoding=true;
-const bool           Defaults::Save::updateProject=true;
-const bool           Defaults::Save::updateTranslatorCopyright=true;
-const CopyrightUpdate Defaults::Save::FSFCopyright=Update;
-const bool           Defaults::Save::updateDescription=true;
-const FileEncoding   Defaults::Save::encoding=UTF8;
-const bool           Defaults::Save::useOldEncoding=true;
-const Qt::DateFormat Defaults::Save::dateFormat=Qt::ISODate;
-const bool           Defaults::Save::autoSyntaxCheck=true;
-const bool	     Defaults::Save::saveObsolete=false;
-const int            Defaults::Save::autoSaveDelay = 0;
-
-const QString Defaults::Save::customDateFormat()
-{
-    return QString("%Y-%m-%d %H:%M%z");
-}
-
 QString KBabel::charsetString(const QTextCodec *codec)
 {
     QString encodingStr = "";
@@ -153,13 +143,13 @@ QString KBabel::GNUPluralForms(const QString& lang)
     return res;
 }
 
-QString KBabel::charsetString(FileEncoding e)
+QString KBabel::charsetString(int e)
 {
     QString encodingStr = "";
 
     switch(e)
     {
-        case Locale:
+        case ProjectSettingsBase::Locale:
         {
             QTextCodec *codec=QTextCodec::codecForLocale();
             if(codec)
@@ -169,12 +159,12 @@ QString KBabel::charsetString(FileEncoding e)
 
             break;
         }
-        case UTF8:
+        case ProjectSettingsBase::UTF8:
         {
             encodingStr="UTF-8";
             break;
         }
-        case UTF16:
+        case ProjectSettingsBase::UTF16:
         {
             encodingStr="UTF-16";
             break;
@@ -184,141 +174,64 @@ QString KBabel::charsetString(FileEncoding e)
     return encodingStr;
 }
 
-const QString Defaults::Save::projectString()
-{
-    return QString("@PACKAGE@");
-}
-
-const QString Defaults::Save::descriptionString()
-{
-    return QString("translation of @PACKAGE@.po to @LANGUAGE@");
-}
-
-Defaults::Identity::Identity(): 
-    numberOfPluralForms(2), 
-    checkPluralArgument(true),
-    gnuPluralFormHeader("")
-{
-    _authorName=QString::null;
-    _authorLocalizedName=QString::null;
-    _authorEmail=QString::null;
-    _mailingList=QString::null;
-    _timezone=QString::null;
-    _languageName=QString::null;
-    _languageCode=QString::null;
-}
-
 QString Defaults::Identity::authorName()
 {
-   if(_authorName.isNull())
-   {
-       KEMailSettings emSet;
-      _authorName=emSet.getSetting(KEMailSettings::RealName);
-   }
-
-   return _authorName;
+    KEMailSettings emSet;
+    return emSet.getSetting(KEMailSettings::RealName);
 }
 
 QString Defaults::Identity::authorLocalizedName()
 {
-    _authorLocalizedName=_authorName;
-    return _authorLocalizedName;
+    return authorName();
 }
 
 QString Defaults::Identity::authorEmail()
 {
-   if(_authorEmail.isNull())
-   {
-       KEMailSettings emSet;
-      _authorEmail=emSet.getSetting(KEMailSettings::EmailAddress);
-   }
-
-   return _authorEmail;
+    KEMailSettings emSet;
+    return emSet.getSetting(KEMailSettings::EmailAddress);
 }
 
 QString Defaults::Identity::mailingList()
 {
-   if(_mailingList.isNull())
-   {
-      QString lang=Defaults::Identity::languageCode();
-      int temp=lang.find("_");
-      lang=lang.left(temp);
-      _mailingList=lang+"@li.org";
-   }
-
-   return _mailingList;
-}
-
-QString Defaults::Identity::languageName()
-{
-   if(_languageName.isNull())
-   {
-      _languageName="";
-   }
-
-   return _languageName;
+    QString lang=Defaults::Identity::languageCode();
+    int temp=lang.find("_");
+    lang=lang.left(temp);
+    return lang+"@li.org";
 }
 
 QString Defaults::Identity::languageCode()
 {   
-   if(_languageCode.isNull())
-   {     
-      // first try to get the language from KDE settings
-      KLocale *locale = KGlobal::locale();
-      QString lang;
-      if(locale)
-      {
-          lang=locale->languageList().first();
-      }
+    // first try to get the language from KDE settings
+    KLocale *locale = KGlobal::locale();
+    QString lang;
+    if(locale)
+    {
+        lang=locale->languageList().first();
+    }
 
-      if(lang.isEmpty())
-      {
-         lang=getenv("LC_ALL");
-         if(lang.isEmpty())
-         {
+    if(lang.isEmpty())
+    {
+        lang=getenv("LC_ALL");
+        if(lang.isEmpty())
+        {
             lang=getenv("LC_MESSAGES");
             if(lang.isEmpty())
             {
-               lang=getenv("LANG");
+        	lang=getenv("LANG");
             }
          }
-      }
+    }
 
-      _languageCode=lang;
-   }
-
-   return _languageCode;
+    return lang;
 }
 
 QString Defaults::Identity::timezone()
 {
-   if(_timezone.isNull())
-   {
-     _timezone=getenv("TIMEZONE");
-     if(_timezone.isEmpty())
-       _timezone="GMT";
-   }
+    QString timezone=getenv("TIMEZONE");
+    if(timezone.isEmpty())
+       timezone="GMT";
 
-   return _timezone;
-}
-
-
-const bool Defaults::Misc::useBzip = true;
-const bool Defaults::Misc::compressSingleFile = true;
-
-QChar Defaults::Misc::accelMarker()
-{
-    return QChar('&');
-}
-
-QRegExp Defaults::Misc::contextInfo()
-{
-    return QRegExp("^_:((?!\\\\n\\n).)+\\\\n\\n");
-}
-
-QRegExp Defaults::Misc::singularPlural()
-{
-    return QRegExp("_n:\\s");
+    return timezone;
 }
 
 QStringList Defaults::Tag::tagExpressions()
