@@ -3,7 +3,7 @@
 
   Copyright (C) 1999-2000 by Matthias Kiefer
                             <matthias.kiefer@gmx.de>
-		2001-2003 by Stanislav Visnovsky
+		2001-2004 by Stanislav Visnovsky
 			    <visnovsky@kde.org>
 
   This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,16 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
 **************************************************************************** */
 #ifndef CATALOGSETTINGS_H
 #define CATALOGSETTINGS_H
@@ -27,6 +37,9 @@
 #include <qstring.h>
 #include <qregexp.h>
 #include <qdatetime.h>
+#include <kdemacros.h>
+
+#include "kbprojectsettings.h"
 
 class QTextCodec;
 class QStringList;
@@ -34,11 +47,7 @@ class QStringList;
 namespace KBabel
 {
 
-enum FileEncoding{Locale,UTF8,UTF16};
-
-enum CopyrightUpdate{Remove,Update,NoChange,RemoveLine};
-
-struct SaveSettings
+struct KDE_EXPORT SaveSettings
 {
     bool autoUpdate;
     bool updateLastTranslator;
@@ -51,9 +60,9 @@ struct SaveSettings
     bool updateDescription;
     QString descriptionString;
     bool updateTranslatorCopyright;
-    CopyrightUpdate FSFCopyright;
+    int FSFCopyright;
 
-    FileEncoding encoding;
+    int encoding;
     bool useOldEncoding;
 
     Qt::DateFormat dateFormat;
@@ -67,7 +76,7 @@ struct SaveSettings
     int autoSaveDelay;
 };
 
-struct IdentitySettings
+struct KDE_EXPORT IdentitySettings
 {
     QString authorName;
     QString authorLocalizedName;
@@ -91,7 +100,7 @@ struct IdentitySettings
 };
 
 
-struct MiscSettings
+struct KDE_EXPORT MiscSettings
 {
    /** 
     * The char, that marks keyboard accelerators.
@@ -122,7 +131,7 @@ struct MiscSettings
    * Use compression for single file attachments. 
    * Default is true.
    */
-  bool compressSingleFile;
+  bool KDE_EXPORT compressSingleFile;
 };
 
 struct TagSettings
@@ -137,89 +146,30 @@ struct TagSettings
     QStringList argExpressions;
 };
 
-QString charsetString(FileEncoding);
-QString charsetString(const QTextCodec *codec);
-QString GNUPluralForms(const QString& lang);
+KDE_EXPORT QString charsetString(int encoding);
+KDE_EXPORT QString charsetString(const QTextCodec *codec);
+KDE_EXPORT QString GNUPluralForms(const QString& lang);
 
 namespace Defaults
 {
-   /**
-   * this class contains or computes the defaults values of the
-   * Save preferences
-   */
-   class Save
+   class KDE_EXPORT Identity
    {
     public:
-       static const bool autoUpdate;
-       static const bool updateLastTranslator;
-       static const bool updateRevisionDate;
-       static const bool updateLanguageTeam;
-       static const bool updateCharset;
-       static const bool updateEncoding;
-       static const bool updateProject;
-       static const bool updateTranslatorCopyright;
-       static const bool updateDescription;
-       
-       static const CopyrightUpdate FSFCopyright;
-
-       static const FileEncoding encoding;
-       static const bool useOldEncoding;
-
-       static const Qt::DateFormat dateFormat;
-
-       static const bool autoSyntaxCheck;
-       static const bool saveObsolete;
-       
-       static const int autoSaveDelay;
-
-       static const QString customDateFormat();
-       static const QString projectString();
-       static const QString descriptionString();
+       static QString authorName();
+       static QString authorLocalizedName();
+       static QString authorEmail();
+       static QString languageCode();
+       static QString mailingList();
+       static QString timezone();
    };
 
-   class Identity
-   {
-    public:
-       Identity();
-       QString authorName();
-       QString authorLocalizedName();
-       QString authorEmail();
-       QString languageName();
-       QString languageCode();
-       QString mailingList();
-       QString timezone();
-
-       const int numberOfPluralForms;
-       const bool checkPluralArgument;
-       QString gnuPluralFormHeader;
-
-    private:
-       QString _authorName;
-       QString _authorLocalizedName;
-       QString _authorEmail;
-       QString _mailingList;
-       QString _timezone;
-       QString _languageName;
-       QString _languageCode;
-   };
-
-   class Misc
-   {
-      public:
-         QChar accelMarker();
-         QRegExp contextInfo();
-         QRegExp singularPlural();
-
-	 static const bool useBzip;
-	 static const bool compressSingleFile;
-   };
-   
-   class Tag
+   class KDE_EXPORT Tag
    {
       public:
 	 static QStringList tagExpressions();
 	 static QStringList argExpressions();
    };
+   
 }
 
 }
