@@ -194,6 +194,12 @@ ConversionStatus GettextImportPlugin::load(const QString& filename, const QStrin
       emit signalClearProgressBar();
 
       kdDebug(KBABEL) << "ready." << endl;
+      
+      if(counter==0)
+      {
+        // no items read
+	return PARSE_ERROR;
+      }
 
       if(success!=PARSE_ERROR)
       {
