@@ -13,6 +13,17 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
+
  ***************************************************************************/
 
 #include "hidingmsgedit.h"
@@ -177,6 +188,7 @@ void HidingMsgEdit::setNumberOfPlurals( uint numberOfPlurals )
       connect( e, SIGNAL( signalUndoCmd( KBabel::EditCommand* )), 
         this, SIGNAL( signalUndoCmd( KBabel::EditCommand* )));
       connect( e, SIGNAL( textChanged() ) , this, SIGNAL( textChanged() ));
+      connect( e, SIGNAL( textChanged() ) , this, SLOT( emitTextChanged() ));
       connect( e, SIGNAL( cursorPositionChanged( int, int )), 
         this, SLOT( emitCursorPositionChanged( int, int )) );
   }
@@ -186,6 +198,11 @@ void HidingMsgEdit::setNumberOfPlurals( uint numberOfPlurals )
   _currentEdit->setReadOnly (readonly);
 }
 
+void HidingMsgEdit::emitTextChanged()
+{
+    emit textChanged(_currentEdit->text());
+}
+
 uint HidingMsgEdit::currentForm()
 {
     if( _currentEdit == _singleEdit ) return 0;
