@@ -3,7 +3,7 @@
 
   Copyright (C) 1999-2000 by Matthias Kiefer
                             <matthias.kiefer@gmx.de>
-		2002-2003 by Stanislav Visnovsky
+		2002-2004 by Stanislav Visnovsky
 			    <visnovsky@kde.org>
 
   This program is free software; you can redistribute it and/or modify
@@ -20,12 +20,23 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
+
 **************************************************************************** */
 
 #include <kdatatool.h>
 #include <kdebug.h>
 #include <kfiledialog.h>
-#include <klineeditdlg.h>
+#include <kinputdialog.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <knotifyclient.h>
@@ -44,10 +55,10 @@
 #include "tagextractor.h"
 #include "kbabelview.h"
 #include "kbabeldictbox.h"
-#include "kbcharselect.h"
 #include "mymultilineedit.h"
 #include "hidingmsgedit.h"
 #include "roughtransdlg.h"
+#include "kbabelsettings.h"
 
 #include "resources.h"
 
@@ -78,8 +89,11 @@ void KBabelView::aboutDictionary(const QString id)
 
 void KBabelView::informDictionary()
 {
-    if( msgstrEdit->isModified() || commentEdit->isModified() )
-	dictBox->setTextChanged(_catalog->msgid(_currentIndex)
+    if( isSearching () )
+	stopSearch ();
+
+    if( msgstrEdit->isModified() )
+	dictBox->setTextChanged(_catalog->msgid(_currentIndex,true)
 	    ,*(_catalog->msgstr(_currentIndex).at(msgstrEdit->currentForm()))
 	    ,msgstrEdit->currentForm()
 	    ,_catalog->comment(_currentIndex));
@@ -90,15 +104,15 @@ void KBabelView::setNewLanguage()
     IdentitySettings s = _catalog->identitySettings();
 
     dictBox->setLanguage(s.languageCode, s.languageName);
-    
+
     // setup new plural form number
     int form = msgstrEdit->currentForm();
     if( form >= s.numberOfPluralForms )
 	form = s.numberOfPluralForms-1;
 
     msgstrEdit->setNumberOfPlurals (s.numberOfPluralForms);
-    setSettings( _settings );
-    
+    updateSettings();
+
     if (! _catalog->currentURL().isEmpty())
     {
 	updateEditor( form );
@@ -251,7 +265,7 @@ void KBabelView::selectTag()
 	if( msgid[i] == '\n' ) diff++;
     }
     msgidLabel->selectTag(_tagExtractor->matchIndex(_currentTag),_tags[_currentTag].length()+diff);
-    taglistView->setCurrentItem(_currentTag);
+    emit signalNextTag (_currentTag);
 }
 
 void KBabelView::setTagsMenu(QPopupMenu *menu)
@@ -313,20 +327,8 @@ void KBabelView::insertNextTagMsgid()
 
     QString t;
 
-    if( taglistView->isVisible() )
+    if( _catalog->pluralForm( _currentIndex ) == KDESpecific )
     {
-	if(taglistView->currentItem() == -1)
-	{
-    	    KNotifyClient::beep();
-    	    return;
-	}
-
-	t=taglistView->currentText();
-    }
-    else
-    {
-	if( _catalog->pluralForm( _currentIndex ) == KDESpecific )
-	{
 	    int pos = msgstrEdit->currentIndex();
             int currentFormBegin=s.findRev("\\n",pos);
 	    if( currentFormBegin == -1 ) currentFormBegin=0;
@@ -335,26 +337,18 @@ void KBabelView::insertNextTagMsgid()
 	    if( currentFormEnd == -1 ) currentFormEnd=s.length();
 
 	    s=s.mid(currentFormBegin,currentFormEnd-currentFormBegin);
-	}
-
-	extractor.setString(s);
-	uint num= extractor.countMatches();
-	if(num >= _tags.count())
-	{
-	    KNotifyClient::beep();
-	    return;
-	}
-
-	t=_tags[num];
     }
 
-    if( taglistView->isVisible() && taglistView->currentItem() != -1 )
+    extractor.setString(s);
+    uint num= extractor.countMatches();
+    if(num >= _tags.count())
     {
-        QListBoxItem* item= taglistView->item(taglistView->currentItem())->next();
-	if( item ) taglistView->setCurrentItem(item);
-	else taglistView->clearSelection();
+	KNotifyClient::beep();
+	return;
     }
 
+    t=_tags[num];
+
     modifyMsgstrText( offset, t );
 }
 
@@ -518,9 +512,9 @@ void KBabelView::diffInternal(bool autoDf)
 
     if(r == Catalog::DiffNeedList)
     {
-	switch( _settings.diffSource )
+	switch( KBabelSettings::useDBForDiff() )
 	{
-	    case EditorSettings::Database:
+	    case 1:
     	    {
         	_loadingDiffFile=true;
     		bool wasEnabled=_diffEnabled;
@@ -556,7 +550,7 @@ void KBabelView::diffInternal(bool autoDf)
 	        _loadingDiffFile=false;
 		break;
     	    }
-	    case EditorSettings::File:
+	    case 0:
     	    {
         	_diffing=false;
                 if(!openDiffFile(true))
@@ -571,7 +565,7 @@ void KBabelView::diffInternal(bool autoDf)
     	        _diffing = true;
 		break;
             }
-	    case EditorSettings::Msgstr:
+	    case 2:
 	    {
 		// get the list of all entries
         	QValueList<DiffEntry> diffList = _catalog->asDiffList();
@@ -662,14 +656,12 @@ bool KBabelView::openDiffFile(bool autoDiff)
 
     KURL url;
 
-    if( autoDiff && !_settings.diffBaseDir.isEmpty() )
+    if( autoDiff && ! KBabelSettings::diffBaseDir().isEmpty() )
     {
         KURL fileURL = _catalog->currentURL();
-
-	KConfig* config = KGlobal::config();
-	config->setGroup("CatalogManager");
-
-	KURL poBaseURL(config->readPathEntry("PoBaseDir"));
+	
+	KURL poBaseURL( _project->catManSettings().poBaseDir );
+	
         QString poBase = poBaseURL.path();
         int len = poBase.length();
         if(fileURL.path().left(len) == poBase)
@@ -678,15 +670,15 @@ bool KBabelView::openDiffFile(bool autoDiff)
            if(fileRelPath[0] == '/')
                fileRelPath=fileRelPath.mid(1);
 
-           if(_settings.diffBaseDir.right(1) != "/")
-               _settings.diffBaseDir += '/';
+           if(KBabelSettings::diffBaseDir().right(1) != "/")
+               KBabelSettings::diffBaseDir() += '/';
 
-           QString diffFilePath = _settings.diffBaseDir + fileRelPath;
+           QString diffFilePath = KBabelSettings::diffBaseDir() + fileRelPath;
 
 
            KURL diffFileURL(diffFilePath);
 
-           if(diffFileURL.isValid() && KIO::NetAccess::exists(diffFileURL))
+           if(diffFileURL.isValid() && KIO::NetAccess::exists(diffFileURL,true,NULL))
            {
                url = diffFileURL;
 
@@ -699,7 +691,7 @@ bool KBabelView::openDiffFile(bool autoDiff)
 
     if(url.isEmpty())
     {
-        url = KFileDialog::getOpenURL(_settings.diffBaseDir, i18n("*.po *.pot|Gettext files")
+        url = KFileDialog::getOpenURL(KBabelSettings::diffBaseDir(), i18n("*.po *.pot|Gettext Files")
                ,this, i18n("Select File to Diff With"));
     }
 
@@ -735,7 +727,7 @@ bool KBabelView::openDiffFile(bool autoDiff)
             case NO_PERMISSIONS:
             {
                 KMessageBox::sorry(this,i18n(
-                "You don't have permissions to read file:\n %1")
+                "You do not have permissions to read file:\n %1")
                         .arg(url.prettyURL()));
                 break;
             }
@@ -829,7 +821,7 @@ void KBabelView::showTryLaterMessageBox()
 void KBabelView::setFilePackage()
 {
     bool result=false;
-    QString p = KLineEditDlg::getText(i18n("Enter new package for the current file:"),_catalog->package(),&result,this);
+    QString p = KInputDialog::getText(QString::null, i18n("Enter new package for the current file:"),_catalog->package(),&result,this);
     if( result )
     {
 	_catalog->setPackage(p);
@@ -837,16 +829,16 @@ void KBabelView::setFilePackage()
     }
 }
 
-void KBabelView::insertTagFromTool( QListBoxItem* item )
+void KBabelView::insertTagFromTool( const QString& tag )
 {
-    modifyMsgstrText(msgstrEdit->currentIndex(),item->text());
+    modifyMsgstrText(msgstrEdit->currentIndex(),tag);
 
     msgstrEdit->setFocus();
 }
 
-void KBabelView::skipToTagFromTool( )
+void KBabelView::skipToTagFromTool( int index )
 {
-    _currentTag = taglistView->currentItem();
+    _currentTag = index;
     selectTag();
 }
 
@@ -871,7 +863,7 @@ void KBabelView::plural2msgstr()
 	    _catalog->applyBeginCommand( _currentIndex, Msgstr ,this);
 
 	    i=0;
-	    for( QStringList::Iterator it=msgs.begin() ; it!=msgs.end() ; it++ )
+	    for( QStringList::Iterator it=msgs.begin() ; it!=msgs.end() ; ++it )
 	    {
 		if( i!= msgstrEdit->currentForm() )
 		{
@@ -891,7 +883,7 @@ void KBabelView::plural2msgstr()
 		}
 		i++;
 	    }
-
+	    
 	    // fill the non-initialized ones
 	    while (i != numForms)
 	    {
@@ -1018,7 +1010,15 @@ void KBabelView::insertChar( QChar ch )
     msgstrEdit->setFocus();
 }
 
-void KBabelView::insertSelectedChar()
+void KBabelView::wordCount()
 {
-    charSelector->emitChar();
+    uint total, untranslated, fuzzy;
+    
+    _catalog->wordCount( total, fuzzy, untranslated );
+    
+    KMessageBox::information( this
+	, i18n("Total words: %1\n\n"
+"Words in untranslated messages: %2\n\n"
+"Words in fuzzy messages: %3").arg(total).arg(untranslated).arg(fuzzy)
+	, i18n("Word Count") );
 }
