@@ -238,8 +238,7 @@ bool KBabelHighlighter::isMisspelled(const QString& word)
     // there is no 'spelt correctly' signal so default to Okay
     kdDebug(KBABEL) << "Adding word " << word << endl;
     dict.replace(word, Okay);
-    // yes I tried checkWord, the docs lie and it didn't give useful signals :-(
-    mSpell->check(word, false);
+    mSpell->checkWord(word, false);
     return false;
 }
 
