@@ -20,6 +20,17 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
+
 **************************************************************************** */
 
 
@@ -29,6 +40,7 @@
 #include <qmemarray.h>
 #include <qobject.h>
 #include <qstringlist.h>
+#include <qguardedptr.h>
 
 class KSpell;
 class QColor;
@@ -74,7 +86,7 @@ class KBabelHighlighter : public QObject
     QString accelMarker;
 
     static QDict<int> dict;
-    KSpell *mSpell;
+    QGuardedPtr<KSpell> mSpell;
     QString currentWord;
     int currentPos;
     bool alwaysEndsWithSpace;
