@@ -75,7 +75,7 @@ public:
 	int ignoreAccelMarker, int ignoreContextInfo, int ask, int askForNextFile, int askForSave);
     virtual void spellcheck(QStringList fileList);
 private:
-    KBabelMW* findInstance( QCString file, QCString project, QCString package);
+    KBabelMW* findInstance( QCString file, QString project, QCString package);
 };
 
 
@@ -215,7 +215,7 @@ KBabelInterface::KBabelInterface()
 
 void KBabelInterface::openURL(QCString url, QCString package, WId window, int newWindow)
 {
-    openURL(url,package,window,newWindow, KBABEL_DEFAULTPROJECT);
+    openURL(url,package,window,newWindow, KBABEL_DEFAULTPROJECT.local8Bit());
 }
 
 void KBabelInterface::openURL(QCString url, QCString package, WId window, int newWindow, QCString projectFile)
@@ -305,7 +305,7 @@ void KBabelInterface::openURL(QCString url, QCString package, WId window, int ne
 
 void KBabelInterface::openTemplate(QCString openFilename, QCString saveFilename, QCString package, int newWindow)
 {
-    openTemplate(openFilename, saveFilename, package, newWindow, KBABEL_DEFAULTPROJECT);
+    openTemplate(openFilename, saveFilename, package, newWindow, KBABEL_DEFAULTPROJECT.local8Bit());
 }
 
 void KBabelInterface::openTemplate(QCString openFilename, QCString saveFilename, QCString package, int newWindow, QCString projectFile)
@@ -368,7 +368,7 @@ void KBabelInterface::gotoFileEntry(QCString url, QCString m)
 
 void KBabelInterface::gotoFileEntry(QCString url, QCString package, int m)
 {
-    gotoFileEntry(url, package, m, KBABEL_DEFAULTPROJECT );
+    gotoFileEntry(url, package, m, KBABEL_DEFAULTPROJECT.local8Bit() );
 }
 
 void KBabelInterface::gotoFileEntry(QCString url, QCString package, int m, QCString projectFile)
@@ -454,7 +454,7 @@ void KBabelInterface::spellcheck(QStringList fileList)
     kb->spellcheckMoreFiles( fileList );
 }
 
-KBabelMW* KBabelInterface::findInstance( QCString url, QCString project, QCString package)
+KBabelMW* KBabelInterface::findInstance( QCString url, QString project, QCString package)
 {
     kdDebug () << "Suspending DCOP" << endl;
     kapp->dcopClient()->suspend();
