@@ -3,10 +3,11 @@
 
   Copyright (C) 1999-2000 by Matthias Kiefer
                             <matthias.kiefer@gmx.de>
-		2001-2003 by Stanislav Visnovsky
+		2001-2004 by Stanislav Visnovsky
 			    <visnovsky@kde.org>
-			    
+
   Alt+123 feature idea taken from KOffice by David Faure <david@mandrakesoft.com>.
+  Word wrap support by Jarno Elonen <elonen@iki.fi>, 2003
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,7 +21,18 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
 
 **************************************************************************** */
 
@@ -42,17 +54,22 @@
 
 #include <kdebug.h>
 #include <kglobal.h>
+#include <kglobalsettings.h>
 #include <kmessagebox.h>
 #include <kstdaccel.h>
 
-MyMultiLineEdit::MyMultiLineEdit(QWidget* parent,const char* name)
-                :QTextEdit(parent,name), emitUndo(true), _firstChangedLine(0), 
+#include "kbhighlighting.h"
+
+using namespace KBabel;
+
+MyMultiLineEdit::MyMultiLineEdit(int ID, QWidget* parent,const char* name)
+                :KTextEdit(parent,name), emitUndo(true), _firstChangedLine(0), 
 		_lastChangedLine(0), _lastParagraph(0), 
-		_lastParagraphOffset(0), _dontUpdate(false), _menu(0),
-		_overwrite(false)
+		_lastParagraphOffset(0), _dontUpdate(false), _myID (ID),
+		 _menu(0), _overwrite(false)
 {
    setUndoRedoEnabled(false); // we handle this ourselves
-   setWordWrap( NoWrap );
+   setWordWrap( WidgetWidth );
    viewport()->setAcceptDrops( false ); // we need our parent to get drops
 }
 
@@ -102,7 +119,7 @@ void MyMultiLineEdit::processCommand(EditCommand* cmd, bool undo)
       if(delcmd->str.find("\n")>0 )_lastChangedLine=row+delcmd->str.contains("\n");
       else _lastChangedLine=row;
       
-      QTextEdit::insert( delcmd->str ); 
+      KTextEdit::insert( delcmd->str ); 
 
       offset2Pos( delcmd->offset+delcmd->str.length(), row, col );
       setCursorPosition( row, col);
@@ -117,7 +134,7 @@ void MyMultiLineEdit::processCommand(EditCommand* cmd, bool undo)
       
       setSelection( row, col, rowEnd, colEnd, 0 );
       _firstChangedLine=_lastChangedLine=row;
-      QTextEdit::removeSelectedText();
+      KTextEdit::removeSelectedText();
    }
 
 
@@ -151,8 +168,8 @@ void MyMultiLineEdit::emitCursorPosition()
     int line=0;
     int col=0;
     getCursorPosition(&line,&col);
-
-    emit cursorPositionChanged(line,col);
+    
+    emit cursorPositionChanged(line, col);
 }
 
 void MyMultiLineEdit::wheelEvent(QWheelEvent *e)
@@ -162,7 +179,7 @@ void MyMultiLineEdit::wheelEvent(QWheelEvent *e)
 
 void MyMultiLineEdit::focusInEvent(QFocusEvent *e)
 {
-    QTextEdit::focusInEvent(e);
+    KTextEdit::focusInEvent(e);
     emitCursorPosition();
 }
 
@@ -203,7 +220,7 @@ void MyMultiLineEdit::doKeyboardAction( KeyboardAction action )
 
       case ActionReturn:
 	if( emitUndo) 
-	    emit signalUndoCmd( new InsTextCmd(currentIndex(), "\n") );
+	    emit signalUndoCmd( new InsTextCmd(currentIndex(), "\n", _myID) );
 	break;
   
       case ActionKill:
@@ -216,16 +233,16 @@ void MyMultiLineEdit::doKeyboardAction( KeyboardAction action )
 	    if( y < (int)s.length()-1  ) // not the end of paragraph
 	    {
 		QString delText = s.mid( y, s.length()-y-1);
-	        emit signalUndoCmd( new DelTextCmd(currentIndex(), delText ) );
+	        emit signalUndoCmd( new DelTextCmd(currentIndex(), delText, _myID ) );
 	    } else 
 		if( x < paragraphs()-1 ) // not the end of text
-		    emit signalUndoCmd( new DelTextCmd(currentIndex(), "\n" ) );
+		    emit signalUndoCmd( new DelTextCmd(currentIndex(), "\n", _myID ) );
 	}
         break;
-      default: break;
+     default: break;
   }
 
-  QTextEdit::doKeyboardAction( action );
+  KTextEdit::doKeyboardAction( action );
 
   emitCursorPosition();
 }
@@ -237,7 +254,7 @@ void MyMultiLineEdit::setText(const QString& s)
     // workaround, since insert does not interpret markup
     setTextFormat( Qt::PlainText );
     _firstChangedLine=_lastChangedLine=0;
-    QTextEdit::setText(s);
+    KTextEdit::setText(s);
     setTextFormat( Qt::AutoText );
     // now the number of lines is known, let's do highlight
     _lastChangedLine=paragraphs();
@@ -248,7 +265,7 @@ void MyMultiLineEdit::setText(const QString& s)
 void MyMultiLineEdit::insertAt( const QString & s, int line, int col, bool mark )
 {
    // it will invoke insert, don't need to send InsTextCmd
-   QTextEdit::insertAt(s,line,col);
+   KTextEdit::insertAt(s,line,col);
    
    // code from QMultiLineEdit
    if( mark )  
@@ -269,7 +286,7 @@ void MyMultiLineEdit::insert( const QString & text, bool indent, bool checkNewLi
 	int endRow,endCol;
 	getSelection(&row,&col,&endRow,&endCol);
 
-	if( row<(int)_lastParagraph )
+	if( row < (int)_lastParagraph )
 	{
 	    _lastParagraph=0;
 	    _lastParagraphOffset=0;
@@ -286,15 +303,15 @@ void MyMultiLineEdit::insert( const QString & text, bool indent, bool checkNewLi
 
    if( emitUndo)
    {
-	emit signalUndoCmd( new BeginCommand());
+	emit signalUndoCmd( new BeginCommand(-1,UndefPart));
 	// reimplemented overwrite
 	if( _overwrite && noSelectionRemoved) 
 	{
 	    doKeyboardAction( ActionDelete );
 	}
 	
-	emit signalUndoCmd( new InsTextCmd(currentIndex(), text) );
-	emit signalUndoCmd( new EndCommand());
+	emit signalUndoCmd( new InsTextCmd(currentIndex(), text, _myID) );
+	emit signalUndoCmd( new EndCommand(-1,UndefPart));
    }
    
    int n=text.find("\n");
@@ -310,7 +327,7 @@ void MyMultiLineEdit::insert( const QString & text, bool indent, bool checkNewLi
    if( hasFocus() ) _invisibleHighlight.setActive( newcg );
    else _invisibleHighlight.setInactive( newcg );
    setPalette( _invisibleHighlight );   
-   QTextEdit::insert(text, indent, checkNewLine, removeSelected);
+   KTextEdit::insert(text, indent, checkNewLine, removeSelected);
    setPalette( _visibleHighlight );   
  
    setUpdatesEnabled(true);
@@ -322,14 +339,12 @@ void MyMultiLineEdit::removeLine ( int line )
 {
    kdDebug(KBABEL) << "removeLine invoked" << endl;
 
-   QTextEdit::removeParagraph(line);
+   KTextEdit::removeParagraph(line);
    emitCursorPosition();
 }
 
 void MyMultiLineEdit::clear()
 {
-   kdDebug(KBABEL) << "clear invoked" << endl;
-   
     _lastParagraph=0;
     _lastParagraphOffset=0;
     
@@ -337,12 +352,12 @@ void MyMultiLineEdit::clear()
 
    QString s = text();
    if( !s.isEmpty() && emitUndo ) {
-	emit signalUndoCmd( new BeginCommand() );
-	emit signalUndoCmd( new DelTextCmd(0,s) );
-	emit signalUndoCmd( new EndCommand() );
+	emit signalUndoCmd( new BeginCommand(-1,UndefPart) );
+	emit signalUndoCmd( new DelTextCmd(0,s,_myID) );
+	emit signalUndoCmd( new EndCommand(-1,UndefPart) );
    }
    
-   QTextEdit::clear();
+   KTextEdit::clear();
    
    _dontUpdate=false;
    
@@ -371,11 +386,11 @@ void MyMultiLineEdit::my_backspace()
          if(cursorX != 0)
          {
             QString delTxt(s[cursorX-1]);
-            emit signalUndoCmd(new DelTextCmd(offset-1,delTxt));
+            emit signalUndoCmd(new DelTextCmd(offset-1,delTxt,_myID));
          }
          else if( cursorY > 0 || cursorX > 0 ) // not at the beginning
          {
-            emit signalUndoCmd(new DelTextCmd(offset-1,"\n"));
+            emit signalUndoCmd(new DelTextCmd(offset-1,"\n",_myID));
          }
       }
    }
@@ -401,11 +416,11 @@ void MyMultiLineEdit::my_del()
          if(cursorX != (int)s.length()-1)
          {
             QString delTxt(s[cursorX]);
-            emit signalUndoCmd(new DelTextCmd(offset,delTxt));
+            emit signalUndoCmd(new DelTextCmd(offset,delTxt,_myID));
          }
          else if( cursorY < (int)paragraphs()-1 || ( (cursorY == (int)paragraphs()-1) && (cursorX < (int)text( paragraphs()-1 ).length()-1 ) ) )// !atEnd() )
          {
-            emit signalUndoCmd(new DelTextCmd(offset,"\n"));
+            emit signalUndoCmd(new DelTextCmd(offset,"\n",_myID));
          }
       }
    }
@@ -418,12 +433,12 @@ void MyMultiLineEdit::removeSelectedText(int selNum)
 	_lastParagraph=0;
 	_lastParagraphOffset=0;
 	
-	QTextEdit::removeSelectedText(selNum);
+	KTextEdit::removeSelectedText(selNum);
     }
     else 
     {
 	  int paraFrom, idxFrom, paraTo, idxTo;
-	  QTextEdit::getSelection( &paraFrom, &idxFrom, &paraTo, &idxTo );
+	  KTextEdit::getSelection( &paraFrom, &idxFrom, &paraTo, &idxTo );
 	  
 	  if( paraFrom < (int)_lastParagraph )
 	  {
@@ -432,8 +447,8 @@ void MyMultiLineEdit::removeSelectedText(int selNum)
 	  }
 
 	  int offset = pos2Offset( paraFrom, idxFrom );
-	  emit signalUndoCmd(new DelTextCmd( offset, selectedText() ) );
-	  QTextEdit::removeSelectedText(selNum);
+	  emit signalUndoCmd(new DelTextCmd( offset, selectedText(), _myID ) );
+	  KTextEdit::removeSelectedText(selNum);
     }
 
     emitCursorPosition();
@@ -441,7 +456,7 @@ void MyMultiLineEdit::removeSelectedText(int selNum)
 
 void MyMultiLineEdit::paste()
 {
-    QTextEdit::paste();    
+    KTextEdit::paste();    
     emitCursorPosition();
 }
 
@@ -450,7 +465,7 @@ int MyMultiLineEdit::currentIndex()
     int para; // paragraph of current position
     int index; // index in the current paragraph
     
-    QTextEdit::getCursorPosition(&para,&index);
+    KTextEdit::getCursorPosition(&para,&index);
     
     return pos2Offset( para, index );
 }
@@ -491,7 +506,8 @@ void MyMultiLineEdit::offset2Pos(int offset, int &paragraph, int &index) const
 int MyMultiLineEdit::pos2Offset(uint paragraph, uint index)
 {
     paragraph = QMAX( QMIN( (int)paragraph, paragraphs() - 1), 0 ); // Sanity check
-	index  = QMAX( QMIN( (int)index,  paragraphLength( paragraph )), 0 ); // Sanity check
+    index  = QMAX( QMIN( (int)index,  paragraphLength( paragraph )), 0 ); // Sanity check
+
     {
         uint lastI;
 	lastI  = paragraphLength( paragraph );
@@ -535,16 +551,17 @@ void MyMultiLineEdit::setOverwriteMode( bool b )
 }
 
 /*******************************************************************************/
-MsgMultiLineEdit::MsgMultiLineEdit(QWidget* parent,const char* name)
-                :MyMultiLineEdit(parent,name),
+MsgMultiLineEdit::MsgMultiLineEdit(int ID, KSpell* spell, QWidget* parent,const char* name)
+                :MyMultiLineEdit(ID, parent,name),
                 _quotes(false),
                 _cleverEditing(false),
                 _highlightBg(false),
                 _spacePoints(false),
                 _bgColor(colorGroup().base().dark(110)),
-		_textColor(Qt::black),
+		_textColor(KGlobalSettings::textColor()),
 		_errorColor(Qt::red),
-		_currentColor(Qt::black),
+		_currentColor(KGlobalSettings::textColor()),
+		_whitespace(0),
                 _hlSyntax(true),
                 _quoteColor(Qt::darkGreen),
                 _unquoteColor(Qt::red),
@@ -555,30 +572,45 @@ MsgMultiLineEdit::MsgMultiLineEdit(QWidget* parent,const char* name)
                 _diffStrikeOutDel(true),
                 _diffAddColor(Qt::darkGreen),
                 _diffDelColor(Qt::darkRed),
-		_currentUnicodeNumber(0)
+		_currentUnicodeNumber(0),
+		highlighter(0),
+		_tagStartPara(0), _tagStartIndex(0), _tagEndPara(0), _tagEndIndex(0)
 {
+   diffPos.setAutoDelete(true);
+   diffPos.clear();
+   
    _whitespace = new QPixmap(2,2,-1,QPixmap::BestOptim);
    _whitespace->fill(_textColor);
    _errorWhitespace = new QPixmap(2,2,-1,QPixmap::BestOptim);
    _errorWhitespace->fill(_errorColor);
-
+   
    _whitespaceNB = new QPixmap(3,3,-1,QPixmap::BestOptim);
    _whitespaceNB->fill();
    _errorWhitespaceNB = new QPixmap(3,3,-1,QPixmap::BestOptim);
    _errorWhitespaceNB->fill();
-
+   
    QPainter p(_whitespaceNB);
    p.setPen( _textColor );
    p.drawEllipse(_whitespaceNB->rect());
-
+   
    QPainter q(_errorWhitespaceNB);
    q.setPen( _errorColor );
    q.drawEllipse(_errorWhitespaceNB->rect());
-					  
-   diffPos.setAutoDelete(true);
-   diffPos.clear();
+
+   // this will setup bitBlt pixmaps
+   setFont( font() );
+   highlighter = new KBabelHighlighter( this, spell );
+   connect( this, SIGNAL( signalSyntaxHighlightingChanged( bool ) ), highlighter, SLOT( setSyntaxHighlighting( bool ) ) );
+   
    connect( this, SIGNAL( selectionChanged() ), this, SLOT( paintSpacePoints() ) );   
    connect( this, SIGNAL( cursorPositionChanged( int, int ) ), this, SLOT( paintSpacePoints(int, int) ) );
+   connect( this, SIGNAL( textChanged() ), this, SLOT( emittedTextChanged() ) );
+}
+
+MsgMultiLineEdit::~MsgMultiLineEdit ()
+{
+    if(highlighter)
+	delete highlighter;
 }
 
 void MsgMultiLineEdit::setText(const QString& s)
@@ -779,6 +811,8 @@ void MsgMultiLineEdit::setHighlightSyntax(bool on)
 {
    _hlSyntax=on;
    
+   emit signalSyntaxHighlightingChanged (on);
+   
    update();
 }
 
@@ -791,18 +825,25 @@ void MsgMultiLineEdit::setHighlightColors(const QColor& quoteColor, const QColor
    _accelColor=accelColor;
    _tagColor=tagColor;
 
+   highlighter->setHighlightColor( KBabelHighlighter::Tag, tagColor );
+   highlighter->setHighlightColor( KBabelHighlighter::Entity, accelColor );
+   highlighter->setHighlightColor( KBabelHighlighter::CFormat, cformatColor );
+   highlighter->setHighlightColor( KBabelHighlighter::Masked, quoteColor );
+   
    update();
 }
 
 
 void MsgMultiLineEdit::setFont(const QFont& font)
 {
-   QTextEdit::setFont(font);
-
+   KTextEdit::setFont(font);
+   
+   // we don't need to calculate a special offset for non-breaking space, since
+   // they are very similar in size
    QFontMetrics fm(font);
    _wsOffsetX = QMAX(fm.width(' ')/2-2,1);
    _wsOffsetY = QMAX(fm.height()/2-1,0);
-
+   
    repaint();
 }
 
@@ -834,27 +875,44 @@ void MsgMultiLineEdit::setTextColor(const QColor &color )
   else p.setInactive( newcg );
   setPalette( p );
   _textColor = color;
+  highlighter->setHighlightColor( KBabelHighlighter::Normal, color );
 }
 
 void MsgMultiLineEdit::setErrorColor(const QColor &color )
 {
   _errorColor = color;
+  highlighter->setHighlightColor( KBabelHighlighter::Error, color );
 }
 
 void MsgMultiLineEdit::setCurrentColor(const TextColor color)
 {
-  if( color == NormalColor ) _currentColor = _textColor;
-  else _currentColor = _errorColor;
+  if( color == NormalColor ) {
+    _currentColor = _textColor;
+    highlighter->setHasErrors( false );
+  } else {
+    _currentColor = _errorColor;
+    highlighter->setHasErrors( true );
+  }
   
+  /*
   setUpdatesEnabled(false);
+  // need to block signals (especially textChanged() to avoid recursion with KBabelView::autoCheck
+  blockSignals(true);
   selectAll();
   setColor( _currentColor );
   removeSelection();
   setColor(_currentColor);
+  blockSignals(false);
   setUpdatesEnabled(true);
+  */
   forceUpdate();
 }
 
+void MsgMultiLineEdit::setSpellChecker(KSpell* spell)
+{
+    highlighter->setSpellChecker(spell);
+}
+
 void MsgMultiLineEdit::paintSpacePoints(int, int )
 {
     paintSpacePoints();
@@ -863,162 +921,182 @@ void MsgMultiLineEdit::paintSpacePoints(int, int )
 void MsgMultiLineEdit::paintSpacePoints()
 {
     QRect r;
-    QPoint p;
     QPainter painter(viewport() );
+    const QFontMetrics& fm = fontMetrics();
 
-#if (QT_VERSION < 305)
-    QTextParag* par = document()->firstParag();
-#else
-    QTextParagraph* par = document()->firstParagraph();
-#endif	
-    int row = 0;
-    int rowY = 2;
+    int paranum = paragraphAt(QPoint(contentsX(), contentsY()));    
 
-    while( par && rowY < contentsY())
+    if( _spacePoints )
     {
-	row++;
-	rowY+=par->lineHeight(0);
-	par=par->next();
-    }
-
-    if( _spacePoints ) {
-	painter.setPen( _currentColor );
-
-#if (QT_VERSION < 305)
-	QTextParag* mypar = par;
-#else
-	QTextParagraph* mypar = par;
-#endif
-	int myrow = row;
-	int myrowY = rowY;
-	
+        int curpara = paranum;
+        painter.setPen( _currentColor );
         QPixmap* ws, *wsnb;
-	
-	if( _currentColor== _errorColor )
-	{
-	    ws = _errorWhitespace;
-	    wsnb = _errorWhitespaceNB;
-	}
-	else
-	{
-	    ws = _whitespace;
-	    wsnb = _whitespaceNB;
-	}
-													
-        while( mypar )
-	{
-	    QString s = text(myrow);
-	    int i = s.find( " " );
-	    while( (i >= 0) && (i < (int)s.length()-1) ) // -1 because text will end by EOLN
-	    {
-		QTextStringChar* ch=mypar->at(i);
-		p = contentsToViewport( QPoint(ch->x,myrowY));
-		if( ch->c.unicode() == 0x00A0U )
-		    bitBlt(viewport(), QPoint(p.x()+_wsOffsetX, p.y()+_wsOffsetY), wsnb, wsnb->rect(), Qt::CopyROP);
-		else
-		    bitBlt(viewport(), QPoint(p.x()+_wsOffsetX, p.y()+_wsOffsetY), ws, wsnb->rect(), Qt::CopyROP);
-		i = s.find( " ", i+1 );
-    	    }
-	    myrowY+=mypar->lineHeight(0);
-	    if( myrowY > contentsY()+visibleHeight() ) break;
 
-	    mypar=mypar->next();
-	    myrow++;
+        if( _currentColor== _errorColor )
+        {
+            ws = _errorWhitespace;
+            wsnb = _errorWhitespaceNB;
+        }
+        else
+        {
+            ws = _whitespace;
+            wsnb = _whitespaceNB;
+        }
+
+        while( curpara < paragraphs())
+        {
+            if ( paragraphRect( curpara ).top() > contentsY()+visibleHeight()) break;
+
+            const QString& s = text(curpara);
+            int i = s.find( " " );
+            while( (i >= 0) && (i < (int)s.length()-1) ) // -1 because text will end by EOLN
+            {
+                QPixmap* pm = ( s.at(i).unicode() == 0x00A0U ) ? wsnb : ws;
+                QRect r = mapToView( curpara, i );
+                r.moveBy( r.width()/2, (r.height() - fm.descent())/2 );
+                r.moveBy( -pm->rect().width()/2, -pm->rect().height()/2-1 );
+                bitBlt(viewport(), r.topLeft(), pm, pm->rect(), Qt::CopyROP);
+                i = s.find( " ", i+1 );
+            }
+            ++curpara;
         }
     }
-    
-    if( _quotes ) {
 
-	QFontMetrics fm( font() );
-	QRect qs = fm.boundingRect("\"");
-        
-	for( int myrow = row; myrow < paragraphs() ; myrow++ )
-	{
-	    r = paragraphRect(myrow);
-	    if( r.y() > contentsY()+visibleHeight() ) break;
-	    
-	    QPoint pos( r.x()+r.width()-4, r.y()+r.height()-qs.height() );
-	    pos = contentsToViewport(pos);
-	    painter.drawText( pos, "\""); // 4 is Qt source
-	    pos = QPoint( 0, r.y()+r.height()-qs.height() );
-	    pos = contentsToViewport(pos);
-	    painter.drawText( pos, "\""); // 4 is Qt source
+    if( _quotes )
+    {
+        QFontMetrics fm( font());
+        QRect qs = fm.boundingRect("\"");
+
+        for( int curpara = paranum; curpara < paragraphs() ; curpara++ )
+        {
+            r = paragraphRect(curpara);
+            if( r.y() > contentsY()+visibleHeight() ) break;
+
+            painter.drawText( QPoint( 0, mapToView( curpara, 0 ).top()) +
+                    QPoint(0, qs.height() + 4), "\""); // 4 = hardcoded margin in QT
+            painter.drawText( mapToView( curpara, QMAX( 0,
+                ((int)text( curpara ).length())-1)).topRight() +
+                QPoint(0, qs.height() + 4), "\"");  // 4 = hardcoded margin in QT
         }
     }
-    
+
     if( _showDiff && (!_diffUnderlineAdd || !_diffStrikeOutDel) )
     {
-	if( paragraphs() == (int)diffPos.size() ) // sanity check
-	{
-	painter.setRasterOp( Qt::AndROP );
-	for( int myrow = row; myrow < paragraphs() ; myrow++ ) {
-	    r = paragraphRect(myrow);
-	    if( r.y() > contentsY()+visibleHeight() ) break;
+        if( paragraphs() == (int)diffPos.size() ) // sanity check
+        {
+            painter.setRasterOp( Qt::AndROP );
+            for( int curpara = paranum; curpara < paragraphs() ; curpara++ )
+            {
+                r = paragraphRect(curpara);
+                if( r.y() > contentsY()+visibleHeight() ) break;
 
-    	    QValueList<DiffInfo> *list = diffPos[myrow];
-    	    if(list)
-    	    {
-        	QValueList<DiffInfo>::ConstIterator it;
-        	for(it = list->begin(); it != list->end(); ++it)
-        	{
-            	    QRect beg =  mapToView( myrow, (*it).begin );
-		    QRect end =  mapToView( myrow, (*it).end );
-//		    if( beg && end ) 
-		    {
-			QRect diff  =beg.unite( end);
-		    
-            		if( (*it).add  && !_diffUnderlineAdd)
-            		{
-                	    painter.fillRect( diff, _diffAddColor);
-            		}
-            		else if(!(*it).add && !_diffStrikeOutDel)
-            		{
-                	    painter.fillRect( diff, _diffDelColor );
-            		}
-		    }
-        	}
-    	    }
-	}
-	}
+                QValueList<DiffInfo> *list = diffPos[curpara];
+                if(list)
+                {
+                    QValueList<DiffInfo>::ConstIterator it;
+                    for(it = list->begin(); it != list->end(); ++it)
+                    {
+                        QRect beg =  mapToView( curpara, (*it).begin );
+                        QRect end =  mapToView( curpara, (*it).end );
+
+                        QColor* c = 0;
+                        if( (*it).add  && !_diffUnderlineAdd)
+                            c = &_diffAddColor;
+                        else if(!(*it).add && !_diffStrikeOutDel)
+                            c = &_diffDelColor;
+
+                        if ( c != 0 )
+                        {
+                            // Single or multiple lines?
+                            if ( beg.top() == end.top())
+                            {
+                                painter.fillRect( QRect( beg.topLeft(),
+                                    QPoint( end.right(), end.bottom())), *c );
+                            }
+                            else
+                            {
+                                painter.fillRect( QRect(
+                                    beg.topLeft(),
+                                    QPoint( r.right(), beg.bottom())), *c );
+				if( end.top()-beg.bottom() > 2 ) {
+				    // there is a line, not only thin space
+                            	    painter.fillRect( QRect(
+                                	QPoint( r.left(), beg.bottom()),
+                                	QPoint( r.right(), end.top())), *c );
+				}
+                                painter.fillRect( QRect(
+                                    QPoint( r.left(), end.top()),
+                                    QPoint( end.right(), end.bottom())), *c );
+                            }
+                        }
+                    }
+                }
+            }
+        }
     }
 
     if( _showDiff && (_diffUnderlineAdd || _diffStrikeOutDel) )
     {
-	if( paragraphs() == (int)diffPos.size() ) // sanity check
-	{
-	for( int myrow = row ; myrow < paragraphs() ; myrow++ ) {
-	    r = paragraphRect(myrow);
-	    if( r.y() > contentsY()+visibleHeight() ) break;
+        if( paragraphs() == (int)diffPos.size() ) // sanity check
+        {
+            for( int curpara = paranum; curpara < paragraphs() ; curpara++ )
+            {
+                r = paragraphRect(curpara);
+                if( r.y() > contentsY()+visibleHeight() ) break;
 
-    	    QValueList<DiffInfo> *list = diffPos[myrow];
-    	    if(list)
-    	    {
-        	QPen addPen(_diffAddColor,2);
-        	QPen delPen(_diffDelColor,2);
-        	QValueList<DiffInfo>::ConstIterator it;
-        	for(it = list->begin(); it != list->end(); ++it)
-        	{
-		    QRect beg = mapToView( myrow, (*it).begin );
-		    QRect end = mapToView( myrow, (*it).end );
-//		    if( beg && end )
-		    {
-			QRect diff  = beg.unite( end);
+                QValueList<DiffInfo> *list = diffPos[curpara];
+                if(list)
+                {
+                    QPen addPen(_diffAddColor,2);
+                    QPen delPen(_diffDelColor,2);
+                    QValueList<DiffInfo>::ConstIterator it;
+                    for(it = list->begin(); it != list->end(); ++it)
+                    {
+                        QRect beg = mapToView( curpara, (*it).begin );
+                        QRect end = mapToView( curpara, (*it).end );
 
-                	if( (*it).add && _diffUnderlineAdd)
-            		{
-                	    painter.setPen(addPen);
-                	    painter.drawLine(diff.x(),diff.y()+diff.height()-1,diff.x()+diff.width(),diff.y()+diff.height()-1);
-            	        }
-            		else if(!(*it).add && _diffStrikeOutDel)
-            		{
-                	    painter.setPen(delPen);
-                	    painter.drawLine(diff.x(),diff.y()+diff.height()/2,diff.x()+diff.width(),diff.y()+diff.height()/2);
-            		}
-		    }
-        	}
-    	    }
-	}
-	}
+                        QPen* p = 0;
+                        int dy = 0;
+                        if( (*it).add && _diffUnderlineAdd)
+                            p = &addPen;
+                        else if(!(*it).add && _diffStrikeOutDel)
+                        {
+                            p = &delPen;
+                            dy = fm.ascent()/2-1;
+                        }
+
+                        if ( p != 0 )
+                        {
+                            painter.setPen( *p );
+
+                            // Single or multiple lines?
+                            if ( beg.top() == end.top())
+                                painter.drawLine(
+                                    beg.topLeft() + QPoint(0, fm.ascent()-dy),
+                                    end.topRight()+ QPoint(0, fm.ascent()-dy));
+                            else
+                            {
+                                int y = beg.top() + fm.ascent();
+                                painter.drawLine(
+                                    QPoint(beg.left(), y),
+                                    QPoint(r.right(), y));
+                                y += fm.lineSpacing();
+                                while (y < end.top() + fm.ascent())
+                                {
+                                    painter.drawLine(
+                                        QPoint(r.left(), y),
+                                        QPoint(r.right(), y));
+                                    y += fm.lineSpacing();
+                                }
+                                painter.drawLine(
+                                    QPoint(r.left(), end.top() + fm.ascent()),
+                                    QPoint(end.right(), end.top() + fm.ascent()));
+                            }
+                        }
+                    }
+                }
+            }
+        }
     }
 }
 
@@ -1032,7 +1110,7 @@ void MsgMultiLineEdit::forceUpdate()
 {
     _firstChangedLine=0;
     _lastChangedLine=paragraphs()-1;
-    highlight();
+    highlighter->highlight();
     MyMultiLineEdit::repaint();
 }
 
@@ -1044,7 +1122,7 @@ void MsgMultiLineEdit::ensureCursorVisible()
 
 void MsgMultiLineEdit::highlight()
 {
-    if( _dontUpdate ) return;
+/*    if( _dontUpdate ) return;
     
     QColor bg;
     if( _highlightBg ) bg = _bgColor;
@@ -1189,6 +1267,7 @@ void MsgMultiLineEdit::highlight()
 	updateContents();
     }
     ensureCursorVisible();
+    */
 }
 
 void MsgMultiLineEdit::drawContents( QPainter *painter, int clipx, int clipy, int clipw, int cliph )
@@ -1203,37 +1282,112 @@ void MsgMultiLineEdit::paintEvent( QPaintEvent *event )
     paintSpacePoints();
 }
 
-/**
- * WARNING:
- * This function contains some undocumented magic inspired by Qt source code.
- * It can be quite non-portable to new versions of Qt (even minor revisions)
- */
- 
 QRect MsgMultiLineEdit::mapToView( int para, int index )
 {
-    if( para < 0 || para > paragraphs() || index < 0 || index > paragraphLength(para) ) return QRect(); //invalid rectangle
+    if( para < 0 || para > paragraphs() ||
+        index < 0 || index > paragraphLength(para) )
+            return QRect(); //invalid rectangle
 
-    QPainter painter(viewport() );
-    QTextDocument * doc = document();
-#if (QT_VERSION < 305)
-    QTextParag * par = doc->firstParag();
-#else
-    QTextParagraph * par = doc->firstParagraph();
-#endif
-    int rowY = 2; // found out by experiment
+    const QFontMetrics& fm = fontMetrics();
+    const QString& paratext = text(para);
 
-    for( int row = 0; row < para ; row++ ) 
+    // Find index of the first character on the same line as parameter
+    // 'index' using binary search. Very fast, even for long texts.
+    int linestart = 0;
+    int indexline = lineOfChar( para, index );
+    if ( indexline > 0 )
     {
-	rowY += par->lineHeight(0); // we have no wrap
-        par = par->next();
+        int min = 0, max = index;
+        int i = (min + max)/2;
+        int iline = lineOfChar( para, i );
+        while ( iline != indexline-1 ||
+                lineOfChar( para, i+1 ) != indexline )
+        {
+            Q_ASSERT( min != max && min != i && max != i );
+            if ( iline < indexline )
+                min = i;
+            else
+                max = i;
+            i = (min + max)/2;
+            iline = lineOfChar( para, i );
+        }
+        linestart = i+1;
     }
+    Q_ASSERT( linestart >= 0 );
+    
+    int linewidth;
+    
+    // if the tag is not valid, easy
+    if( (_tagStartPara == _tagEndPara) && (_tagStartIndex == _tagEndIndex) ) {
+	linewidth = fm.width( paratext.mid( linestart, index-linestart ));
+    } else {
+	int tso = pos2Offset( _tagStartPara, _tagStartIndex );
+	int teo = pos2Offset( _tagEndPara, _tagEndIndex );
+	int off = pos2Offset( para, index );
+    
+	if( off < tso ) {
+	    // it is surely before the tag
+	    linewidth = fm.width( paratext.mid( linestart, index-linestart ));
+	} else if( off >= teo ) {
+	    // it is surely after the tag
 	
-    QTextStringChar *chr = par->at(index);
+	    // is it on the same line as the end of the tag?
+	    if( _tagEndPara < para || lineOfChar( _tagEndPara, _tagEndIndex ) < indexline ) {
+		// no tag on the line, no bold
+		linewidth = fm.width( paratext.mid( linestart, index-linestart ));
+	    } else {
+		QFont f( font() );
+		f.setBold( true );
+		QFontMetrics bfm( f );
+		// is tag single displayed line?
+		if( _tagStartPara == _tagEndPara 
+		    && lineOfChar( _tagStartPara, _tagStartIndex ) == lineOfChar( _tagEndPara, _tagEndIndex ) ) 
+		{
+		    // yes, count the non-bold before the tag start
+		    linewidth = fm.width( paratext.mid( linestart, _tagStartIndex-linestart ) )
+			+ bfm.width( paratext.mid( _tagStartIndex, _tagEndIndex-_tagStartIndex ) );
+		} 
+		else
+		{
+		    // count the part of the tag itself
+		    linewidth = bfm.width( paratext.mid( linestart, _tagEndIndex-linestart ) );
+		}
+	    
+		// add the rest from tag to the index
+		linewidth += fm.width( paratext.mid( _tagEndIndex, index-_tagEndIndex ) );
+	    }
+	}
+	else {
+	    // in tag
+	    QFont f( font() );
+	    f.setBold( true );
+	    QFontMetrics bfm( f );
+	    // is it the first line of the tag?
+	    if( para == _tagStartPara && indexline == lineOfChar( _tagStartPara, _tagStartIndex ) ) {
+		// start of the line is normal
+		linewidth = fm.width( paratext.mid( linestart, _tagStartIndex-linestart ) )
+		    + bfm.width( paratext.mid( _tagStartIndex, index-_tagStartIndex ) );
+	    } else {
+		// whole is bold
+		linewidth = bfm.width( paratext.mid( linestart, index-linestart ) );
+	    }
+	}
+    }
     
-    QPoint p( chr->x, rowY );
-    p = contentsToViewport(p);
+    // FIXME as soon as it's possible to ask real margins from QTextEdit:
+    const int left_margin = 4;
+    // const int top_margin = 4;
     
-    return QRect( p.x(), p.y(), par->string()->width(index), par->lineHeight(0) );
+    QPainter painter( viewport());
+    const QRect& linerect = paragraphRect(para);
+    return QRect(
+        contentsToViewport( QPoint(
+            left_margin + linerect.left() + linewidth , 
+            /*top_margin + */linerect.top() + indexline * fm.lineSpacing() + fm.leading())),
+        QSize(
+            fm.charWidth( paratext, index ),
+            fm.lineSpacing()
+        ));
 }
 
 void MsgMultiLineEdit::keyPressEvent(QKeyEvent *e)
@@ -1248,7 +1402,7 @@ void MsgMultiLineEdit::keyPressEvent(QKeyEvent *e)
     
     if(e->key() == Key_Return || e->key() == Key_Enter)
     { 
-        emit signalUndoCmd(new BeginCommand());
+        emit signalUndoCmd(new BeginCommand(-1,UndefPart));
 
         int row, col;
         getCursorPosition(&row,&col);
@@ -1301,7 +1455,7 @@ void MsgMultiLineEdit::keyPressEvent(QKeyEvent *e)
 	    e->accept();
         }
 
-        emit signalUndoCmd(new EndCommand());
+        emit signalUndoCmd(new EndCommand(-1,UndefPart));
         return;
     }
     else if(e->key() == Key_Tab)
@@ -1311,27 +1465,10 @@ void MsgMultiLineEdit::keyPressEvent(QKeyEvent *e)
         e->accept();
         return;
     }
-    else if ( KStdAccel::deleteWordBack().contains( key ) ) {
-      moveCursor(QTextEdit::MoveWordBackward, true);
-      if ( hasSelectedText() )
-        del();
-      
-      e->accept();
-      return;
-    }
-    else if ( KStdAccel::deleteWordForward().contains( key ) ) {
-      // Workaround for QT bug where
-      moveCursor(QTextEdit::MoveWordForward ,true);
-      if ( hasSelectedText() )
-        del();
-      
-      e->accept();
-      return;
-    }
-    else if(e->key() == Key_Delete 
+    else if((e->key() == Key_Delete && !(e->state() & ControlButton))
             || ((e->state() & ControlButton) && e->key() == Key_D) )
     {
-        emit signalUndoCmd(new BeginCommand());
+        emit signalUndoCmd(new BeginCommand(-1,UndefPart));
         
         if(!hasSelectedText())
         {
@@ -1353,7 +1490,7 @@ void MsgMultiLineEdit::keyPressEvent(QKeyEvent *e)
         
         del();
 
-        emit signalUndoCmd(new EndCommand());
+        emit signalUndoCmd(new EndCommand(-1,UndefPart));
         emit textChanged();
         e->accept();
         return;
@@ -1361,7 +1498,7 @@ void MsgMultiLineEdit::keyPressEvent(QKeyEvent *e)
     else if(e->key() == Key_BackSpace
             || ((e->state() & ControlButton) && e->key() == Key_H) )
     {
-        emit signalUndoCmd(new BeginCommand()); 
+        emit signalUndoCmd(new BeginCommand(-1,UndefPart)); 
         
         if(!hasSelectedText())
         {
@@ -1382,14 +1519,14 @@ void MsgMultiLineEdit::keyPressEvent(QKeyEvent *e)
         
         MyMultiLineEdit::keyPressEvent(e);
 
-        emit signalUndoCmd(new EndCommand());
+        emit signalUndoCmd(new EndCommand(-1,UndefPart));
 
         e->accept();
         return;
     }
     else if(e->text() == "\"")   
     {
-        emit signalUndoCmd(new BeginCommand());
+        emit signalUndoCmd(new BeginCommand(-1,UndefPart));
 
         int row, col;
         getCursorPosition(&row,&col);
@@ -1406,7 +1543,7 @@ void MsgMultiLineEdit::keyPressEvent(QKeyEvent *e)
 
         e->accept();
 
-        emit signalUndoCmd(new EndCommand());
+        emit signalUndoCmd(new EndCommand(-1,UndefPart));
         return;
     }
     else if(e->key() == Key_Space && ( e->state() & AltButton ) )
@@ -1467,15 +1604,24 @@ bool MsgMultiLineEdit::isMasked(QString *str, uint col)
     return !(bool)(counter%2);
 }
 
-void MsgMultiLineEdit::emitCursorPosition()
-{
-    MyMultiLineEdit::emitCursorPosition();
-}
-
 void MsgMultiLineEdit::emittedTextChanged()
 {
     highlight();
     paintSpacePoints();    
 }
 
+void MsgMultiLineEdit::selectTag(int start, int length)
+{
+    setUpdatesEnabled(false);
+    setSelection( _tagStartPara, _tagStartIndex, _tagEndPara, _tagEndIndex);
+    setBold( false );
+
+    offset2Pos(start, _tagStartPara, _tagStartIndex);
+    offset2Pos(start+length, _tagEndPara, _tagEndIndex);
+    
+    setSelection( _tagStartPara, _tagStartIndex, _tagEndPara, _tagEndIndex);
+    setBold( true );
+    setUpdatesEnabled(true);
+}
+
 #include "mymultilineedit.moc"
