@@ -20,6 +20,17 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
+
 **************************************************************************** */
 #ifndef MYMULTILINEEDIT_H
 #define MYMULTILINEEDIT_H
@@ -126,6 +137,7 @@ protected slots:
 
 signals:
    void signalUndoCmd(KBabel::EditCommand*);
+   void signalSyntaxHighlightingChanged (bool enable);
 
 protected:
    int _myID;
@@ -143,6 +155,7 @@ public:
    enum TextColor { NormalColor, ErrorColor };
 
    MsgMultiLineEdit(int ID, KSpell* spell=0, QWidget* parent=0,const char* name=0);
+   virtual ~MsgMultiLineEdit();
 
    /** is displaying surrounding quotes enabled? */
    bool quotes() const { return _quotes;}
