@@ -3,6 +3,8 @@
 
   Copyright (C) 1999-2001 by Matthias Kiefer
                             <matthias.kiefer@gmx.de>
+		2003	  by Stanislav Visnovsky
+			    <visnovsky@kde.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -16,8 +18,19 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
+	    
 **************************************************************************** */
 #ifndef DICTCHOOSER_H
 #define DICTCHOOSER_H
@@ -28,6 +41,7 @@
 #include <kbabeldictbox.h>
 
 class KListBox;
+class KTempFile;
 
 class QPushButton;
 class QListBoxItem;
@@ -38,10 +52,15 @@ class DictChooser : public QWidget
     Q_OBJECT
 
 public:
-    DictChooser(QPtrList<ModuleInfo>& list, QStringList selected, 
+    DictChooser(KBabelDictBox* box, QStringList selected, 
             QWidget *parent , const char *name=0);
 
     QStringList selectedDicts();
+    
+    /** If the user configured some of the modules, this 
+     *  restores the original configuration of all these modules
+     */
+    void restoreConfig();
 
     virtual bool eventFilter(QObject *, QEvent *);
     
@@ -54,8 +73,10 @@ private slots:
     void selectedMarked(int);
     void unselectedMarked(int);
 
-
+    void configureSelected();
+    
 private:
+    KBabelDictBox* box;
     QPtrList<ModuleInfo> dictList;
 
     KListBox *selectedBox;
@@ -65,6 +86,9 @@ private:
     QPushButton *unselectBtn;
     QPushButton *upBtn;
     QPushButton *downBtn;
+    QPushButton *configureBtn;
+    
+    QDict<KTempFile> tempConfig;
 };
 
 #endif // DICTCHOOSER_H
