@@ -16,8 +16,19 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
+		  
 **************************************************************************** */
 #include <klocale.h>
 #include <kapplication.h>
@@ -34,14 +45,16 @@
 #include "preferenceswidget.h"
 #include "pa_factory.h"
 
+#include <resources.h>
 #include <catalog.h>
 
+using namespace KBabel;
+
 PoAuxiliary::PoAuxiliary(QObject *parent, const char *name)
         : SearchEngine(parent, name)
 {
     catalog = new Catalog(this, "PoAuxiliary::catalog");
     prefWidget=0;
-    conf=0;
     error=false;
     stop=false;
     active=false;
@@ -104,14 +117,11 @@ void PoAuxiliary::readSettings(KConfigBase *config)
     
     if(needLoading && !loadTimer->isActive())
     {
-        kdDebug() << "readSettings" << endl;
+        kdDebug(KBABEL_SEARCH) << "readSettings" << endl;
         loadTimer->start(100,true);
     }
 
     restoreSettings();
-
-    conf=config;
-    confGroup=config->group();
 }
 
 PrefWidget *PoAuxiliary::preferencesWidget(QWidget *parent)
@@ -147,8 +157,9 @@ QString PoAuxiliary::lastError()
     return errorMsg;
 }
 
-bool PoAuxiliary::startSearch(QString text)
+bool PoAuxiliary::startSearch(const QString& t, uint pluralForm, const SearchFilter*filter)
 {
+    QString text(t);
     if(autoUpdate && prefWidget && prefWidget->settingsChanged())
     {
         applySettings();
@@ -173,7 +184,7 @@ bool PoAuxiliary::startSearch(QString text)
    
     kapp->processEvents(100);
 
-    text.replace(QRegExp("\\n"),"");
+    text.replace("\n","");
     
     Entry *entry = msgidDict[text];
     if(entry)
@@ -220,7 +231,7 @@ bool PoAuxiliary::startSearch(QString text)
 }
 
 
-bool PoAuxiliary::startSearchInTranslation(QString text)
+bool PoAuxiliary::startSearchInTranslation(const QString& text)
 {
     if(autoUpdate && prefWidget && prefWidget->settingsChanged())
     {
@@ -319,12 +330,6 @@ void PoAuxiliary::applySettings()
     
     ignoreFuzzy = prefWidget->ignoreFuzzy();
 
-    if(conf)
-    {
-        KConfigGroupSaver gs(conf,confGroup);
-        saveSettings(conf);
-    }
-    
     if(needLoading && !loadTimer->isActive())
     {
         loadTimer->start(100,true);
@@ -355,13 +360,13 @@ void PoAuxiliary::loadAuxiliary()
    
     if(path.contains("@LANG@"))
     {
-        path.replace(QRegExp("@LANG@"),langCode);
+        path.replace("@LANG@",langCode);
     }
     if(path.contains("@PACKAGE@"))
     {
 	int pos=package.findRev("/");
 	if( pos<0 ) pos=0;
-        path.replace(QRegExp("@PACKAGE@"),package.mid(pos));
+        path.replace("@PACKAGE@",package.mid(pos));
     }
     if(path.contains("@PACKAGEDIR@"))
     {
@@ -369,8 +374,8 @@ void PoAuxiliary::loadAuxiliary()
 	int pos=package.findRev("/");
 	if( pos > 0 ) packagedir=package.left(pos);
 	else packagedir="";
-        path.replace(QRegExp("@PACKAGEDIR@"),packagedir);
-	kdDebug() << "Packagedir found " << packagedir << endl;
+        path.replace("@PACKAGEDIR@",packagedir);
+	kdDebug(KBABEL_SEARCH) << "Packagedir found " << packagedir << endl;
     }
     QRegExp reg("@DIR[0-9]+@");
     if(path.contains(reg))
@@ -388,7 +393,7 @@ void PoAuxiliary::loadAuxiliary()
             {
                 QString dir=directory(editedFile,number);
                 QString s("@DIR%1@");
-                path.replace(QRegExp(s.arg(number)),dir);
+                path.replace(s.arg(number),dir);
 
                 pos+=dir.length();
             }
@@ -408,25 +413,24 @@ void PoAuxiliary::loadAuxiliary()
     {
         KURL temp(editedFile);
         QString dir = temp.directory();
-        kdDebug() << dir << endl;
+        kdDebug(KBABEL_SEARCH) << dir << endl;
         u.setPath(dir+"/"+path);
         u.cleanPath();
-        kdDebug() << u.prettyURL() << endl;
+        kdDebug(KBABEL_SEARCH) << u.prettyURL() << endl;
     }
     else
     {
         u.setPath(path);
     }
-    bool e;
         
     emit progressStarts(i18n("Loading PO auxiliary"));
     connect(catalog, SIGNAL(signalProgress(int))
                         , this, SIGNAL(progress(int)));
       
-    Catalog::IOStatus stat = catalog->openURL(u,e);
-    if( stat != Catalog::OK && stat != Catalog::RECOVERED_PARSE_ERROR)
+    ConversionStatus stat = catalog->openURL(u);
+    if( stat != OK && stat != RECOVERED_PARSE_ERROR)
     {
-        kdDebug() << "error while opening file " << u.prettyURL() << endl;
+        kdDebug(KBABEL_SEARCH) << "error while opening file " << u.prettyURL() << endl;
 	
 	if( !error )
 	{
@@ -455,20 +459,21 @@ void PoAuxiliary::loadAuxiliary()
             }
 
             Entry *e = new Entry;
-            e->orig = catalog->msgid(i);
-	    e->orig.replace(QRegExp("\\n"),"");
-            e->translation = catalog->msgstr(i);
+	    // FIXME: should care about plural forms
+            e->orig = catalog->msgid(i).first();
+	    e->orig.replace("\n","");
+	    kdWarning() << "PoAuxialiary does not support plural forms" << endl;
+            e->translation = catalog->msgstr(i).first();
             e->comment = catalog->comment(i);
             e->fuzzy = catalog->isFuzzy(i);
 
-            msgidDict.insert(catalog->msgid(i,true),e);
+            msgidDict.insert(catalog->msgid(i,true).first(),e);
             msgstrDict.insert(e->translation,e);
         }
 
         auxPackage = catalog->packageName();
         auxURL = catalog->currentURL().url();
-        PoInfo catalogInfo = Catalog::headerInfo(catalog->header());
-        auxTranslator = catalogInfo.lastTranslator;
+        auxTranslator = catalog->lastTranslator();
     }
 
     disconnect(catalog, SIGNAL(signalProgress(int))
@@ -484,7 +489,7 @@ void PoAuxiliary::loadAuxiliary()
 }
 
 
-void PoAuxiliary::setEditedFile(QString file)
+void PoAuxiliary::setEditedFile(const QString& file)
 {
     if(initialized && (url.contains("@DIR") || KURL::isRelativeURL(url)) 
             && file!=editedFile && !loadTimer->isActive() )
@@ -496,7 +501,7 @@ void PoAuxiliary::setEditedFile(QString file)
 }
 
 
-void PoAuxiliary::setEditedPackage(QString pack)
+void PoAuxiliary::setEditedPackage(const QString& pack)
 {
     if(initialized && url.contains("@PACKAGE@") && pack!=package 
             && !loadTimer->isActive() )
@@ -508,7 +513,7 @@ void PoAuxiliary::setEditedPackage(QString pack)
 }
 
 
-void PoAuxiliary::setLanguageCode(QString lang)
+void PoAuxiliary::setLanguageCode(const QString& lang)
 {
     if(initialized && url.contains("@LANG@") && lang!=langCode 
             && !loadTimer->isActive() )
@@ -524,7 +529,7 @@ bool PoAuxiliary::usesRichTextResults()
     return true;
 }
 
-QString PoAuxiliary::translate(const QString text)
+QString PoAuxiliary::translate(const QString& text, uint pluralForm)
 {
     if(!initialized)
     {
