@@ -16,13 +16,26 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
 
 **************************************************************************** */
 
 
 #include "searchengine.h"
 
+#include <kdebug.h>
+
 SearchResult::SearchResult()
 {
     score=0;
@@ -70,16 +83,7 @@ SearchEngine::~SearchEngine()
 {
 }
 
-bool SearchEngine::messagesForPackage(const QString&
-                         , QValueList<Message>& resultList, QString& error)
-{
-    resultList.clear();
-    error=QString::null;
-    
-    return true;
-}
-
-bool SearchEngine::startSearchInTranslation(QString)
+bool SearchEngine::startSearchInTranslation(const QString&, unsigned int, const SearchFilter*)
 {
     return true;
 }
@@ -171,26 +175,26 @@ void SearchEngine::clearResults()
 
 
 
-void SearchEngine::setEditedFile(QString)
+void SearchEngine::setEditedFile(const QString&)
 {
 }
 
 
-void SearchEngine::setEditedPackage(QString)
+void SearchEngine::setEditedPackage(const QString&)
 {
 }
 
-void SearchEngine::setLanguageCode(QString)
+void SearchEngine::setLanguageCode(const QString&)
 {
 }
 
-void SearchEngine::setLanguage(QString,QString)
+void SearchEngine::setLanguage(const QString&,const QString&)
 {
 }
 
 
 
-void SearchEngine::stringChanged(QString,QString,QString)
+void SearchEngine::stringChanged(const QStringList&,const QString&,unsigned int, const QString&)
 {
 }
 
@@ -205,7 +209,7 @@ QString SearchEngine::directory(const QString& path, int n)
 
 	if(begin < 0)
 	{
-		qDebug("not found");
+		kdDebug() << "not found" << endl;
 		if(n==0)
 			return path;
 		else
@@ -252,7 +256,7 @@ uint SearchEngine::score(const QString& orig, const QString& found)
     return score;
 }
 
-uint SearchEngine::ngramMatch (const QString text1, const QString text2, 
+uint SearchEngine::ngramMatch (const QString& text1, const QString& text2, 
 			       uint ngram_len)
 {
   if (text1.length() < ngram_len || text2.length() < ngram_len)
@@ -270,4 +274,10 @@ uint SearchEngine::ngramMatch (const QString text1, const QString text2,
 }
 
 
+bool SearchFilter::match( const TranslationInfo& toCheck )
+{
+    // FIXME: not implemented yet
+    return true;
+}
+
 #include "searchengine.moc"
