@@ -4,8 +4,8 @@
 #include <qstring.h>
 #include <qstringlist.h>
 
-#include <bug.h>
-#include <package.h>
+#include "bug.h"
+#include "package.h"
 
 class KConfig;
 
@@ -17,23 +17,23 @@ class BugCommand {
     BugCommand( const Bug &bug ) : m_bug( bug ) {}
     BugCommand( const Bug &bug, const Package &pkg ) : m_bug( bug ), m_package( pkg ) {}
     virtual ~BugCommand() {}
-    
+
     virtual QString controlString() const { return QString::null; }
-    
+
     virtual QString mailAddress() const { return QString::null; }
     virtual QString mailText() const { return QString::null; }
-    
+
     Bug bug() const { return m_bug; }
     Package package() const { return m_package; }
-    
+
     virtual QString name();
     virtual QString details();
 
     virtual QString type() const { return QString::null; }
-    
+
     virtual void save( KConfig * ) = 0;
     static BugCommand *load( KConfig *, const QString &type );
-    
+
   protected:
     Bug m_bug;
     Package m_package;
@@ -43,18 +43,18 @@ class BugCommandClose : public BugCommand {
   public:
     BugCommandClose( const Bug &bug, const QString &message, const Package &pkg )  :
         BugCommand( bug, pkg ), m_message( message ) {}
-    
+
     QString controlString() const;
     QString mailAddress() const;
     QString mailText() const;
 
     QString name();
-    QString details();
+    QString details() const;
 
     QString type() const { return QString::fromLatin1("Close"); }
 
     void save( KConfig * );
-    
+
   private:
     QString m_message;
 };
@@ -63,44 +63,44 @@ class BugCommandCloseSilently : public BugCommand {
   public:
     BugCommandCloseSilently( const Bug &bug, const Package &pkg )  :
         BugCommand( bug, pkg ) {}
-    
+
     QString controlString() const;
 
     QString name();
-	
+
     QString type() const { return QString::fromLatin1("CloseSilently"); }
 
-    void save( KConfig * );    
+    void save( KConfig * );
 };
 
 class BugCommandReopen : public BugCommand {
   public:
     BugCommandReopen( const Bug &bug, const Package &pkg )  :
         BugCommand( bug, pkg ) {}
-    
+
     QString controlString() const;
 
     QString name();
-	
+
     QString type() const { return QString::fromLatin1("Reopen"); }
 
-    void save( KConfig * );    
+    void save( KConfig * );
 };
 
 class BugCommandRetitle : public BugCommand {
   public:
     BugCommandRetitle( const Bug &bug, const QString &title, const Package &pkg )  :
         BugCommand( bug, pkg ), m_title( title ) {}
-    
+
     QString controlString() const;
-    
+
     QString name();
-    QString details();
-	
+    QString details() const;
+
     QString type() const { return QString::fromLatin1("Retitle"); }
 
     void save( KConfig * );
-    
+
   private:
     QString m_title;
 };
@@ -109,16 +109,16 @@ class BugCommandMerge : public BugCommand {
   public:
     BugCommandMerge( const QStringList &bugNumbers, const Package &pkg )  :
         BugCommand( Bug(), pkg ), m_bugNumbers( bugNumbers ) {}
-    
+
     QString controlString() const;
-    
+
     QString name();
-    QString details();
-	
+    QString details() const;
+
     QString type() const { return QString::fromLatin1("Merge"); }
 
     void save( KConfig * );
-    
+
   private:
     QStringList m_bugNumbers;
 };
@@ -146,7 +146,7 @@ class BugCommandReply : public BugCommand {
     QString mailText() const;
 
     QString name();
-    QString details();
+    QString details() const;
 
     QString type() const { return QString::fromLatin1("Reply"); }
 
@@ -167,12 +167,12 @@ class BugCommandReplyPrivate : public BugCommand {
     QString mailText() const;
 
     QString name();
-    QString details();
+    QString details() const;
 
     QString type() const { return QString::fromLatin1("ReplyPrivate"); }
 
     void save( KConfig * );
-    
+
   private:
     QString m_address;
     QString m_message;
@@ -182,16 +182,16 @@ class BugCommandSeverity : public BugCommand {
   public:
     BugCommandSeverity( const Bug &bug, const QString &severity, const Package &pkg ) :
         BugCommand( bug, pkg ), m_severity( severity ) {}
-  
+
     QString name();
-    QString details();
-	
+    QString details() const;
+
     QString type() const { return QString::fromLatin1("Severity"); }
 
     QString controlString() const;
-  
+
     void save( KConfig * );
-    
+
   private:
     QString m_severity;
 };
@@ -202,14 +202,14 @@ class BugCommandReassign : public BugCommand {
         BugCommand( bug, pkg ), m_package( package ) {}
 
     QString name();
-    QString details();
-	
+    QString details() const;
+
     QString type() const { return QString::fromLatin1("Reassign"); }
 
     QString controlString() const;
-    
+
     void save( KConfig * );
-    
+
   private:
     QString m_package;
 };
