@@ -275,7 +275,7 @@ void BugServer::sendCommands( MailSender *mailer, const QString &senderName,
                 mCache->invalidateBugList( pkg, QString::null ); // the status of the bug comes from the buglist...
 
                 QStringList::ConstIterator it2;
-                for (it2 = pkg.components().begin();it2 != pkg.components().end();it2++) {
+                for (it2 = pkg.components().begin();it2 != pkg.components().end();++it2) {
                     mCache->invalidateBugList( pkg, (*it2) ); // the status of the bug comes from the buglist...
                 }
             }
