@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -31,7 +31,7 @@ class RdfProcessor : public DomProcessor
     RdfProcessor( BugServer * );
     virtual ~RdfProcessor();
 
-    Bug::List parseBugList( const QDomElement & );
+    KBB::Error parseDomBugList( const QDomElement &, Bug::List & );
 
     void setBugListQuery( KURL &, const Package &, const QString &component );
 };
