@@ -146,7 +146,7 @@ void KBBMainWindow::initActions()
     // Prepare and create XML GUI
     fileQuit          = KStdAction::quit( this,
                         SLOT( close() ), actionCollection() );
-    fileQuit->setStatusText( i18n( "Quit KBugBuster" ) );
+    fileQuit->setToolTip( i18n( "Quit KBugBuster" ) );
 
     new KAction( i18n("See &Pending Changes"), "contents", 0, this, SLOT( slotListChanges() ),
                  actionCollection(), "file_seechanges" );
