@@ -132,6 +132,13 @@ QString CallMapView::whatsThis() const
   return s;
 }
 
+void CallMapView::setData(TraceData* d)
+{
+  TraceItemView::setData(d);
+
+  ((CallMapBaseItem*)base())->setFunction(0);
+}
+
 void CallMapView::context(TreeMapItem* i,const QPoint & p)
 {
   if (!i) return;
@@ -170,9 +177,7 @@ void CallMapView::context(TreeMapItem* i,const QPoint & p)
     popup.insertSeparator();
   }
 
-  popup.insertItem(i18n("Go Back"), 90);
-  popup.insertItem(i18n("Go Forward"), 91);
-  popup.insertItem(i18n("Go Up"), 92);
+  addGoMenu(&popup);
   popup.insertSeparator();
 
   l1popup.setCheckable(true);
@@ -341,10 +346,6 @@ void CallMapView::context(TreeMapItem* i,const QPoint & p)
   }
 
   switch(r) {
-  case 90: activated(Back); break;
-  case 91: activated(Forward); break;
-  case 92: activated(Up); break;
-
   case 20:
     setFieldVisible(0, !vpopup.isItemChecked(20));
     break;
@@ -469,6 +470,8 @@ TraceItem* CallMapView::canShow(TraceItem* i)
 
 void CallMapView::doUpdate(int changeType)
 {
+    if (changeType == costType2Changed) return;
+
     // if there is a selected item, always draw marking...
     if (changeType & selectedItemChanged) {
 	TraceFunction* f = 0;
@@ -567,7 +570,7 @@ TraceCost* CallMapView::totalCost()
   TraceFunction* f = ((CallMapBaseItem*)base())->function();
   if (!f) return 0;
 
-  return Configuration::showExpanded() ? f->cumulative() : f->data();
+  return Configuration::showExpanded() ? f->inclusive() : f->data();
 }
 
 
@@ -612,25 +615,23 @@ QString CallMapBaseItem::text(int textNo) const
       if (total == 0.0)
 	  sum = 100.0;
       else
-	  sum = 100.0 * _f->cumulative()->subCost(ct) / total;
+	  sum = 100.0 * _f->inclusive()->subCost(ct) / total;
 
       return QString("%1 %")
 	  .arg(sum, 0, 'f', Configuration::percentPrecision());
   }
-  return _f->cumulative()->prettySubCost(ct);
+  return _f->inclusive()->prettySubCost(ct);
 }
 
 QPixmap CallMapBaseItem::pixmap(int i) const
 {
     if ((i != 1) || !_f) return QPixmap();
 
-    // Cost pixmap
-
     TraceCostType* ct = ((CallMapView*)widget())->costType();
     TraceCost* t      = ((CallMapView*)widget())->totalCost();
 
-    return costPixmap( ct, _f->cumulative(),
-		       (double) (t->subCost(ct)) );
+    // colored level meter with frame
+    return costPixmap( ct, _f->inclusive(), (double) (t->subCost(ct)), true);
 }
 
 
@@ -640,7 +641,7 @@ double CallMapBaseItem::value() const
 
   TraceCostType* ct;
   ct = ((CallMapView*)widget())->costType();
-  return (double) _f->cumulative()->subCost(ct);
+  return (double) _f->inclusive()->subCost(ct);
 }
 
 
@@ -653,7 +654,7 @@ double CallMapBaseItem::sum() const
   if (w->showCallers())
     return 0.0;
   else
-    return (double) _f->cumulative()->subCost(w->costType());
+    return (double) _f->inclusive()->subCost(w->costType());
 }
 
 
@@ -668,7 +669,7 @@ TreeMapItemList* CallMapBaseItem::children()
     CallMapView* w = (CallMapView*)widget();
 
     if (0) qDebug("Create Function %s (%s)",
-		  w->showCallers() ? "Callers":"Callings",
+		  w->showCallers() ? "Callers":"Callees",
 		  text(0).ascii());
 
     TraceCall* call;
@@ -700,7 +701,7 @@ TreeMapItemList* CallMapBaseItem::children()
         addItem(i);
       }
 
-      setSum(_f->cumulative()->subCost(w->costType()));
+      setSum(_f->inclusive()->subCost(w->costType()));
     }
     setSorting(-2, false);
   }
@@ -765,11 +766,11 @@ QPixmap CallMapCallingItem::pixmap(int i) const
     if (i != 1) return QPixmap();
 
     // Cost pixmap
-
     TraceCostType* ct = ((CallMapView*)widget())->costType();
     TraceCost* t      = ((CallMapView*)widget())->totalCost();
 
-    return costPixmap( ct, _c, t->subCost(ct) / _factor );
+    // colored level meter with frame
+    return costPixmap( ct, _c, t->subCost(ct) / _factor, true);
 }
 
 
@@ -800,7 +801,7 @@ TreeMapItemList* CallMapCallingItem::children()
     ct = ((CallMapView*)widget())->costType();
 
     // same as sum()
-    SubCost s = _c->called()->cumulative()->subCost(ct);
+    SubCost s = _c->called()->inclusive()->subCost(ct);
     SubCost v = _c->subCost(ct);
     if (v>s) {
       qDebug("Warning: CallingItem subVal %u > Sum %u (%s)",
@@ -882,11 +883,11 @@ QPixmap CallMapCallerItem::pixmap(int i) const
     if (i != 1) return QPixmap();
 
     // Cost pixmap
-
     TraceCostType* ct = ((CallMapView*)widget())->costType();
     TraceCost* t      = ((CallMapView*)widget())->totalCost();
 
-    return costPixmap( ct, _c, t->subCost(ct) / _factor );
+    // colored level meter with frame
+    return costPixmap( ct, _c, t->subCost(ct) / _factor, true );
 }
 
 
@@ -911,7 +912,7 @@ TreeMapItemList* CallMapCallerItem::children()
     TraceCostType* ct;
     ct = ((CallMapView*)widget())->costType();
 
-    SubCost s = _c->caller()->cumulative()->subCost(ct);
+    SubCost s = _c->caller()->inclusive()->subCost(ct);
     SubCost v = _c->subCost(ct);
     double newFactor = _factor * v / s;
 
@@ -947,7 +948,8 @@ QColor CallMapCallerItem::backColor() const
   return w->groupColor(_c->caller());
 }
 
-void CallMapView::readViewConfig(KConfig* c, QString prefix, QString postfix)
+void CallMapView::readViewConfig(KConfig* c,
+				 QString prefix, QString postfix, bool)
 {
     KConfigGroup* g = configGroup(c, prefix, postfix);
 
@@ -969,9 +971,12 @@ void CallMapView::readViewConfig(KConfig* c, QString prefix, QString postfix)
     setFieldStop(0, g->readEntry("StopName"));
     setMaxDrawingDepth(g->readNumEntry("MaxDepth", -1));
     setMinimalArea(g->readNumEntry("MaxArea", DEFAULT_MAXAREA));
+
+    delete g;
 }
 
-void CallMapView::saveViewConfig(KConfig* c, QString prefix, QString postfix)
+void CallMapView::saveViewConfig(KConfig* c, 
+				 QString prefix, QString postfix, bool)
 {
     KConfigGroup g(c, (prefix+postfix).ascii());
 
