@@ -167,8 +167,8 @@ QString InstrView::whatsThis() const
 		 "executing an instruction. If this is a call "
 		 "instruction, lines with details on the "
 		 "call happening are inserted into the source: "
-		 "These are (cumulative) cost spent inside of the call, "
-		 "number of calls happening, and call destination.</p>"
+		 "the cost spent inside of the call, the "
+		 "number of calls happening, and the call destination.</p>"
 		 "<p>The disassembler output shown is generated with "
 		 "the 'objdump' utility from the 'binutils' package.</p>"
 		 "<p>Select a line with call information to "
@@ -871,7 +871,7 @@ bool InstrView::fillInstrRange(TraceFunction* function,
 		      i18n("This happens because the code of"));
 	new InstrItem(this, this, 3, QString("        %1").arg(objfile));
 	new InstrItem(this, this, 4,
-		      i18n("doesn't seem to match the profile data file."));
+		      i18n("does not seem to match the profile data file."));
 	new InstrItem(this, this, 5, "");
 	new InstrItem(this, this, 6,
 		      i18n("Are you using an old profile data file or is the above mentioned"));
