@@ -32,8 +32,11 @@
 #include <qptrvector.h>
 #include <qcolor.h>
 
+#include "subcost.h"
 #include "utils.h"
 
+class QFile;
+
 /**
  * All cost items are classes prefixed with "Trace".
  * "TraceCost" holds basic cost metrics for the simplest, smallest
@@ -45,10 +48,10 @@
  * Abstract, i.e. never instantiated cost items are
  * - TraceCost: Basic cost metrics (instr/read/write access + cache events)
  * - TraceCallCost: Additional call count cost metric.
- * - TraceCumulativeCost: Additional TraceCost aggregated.
+ * - TraceInclusiveCost: Additional TraceCost aggregated.
  * - TraceListCost: Adds dependency to a list of TraceCost's
  * - TraceCallListCost: same for list of TraceCallCost's
- * - TraceCumulativeListCost: same for list of TraceCumulativeCost's
+ * - TraceInclusiveListCost: same for list of TraceInclusiveCost's
  * - TraceCostItem: Base for cost items for "interesting" costs:
  *              TraceFunction, TraceClass, TraceFile, TraceObject
  *
@@ -119,6 +122,7 @@ class FixCost;
 class FixCallCost;
 class FixJump;
 class FixPool;
+class DynPool;
 class TopLevel;
 
 class TraceCost;
@@ -127,13 +131,14 @@ class TraceCostMapping;
 class TraceSubMapping;
 class TraceJumpCost;
 class TraceCallCost;
-class TraceCumulativeCost;
+class TraceInclusiveCost;
 
 class TracePartInstr;
 class TracePartInstrCall;
 class TracePartLine;
 class TracePartLineCall;
 class TracePartCall;
+class TracePartLineRegion;
 class TracePartFunction;
 class TracePartClass;
 class TracePartObject;
@@ -146,6 +151,7 @@ class TraceLine;
 class TraceLineJump;
 class TraceLineCall;
 class TraceCall;
+class TraceLineRegion;
 class TraceFunctionSource;
 class TraceFunction;
 class TraceFunctionCycle;
@@ -158,11 +164,12 @@ class TraceData;
 typedef QPtrList<TraceCost> TraceCostList;
 typedef QPtrList<TraceJumpCost> TraceJumpCostList;
 typedef QPtrList<TraceCallCost> TraceCallCostList;
-typedef QPtrList<TraceCumulativeCost> TraceCumulativeCostList;
+typedef QPtrList<TraceInclusiveCost> TraceInclusiveCostList;
 
 typedef QPtrList<TracePartCall>  TracePartCallList;
 typedef QPtrList<TracePartInstr> TracePartInstrList;
 typedef QPtrList<TracePartLine>  TracePartLineList;
+typedef QPtrList<TracePartLineRegion>  TracePartLineRegionList;
 typedef QPtrList<TracePartFunction>  TracePartFunctionList;
 typedef QPtrList<TracePartInstrCall> TracePartInstrCallList;
 typedef QPtrList<TracePartLineCall>  TracePartLineCallList;
@@ -174,6 +181,7 @@ typedef QPtrList<TraceInstrCall> TraceInstrCallList;
 typedef QPtrList<TraceLineCall> TraceLineCallList;
 typedef QPtrList<TraceCall> TraceCallList;
 typedef QPtrList<TraceFile> TraceFileList;
+typedef QPtrList<TraceLineRegion> TraceLineRegionList;
 typedef QPtrList<TraceFunctionSource> TraceFunctionSourceList;
 typedef QPtrList<TraceFunction> TraceFunctionList;
 typedef QPtrList<TraceFunctionCycle> TraceFunctionCycleList;
@@ -182,54 +190,48 @@ typedef QMap<QString, TraceClass> TraceClassMap;
 typedef QMap<QString, TraceFile> TraceFileMap;
 typedef QMap<QString, TraceFunction> TraceFunctionMap;
 typedef QMap<uint, TraceLine> TraceLineMap;
-typedef QMap<uint, TraceInstr> TraceInstrMap;
 
 
-typedef unsigned long long uint64;
-
 /**
- * Cost event counter, simple wrapper around a 64bit entity
+ * Addresses are 64bit values like costs to be able
+ * to always load profile data produced on 64bit
+ * architectures.
  */
-class SubCost
+class Addr
 {
  public:
-    SubCost() {}
-    SubCost(uint64 i) { v=i; }
-    SubCost(unsigned i) { v=i; }
-    SubCost(int i) { v=(unsigned)i; }
-    SubCost(double d) { v= (uint64)(d + .5); }
+  Addr() { _v=0; }
+  Addr(uint64 v) { _v = v; }
 
-    SubCost& operator=(uint64 i) { v = i; return *this; }
-    SubCost& operator=(unsigned i) { v = i; return *this; }
-    SubCost& operator=(int i) { v = i; return *this; }
-    SubCost& operator=(double d) { v = (uint64)(d + .5); return *this; }
+  // Interpretes char data at s as hex (without "0x" prefix)
+  // and return number of interpreted chars.
+  int set(const char *s);
+  bool set(FixString& s);
+  QString toString() const;
+  // similar to toString(), but adds a space every 4 digits
+  QString pretty() const;
 
-    bool set(const char** s);
-    bool set(FixString& s) { return s.stripUInt64(v); }
+  // returns true if this address is in [a-distance;a+distance]
+  bool isInRange(Addr a, int distance);
 
-    operator uint64&() { return v; }
+  bool operator==(const Addr& a) const { return (_v == a._v); }
+  bool operator!=(const Addr& a) const { return (_v != a._v); }
+  bool operator>(const Addr& a) const { return _v > a._v; }
+  bool operator>=(const Addr& a) const { return _v >= a._v; }
+  bool operator<(const Addr& a) const { return _v < a._v; }
 
-    bool operator==(unsigned i) const { return v == i; }
-    bool operator==(int i) const { return v == (unsigned)i; }
-    bool operator<(unsigned i) const { return v < i; }
-    bool operator<(int i) const { return v < (unsigned)i; }
-    bool operator<(const SubCost& s) const { return v < s.v; }
-    bool operator>(unsigned i) const { return v > i; }
-    bool operator>(int i) const { return v > (unsigned)i; }
-    bool operator>(const SubCost& s) const { return v > s.v; }
-
-    /**
-     * Convert SubCost value into a QString,
-     * spaced every 3 digits.
-     */
-    QString pretty();
-
-    uint64 v;
+  Addr operator+(int d) const { return Addr(_v + d); }
+  Addr operator-(int d) const { return Addr(_v - d); }
+  
+ private:
+  uint64 _v;
 };
 
+typedef QMap<Addr, TraceInstr> TraceInstrMap;
+
 
 /**
- * An abstract item from a trace file.
+ * Base class for cost items.
  */
 class TraceItem
 {
@@ -244,6 +246,7 @@ public:
 		  PartInstrCall, InstrCall,
                   PartLineCall, LineCall,
                   PartCall, Call,
+		  PartLineRegion, LineRegion,
                   PartFunction, FunctionSource, Function, FunctionCycle,
                   PartClass, Class, ClassCycle,
                   PartFile, File, FileCycle,
@@ -263,6 +266,8 @@ public:
   // these use localisation settings
   static QString     i18nTypeName(CostType);
   static CostType    i18nCostType(QString);
+  // clean up some static data
+  static void cleanup();
 
   /**
    * Returns dynamic name info (without type)
@@ -312,15 +317,18 @@ public:
   TraceItem* dependant() { return _dep; }
 
   /**
-   * If this item is not derived, but from a single trace file
-   * set the trace file (=part) with this function.
-   * Call this function directly after the constructor.
+   * If this item is from a single profile data file, position
+   * points to a TracePart, otherwise to a TraceData object.
    */
-  void setPart(TracePart* p) { _part = p; }
+  void setPosition(TraceItem* p) { _position = p; }  
 
-  TracePart* part() const { return _part; }
+  // getters for specific positions, to be overwritten
+  virtual TracePart* part();
+  virtual const TracePart* part() const;
+  virtual TraceData* data();
+  virtual const TraceData* data() const;
 
-protected:
+ protected:
   /** Updates cost attributes.
    * This has to be called by subclasses that access cost attributes
    * directly
@@ -329,8 +337,11 @@ protected:
 
   bool _dirty;
 
-  TracePart* _part;
+  TraceItem* _position;
   TraceItem* _dep;
+
+ private:
+  static QString *_typeName, *_i18nTypeName;
 };
 
 
@@ -351,6 +362,7 @@ public:
 #define MaxRealIndexValue 10
   static const int InvalidIndex;
 
+
   TraceCost();
   virtual ~TraceCost();
 
@@ -368,6 +380,11 @@ public:
   // add the cost of another item
   void addCost(TraceCost* item);
   void addCost(int index, SubCost value);
+
+  // maximal cost
+  void maxCost(TraceSubMapping*, FixString&);
+  void maxCost(TraceCost* item);
+  void maxCost(int index, SubCost value);
   TraceCost diff(TraceCost* item);
 
   virtual void invalidate();
@@ -383,11 +400,11 @@ public:
   SubCost subCost(int);
 
   /** Returns a cost attribute converted to a string
-   * (with space after 3 digits)
+   * (with space after every 3 digits)
    */
   QString prettySubCost(TraceCostType*);
 
-protected:
+ protected:
   virtual void update();
 
   SubCost _cost[MaxRealIndexValue];
@@ -459,9 +476,11 @@ public:
   int histCost(TraceCost* c, double total, double* hist);
 
   // application wide known types, referenced by short name
+  // next 2 functions return a new type object instance
   static TraceCostType* knownRealType(QString);
   static TraceCostType* knownVirtualType(QString);
   static void add(TraceCostType*);
+  static bool remove(QString);
   static int knownTypeCount();
   static TraceCostType* knownType(int);
 
@@ -563,10 +582,9 @@ public:
     { return (i<0 || i>=_count) ? TraceCost::InvalidIndex : _realIndex[i]; }
 
   /*
-   * Allows an iteration over all real indexes not used in this submapping.
-   * Usage: for(i = firstUnused();
-   *            i != TraceCost::InvalidIndex;
-   *            i = nextUnused(i)) { LOOP }
+   * Allows an iteration over the sorted list of all real indexes not used in
+   * this submapping, up to topIndex (use TraceCost::MaxRealIndex for all).
+   * Usage: for(i = firstUnused(); i < topIndex; i = nextUnused(i)) { LOOP }
    */
   int firstUnused() { return _firstUnused; }
   int nextUnused(int i) {
@@ -603,7 +621,7 @@ class TraceJumpCost: public TraceItem
     void addFollowedCount(SubCost c) { _followedCount += c; }
     void addExecutedCount(SubCost c) { _executedCount += c; }
 
-protected:
+ protected:
     SubCost _executedCount, _followedCount;
 };
 
@@ -614,7 +632,7 @@ protected:
  */
 class TraceCallCost: public TraceCost
 {
-public:
+ public:
   TraceCallCost();
   virtual ~TraceCallCost();
 
@@ -627,30 +645,30 @@ public:
   QString prettyCallCount();
   void addCallCount(SubCost c);
 
-protected:
+ protected:
   SubCost _callCount;
 };
 
 
 /**
- * Cost item with additional cumulative metric
+ * Cost item with additional inclusive metric
  */
-class TraceCumulativeCost: public TraceCost
+class TraceInclusiveCost: public TraceCost
 {
-public:
-  TraceCumulativeCost();
-  virtual ~TraceCumulativeCost();
+ public:
+  TraceInclusiveCost();
+  virtual ~TraceInclusiveCost();
 
   // reimplementations for cost addition
   virtual QString costString(TraceCostMapping* m);
   virtual void clear();
 
   // additional cost metric
-  TraceCost* cumulative();
-  void addCumulative(TraceCost*);
+  TraceCost* inclusive();
+  void addInclusive(TraceCost*);
 
-protected:
-  TraceCost _cumulative;
+ protected:
+  TraceCost _inclusive;
 };
 
 
@@ -660,8 +678,8 @@ protected:
  */
 class TraceListCost: public TraceCost
 {
-public:
-  TraceListCost(bool onlyActiveParts = false);
+ public:
+  TraceListCost();
   virtual ~TraceListCost();
 
   // reimplementation for dependency list
@@ -669,13 +687,15 @@ public:
 
   TraceCostList& deps() { return _deps; }
   void addDep(TraceCost*);
-  TraceCost* findDep(TracePart*);
+  TraceCost* findDepFromPart(TracePart*);
+
+ protected:
+  // overwrite in subclass to change update behaviour
+  virtual bool onlyActiveParts() { return false; }
 
-protected:
   TraceCostList _deps;
-  bool _onlyActiveParts;
 
-private:
+ private:
   // very temporary: cached
   TraceCost* _lastDep;
 };
@@ -687,8 +707,8 @@ private:
  */
 class TraceJumpListCost: public TraceJumpCost
 {
-public:
-  TraceJumpListCost(bool onlyActiveParts = false);
+ public:
+  TraceJumpListCost();
   virtual ~TraceJumpListCost();
 
   // reimplementation for dependency list
@@ -696,14 +716,15 @@ public:
 
   TraceJumpCostList deps() { return _deps; }
   void addDep(TraceJumpCost*);
-  TraceJumpCost* findDep(TracePart*);
+  TraceJumpCost* findDepFromPart(TracePart*);
 
-protected:
+ protected:
+  // overwrite in subclass to change update behaviour
+  virtual bool onlyActiveParts() { return false; }
 
   TraceJumpCostList _deps;
-  bool _onlyActiveParts;
 
-private:
+ private:
   // very temporary: cached
   TraceJumpCost* _lastDep;
 };
@@ -717,8 +738,8 @@ private:
  */
 class TraceCallListCost: public TraceCallCost
 {
-public:
-  TraceCallListCost(bool onlyActiveParts = false);
+ public:
+  TraceCallListCost();
   virtual ~TraceCallListCost();
 
   // reimplementation for dependency list
@@ -726,42 +747,45 @@ public:
 
   TraceCallCostList deps() { return _deps; }
   void addDep(TraceCallCost*);
-  TraceCallCost* findDep(TracePart*);
+  TraceCallCost* findDepFromPart(TracePart*);
 
-protected:
+ protected:
+  // overwrite in subclass to change update behaviour
+  virtual bool onlyActiveParts() { return false; }
 
   TraceCallCostList _deps;
-  bool _onlyActiveParts;
 
-private:
+ private:
   // very temporary: cached
   TraceCallCost* _lastDep;
 };
 
 
 /**
- * Cumulative Cost Item dependend on a list of cumulative cost items.
+ * Inclusive Cost Item dependend on a list of inclusive cost items.
  */
-class TraceCumulativeListCost: public TraceCumulativeCost
+class TraceInclusiveListCost: public TraceInclusiveCost
 {
-public:
-  TraceCumulativeListCost(bool onlyActiveParts = false);
-  virtual ~TraceCumulativeListCost();
+ public:
+  TraceInclusiveListCost();
+  virtual ~TraceInclusiveListCost();
 
   // reimplementation for dependency
   virtual void update();
 
-  TraceCumulativeCostList deps() { return _deps; }
-  void addDep(TraceCumulativeCost*);
-  TraceCumulativeCost* findDep(TracePart*);
+  TraceInclusiveCostList deps() { return _deps; }
+  void addDep(TraceInclusiveCost*);
+  TraceInclusiveCost* findDepFromPart(TracePart*);
 
-protected:
-  TraceCumulativeCostList _deps;
-  bool _onlyActiveParts;
+ protected:
+  // overwrite in subclass to change update behaviour
+  virtual bool onlyActiveParts() { return false; }
 
-private:
+  TraceInclusiveCostList _deps;
+
+ private:
   // very temporary: cached
-  TraceCumulativeCost* _lastDep;
+  TraceInclusiveCost* _lastDep;
 };
 
 
@@ -779,13 +803,18 @@ private:
 class TracePartInstrJump: public TraceJumpCost
 {
  public:
-    TracePartInstrJump(TraceInstrJump*, TracePart*);
-    virtual ~TracePartInstrJump();
+  TracePartInstrJump(TraceInstrJump*, TracePartInstrJump*);
+  virtual ~TracePartInstrJump();
 
     virtual CostType type() const { return PartInstrJump; }
     // fix cost item
     virtual void update() {}
     TraceInstrJump* instrJump() const { return (TraceInstrJump*) _dep; }
+    TracePartInstrJump* next() const { return _next; }
+
+ private:
+    // chaining all parts for InstrJump
+    TracePartInstrJump* _next;
 };
 
 
@@ -796,7 +825,7 @@ class TracePartInstrJump: public TraceJumpCost
 class TracePartInstrCall: public TraceCallCost
 {
 public:
-  TracePartInstrCall(TraceInstrCall*, TracePart*);
+  TracePartInstrCall(TraceInstrCall*);
   virtual ~TracePartInstrCall();
 
   virtual CostType type() const { return PartInstrCall; }
@@ -813,7 +842,7 @@ public:
 class TracePartInstr: public TraceCost
 {
 public:
-  TracePartInstr(TraceInstr*, TracePart*);
+  TracePartInstr(TraceInstr*);
   virtual ~TracePartInstr();
 
   virtual CostType type() const { return PartInstr; }
@@ -830,7 +859,7 @@ public:
 class TracePartLineJump: public TraceJumpCost
 {
  public:
-    TracePartLineJump(TraceLineJump*, TracePart*);
+    TracePartLineJump(TraceLineJump*);
     virtual ~TracePartLineJump();
 
     virtual CostType type() const { return PartLineJump; }
@@ -847,7 +876,7 @@ class TracePartLineJump: public TraceJumpCost
 class TracePartLineCall: public TraceCallCost
 {
 public:
-  TracePartLineCall(TraceLineCall*, TracePart*);
+  TracePartLineCall(TraceLineCall*);
   virtual ~TracePartLineCall();
 
   virtual CostType type() const { return PartLineCall; }
@@ -865,7 +894,7 @@ public:
 class TracePartLine: public TraceCost
 {
 public:
-  TracePartLine(TraceLine*, TracePart*);
+  TracePartLine(TraceLine*);
   virtual ~TracePartLine();
 
   virtual CostType type() const { return PartLine; }
@@ -876,6 +905,22 @@ public:
 };
 
 
+/**
+ * Cost of a source region.
+ */
+class TracePartLineRegion: public TraceInclusiveCost
+{
+public:
+  TracePartLineRegion(TraceLineRegion*);
+  virtual ~TracePartLineRegion();
+
+  virtual CostType type() const { return PartLineRegion; }
+  virtual void update();
+
+  TraceLineRegion* region() const { return (TraceLineRegion*)_dep; }
+};
+
+
 /**
  * Cost of a call at a function to another function,
  * from a single trace file.
@@ -883,7 +928,7 @@ public:
 class TracePartCall: public TraceCallListCost
 {
 public:
-  TracePartCall(TraceCall* call, TracePart* part);
+  TracePartCall(TraceCall* call);
   virtual ~TracePartCall();
 
   virtual CostType type() const { return PartCall; }
@@ -908,10 +953,10 @@ private:
  * Cost of a function,
  * from a single trace file.
  */
-class TracePartFunction: public TraceCumulativeCost
+class TracePartFunction: public TraceInclusiveCost
 {
 public:
-  TracePartFunction(TraceFunction*, TracePart*,
+  TracePartFunction(TraceFunction*,
 		    TracePartObject*, TracePartFile*);
   virtual ~TracePartFunction();
 
@@ -973,10 +1018,10 @@ private:
  * Cost of a class,
  * from a single trace file.
  */
-class TracePartClass: public TraceCumulativeListCost
+class TracePartClass: public TraceInclusiveListCost
 {
 public:
-  TracePartClass(TraceClass*, TracePart*);
+  TracePartClass(TraceClass*);
   virtual ~TracePartClass();
 
   virtual CostType type() const { return PartClass; }
@@ -991,10 +1036,10 @@ public:
  * Cost of a source file,
  * from a single trace file.
  */
-class TracePartFile: public TraceCumulativeListCost
+class TracePartFile: public TraceInclusiveListCost
 {
 public:
-  TracePartFile(TraceFile*, TracePart*);
+  TracePartFile(TraceFile*);
   virtual ~TracePartFile();
 
   virtual CostType type() const { return PartFile; }
@@ -1007,10 +1052,10 @@ public:
  * Cost of a object,
  * from a single trace file.
  */
-class TracePartObject: public TraceCumulativeListCost
+class TracePartObject: public TraceInclusiveListCost
 {
 public:
-  TracePartObject(TraceObject*, TracePart*);
+  TracePartObject(TraceObject*);
   virtual ~TracePartObject();
 
   virtual CostType type() const { return PartObject; }
@@ -1027,13 +1072,16 @@ public:
 class TracePart: public TraceListCost
 {
 public:
-  TracePart(TraceData*, QString file);
+  TracePart(TraceData*, QFile* file);
   virtual ~TracePart();
 
   virtual CostType type() const { return Part; }
+  virtual TracePart* part() { return this; }
+  virtual const TracePart* part() const { return this; }
 
   QString shortName() const;
   QString prettyName() const;
+  QFile* file() const { return _file; }
   QString name() const { return _name; }
   QString description() const { return _descr; }
   QString trigger() const { return _trigger; }
@@ -1049,9 +1097,8 @@ public:
   void setPartNumber(int n);
   void setThreadID(int t);
   void setProcessID(int p);
-  //const TraceCostList& items() const { return _items; }
-  TraceData* data() const { return _data; }
   TraceCost* totals() { return &_totals; }
+  /* we get owner of the submapping */
   void setFixSubMapping(TraceSubMapping* sm) { _fixSubMapping = sm; }
   TraceSubMapping* fixSubMapping() { return _fixSubMapping; }
 
@@ -1060,16 +1107,14 @@ public:
   bool isActive() { return _active; }
 
 private:
+  QFile* _file;
   QString _name;
-
   QString _descr;
   QString _trigger;
   QString _timeframe;
   QString _version;
 
   int _number, _tid, _pid;
-  //TraceCostList _items;
-  TraceData* _data;
 
   bool _active;
 
@@ -1083,9 +1128,9 @@ private:
 
 class TracePartList: public QPtrList<TracePart>
 {
-public:
+ public:
   QString names() const;
-protected:
+ protected:
   int compareItems ( Item item1, Item item2 );
 };
 
@@ -1099,7 +1144,7 @@ protected:
 /**
  * A jump from an instruction to another inside of a function
  */
-class TraceInstrJump: public TraceJumpListCost
+class TraceInstrJump: public TraceJumpCost
 {
 public:
     TraceInstrJump(TraceInstr* instrFrom, TraceInstr* instrTo,
@@ -1109,6 +1154,8 @@ public:
     virtual CostType type() const { return InstrJump; }
     virtual QString name() const;
 
+    virtual void update();
+
     TraceInstr* instrFrom() const { return _instrFrom; }
     TraceInstr* instrTo() const { return _instrTo; }
     bool isCondJump() const { return _isCondJump; }
@@ -1119,6 +1166,8 @@ public:
  private:
     TraceInstr *_instrFrom, *_instrTo;
     bool _isCondJump;
+    // list of parts for this InstrJump
+    TracePartInstrJump* _first;
 };
 
 class TraceInstrJumpList: public QPtrList<TraceInstrJump>
@@ -1127,7 +1176,7 @@ class TraceInstrJumpList: public QPtrList<TraceInstrJump>
     TraceInstrJumpList() { _sortLow = true; }
     void setSortLow(bool s) { _sortLow = s; }
 
-protected:
+ protected:
   int compareItems ( Item item1, Item item2 );
 
  private:
@@ -1140,7 +1189,7 @@ protected:
  */
 class TraceLineJump: public TraceJumpListCost
 {
-public:
+ public:
   TraceLineJump(TraceLine* lineFrom, TraceLine* lineTo,
 		bool isCondJump);
   virtual ~TraceLineJump();
@@ -1155,7 +1204,10 @@ public:
   // part factory
   TracePartLineJump* partLineJump(TracePart*);
 
-private:
+ protected:
+  bool onlyActiveParts() { return true; }
+
+ private:
   TraceLine *_lineFrom, *_lineTo;
   bool _isCondJump;
 };
@@ -1167,7 +1219,7 @@ class TraceLineJumpList: public QPtrList<TraceLineJump>
     TraceLineJumpList() { _sortLow = true; }
     void setSortLow(bool s) { _sortLow = s; }
 
-protected:
+ protected:
   int compareItems ( Item item1, Item item2 );
 
  private:
@@ -1180,7 +1232,7 @@ protected:
  */
 class TraceInstrCall: public TraceCallListCost
 {
-public:
+ public:
   TraceInstrCall(TraceCall* call, TraceInstr* instr);
   virtual ~TraceInstrCall();
 
@@ -1192,7 +1244,11 @@ public:
 
   // part factory
   TracePartInstrCall* partInstrCall(TracePart*, TracePartCall*);
-private:
+
+ protected:
+  bool onlyActiveParts() { return true; }
+
+ private:
   TraceInstr* _instr;
   TraceCall* _call;
 };
@@ -1203,7 +1259,7 @@ private:
  */
 class TraceLineCall: public TraceCallListCost
 {
-public:
+ public:
   TraceLineCall(TraceCall* call, TraceLine* line);
   virtual ~TraceLineCall();
 
@@ -1215,7 +1271,11 @@ public:
 
   // part factory
   TracePartLineCall* partLineCall(TracePart*, TracePartCall*);
-private:
+
+ protected:
+  bool onlyActiveParts() { return true; }
+
+ private:
   TraceLine* _line;
   TraceCall* _call;
 };
@@ -1227,7 +1287,7 @@ private:
  */
 class TraceCall: public TraceCallListCost
 {
-public:
+ public:
   TraceCall(TraceFunction* caller, TraceFunction* called);
   virtual ~TraceCall();
 
@@ -1260,7 +1320,10 @@ public:
   FixCallCost* setFirstFixCost(FixCallCost* fc)
     { FixCallCost* t = _firstFixCost; _firstFixCost = fc; return t; }
 
-private:
+ protected:
+  bool onlyActiveParts() { return true; }
+
+ private:
   TraceInstrCallList _instrCalls;
   TraceLineCallList _lineCalls;
   TraceFunction* _caller;
@@ -1277,7 +1340,7 @@ private:
  */
 class TraceInstr: public TraceListCost
 {
-public:
+ public:
   TraceInstr();
   virtual ~TraceInstr();
 
@@ -1285,7 +1348,7 @@ public:
   virtual QString name() const;
   QString prettyName() const;
 
-  bool isValid() { return _addr != 0; }
+  bool isValid() { return _addr != Addr(0); }
 
   // factories
   TracePartInstr* partInstr(TracePart* part,
@@ -1294,7 +1357,7 @@ public:
 
   void addInstrCall(TraceInstrCall*);
 
-  uint addr() const { return _addr; }
+  Addr addr() const { return _addr; }
   TraceFunction* function() const { return _function; }
   TraceLine* line() const { return _line; }
   const TraceInstrJumpList& instrJumps() const { return _instrJumps; }
@@ -1302,16 +1365,18 @@ public:
   bool hasCost(TraceCostType*);
 
   // only to be called after default constructor
-  void setAddr(uint addr) { _addr = addr; }
+  void setAddr(const Addr addr) { _addr = addr; }
   void setFunction(TraceFunction* f) { _function = f; }
   void setLine(TraceLine* l) { _line = l; }
 
-private:
-  uint _addr;
+ protected:
+  bool onlyActiveParts() { return true; }
+
+ private:
+  Addr _addr;
   TraceFunction* _function;
   TraceLine* _line;
 
-  TracePartInstrList _items;
   TraceInstrJumpList _instrJumps;
   TraceInstrCallList _instrCalls;
 };
@@ -1344,7 +1409,6 @@ public:
   bool hasCost(TraceCostType*);
   TraceFunctionSource* functionSource() const { return _sourceFile; }
   uint lineno() const { return _lineno; }
-  uint items() const { return _items.count(); }
   const TraceLineCallList& lineCalls() const { return _lineCalls; }
   const TraceLineJumpList& lineJumps() const { return _lineJumps; }
 
@@ -1352,11 +1416,13 @@ public:
   void setSourceFile(TraceFunctionSource* sf) { _sourceFile = sf; }
   void setLineno(uint lineno) { _lineno = lineno; }
 
-private:
+ protected:
+  bool onlyActiveParts() { return true; }
+
+ private:
   TraceFunctionSource* _sourceFile;
   uint _lineno;
 
-  TracePartLineList _items;
   TraceLineJumpList _lineJumps;
   TraceLineCallList _lineCalls;
 };
@@ -1365,34 +1431,57 @@ private:
 /*
  * Base class for all costs which
  * represent "interesting" items or group of items
- * with settable name and cumulative cost
+ * with settable name and inclusive cost
  */
-class TraceCostItem: public TraceCumulativeListCost
+class TraceCostItem: public TraceInclusiveListCost
 {
-public:
+ public:
   TraceCostItem();
   virtual ~TraceCostItem();
 
   virtual QString name() const { return _name; }
-  TraceData* data() const { return _data; }
-
-  void setData(TraceData* data) { _data = data; }
   virtual void setName(const QString& name) { _name = name; }
 
-protected:
-  TraceData* _data;
+ protected:
+  bool onlyActiveParts() { return true; }
+
+ protected:
   QString _name;
 };
 
 
+/**
+ * Cost of a source region.
+ */
+class TraceLineRegion: public TraceInclusiveListCost
+{
+public:
+  TraceLineRegion(uint from, uint to, QString name);
+  virtual ~TraceLineRegion();
+
+  virtual CostType type() const { return LineRegion; }
+  virtual void update();
+
+  uint from() const { return _from; }
+  uint to() const { return _to; }
+  QString name() const { return _name; }
+
+  // factories
+  TracePartLine* partLineRegion(TracePart* part,
+			    TracePartFunction* partFunction);
+ private:
+  uint _from, _to;
+  QString _name;
+};
+
 
 /**
  * A container helper class for TraceFunction for source lines
  * where a function is implemented in.
  * With inlining, lines of the same function can come from
  * different source files.
- * A instance of this class holds all lines of one source file
- * in a map
+ * An instance of this class holds all lines of one source file
+ * for a function in a map
  */
 class TraceFunctionSource: public TraceCost
 {
@@ -1414,14 +1503,17 @@ public:
 
   void invalidateDynamicCost();
 
-  /* factory */
+  /* factories */
   TraceLine* line(uint lineno, bool createNew = true);
+  TraceLineRegion* region(uint from, uint to, QString name,
+		      bool createNew = true);
 
-private:
+ private:
   TraceFile* _file;
   TraceFunction* _function;
   TraceLineMap* _lineMap;
   TraceLine* _line0;
+  TraceLineRegionList* _regions;
 
   bool _lineMapFilled;
 };
@@ -1433,7 +1525,7 @@ private:
  */
 class TraceAssoziation
 {
-public:
+ public:
   /**
    * Creates an invalid assoziation.
    */
@@ -1471,7 +1563,7 @@ public:
    */
   static void invalidate(TraceData* data, int rtti);
 
-protected:
+ protected:
   TraceFunction* _function;
   bool _valid;
 };
@@ -1487,7 +1579,7 @@ typedef QPtrList<TraceAssoziation> TraceAssoziationList;
  */
 class TraceFunction: public TraceCostItem
 {
-public:
+ public:
   TraceFunction();
   TraceFunction(TraceData* data, const QString& name,
                 TraceClass* cls, TraceFile* file, TraceObject* object);
@@ -1505,7 +1597,7 @@ public:
   // factories
   TraceCall* calling(TraceFunction* called);
   TraceLine* line(TraceFile*, uint lineno, bool createNew = true);
-  TraceInstr* instr(uint addr, bool createNew = true);
+  TraceInstr* instr(Addr addr, bool createNew = true);
   TracePartFunction* partFunction(TracePart*,
                                   TracePartFile*, TracePartObject*);
 
@@ -1523,8 +1615,8 @@ public:
   TraceCallList callers(bool skipCycle=false) const;
   const TraceCallList& callings(bool skipCycle=false) const;
 
-  uint firstAddress() const;
-  uint lastAddress() const;
+  Addr firstAddress() const;
+  Addr lastAddress() const;
   TraceInstrMap* instrMap();
 
   // cost metrics
@@ -1556,12 +1648,12 @@ public:
   void cycleReset();
   void cycleDFS(int d, int& pNo, TraceFunction** pTop);
 
-protected:
+ protected:
   TraceCallList _callers; // list of calls we are called from
   TraceCallList _callings; // list of calls we are calling (we are owner)
   TraceFunctionCycle* _cycle;
 
-private:
+ private:
   bool isUniquePrefix(QString) const;
   TraceFunctionMap::Iterator _myMapIterator;
 
@@ -1593,7 +1685,7 @@ private:
  */
 class TraceFunctionCycle: public TraceFunction
 {
-public:
+ public:
   TraceFunctionCycle(TraceFunction*, int n);
 
   virtual CostType type() const { return FunctionCycle; }
@@ -1608,7 +1700,7 @@ public:
   int cycleNo() const { return _cycleNo; }
   const TraceFunctionList& members() const { return _members; }
 
-private:
+ private:
   TraceFunction* _base;
   int _cycleNo;
 
@@ -1625,7 +1717,7 @@ private:
  */
 class TraceClass: public TraceCostItem
 {
-public:
+ public:
   TraceClass();
   virtual ~TraceClass();
 
@@ -1638,7 +1730,7 @@ public:
   // part factory
   TracePartClass* partClass(TracePart*);
 
-private:
+ private:
   TraceFunctionList _functions;
 };
 
@@ -1649,7 +1741,7 @@ private:
  */
 class TraceFile: public TraceCostItem
 {
-public:
+ public:
   TraceFile();
   virtual ~TraceFile();
 
@@ -1672,7 +1764,7 @@ public:
   // part factory
   TracePartFile* partFile(TracePart*);
 
-private:
+ private:
   TraceFunctionList _functions;
   TraceFunctionSourceList _sourceFiles;
   QString _dir;
@@ -1685,7 +1777,7 @@ private:
  */
 class TraceObject: public TraceCostItem
 {
-public:
+ public:
   TraceObject();
   virtual ~TraceObject();
 
@@ -1701,7 +1793,7 @@ public:
   // part factory
   TracePartObject* partObject(TracePart*);
 
-private:
+ private:
   TraceFunctionList _functions;
   QString _shortName;
 };
@@ -1715,12 +1807,14 @@ private:
  */
 class TraceData: public TraceCost
 {
-public:
+ public:
   TraceData(TopLevel* top = 0);
   TraceData(const QString& base);
   virtual ~TraceData();
 
   virtual CostType type() const { return Data; }
+  virtual TraceData* data() { return this; }
+  virtual const TraceData* data() const { return this; }
 
   /**
    * Loads a trace file.
@@ -1729,8 +1823,6 @@ public:
    */
   void load(const QString&);
 
-  static QString tracePrefix();
-
   /** returns true if something changed. These do NOT
    * invalidate the dynamic costs on a activation change,
    * i.e. all cost items dependend on active parts.
@@ -1754,8 +1846,9 @@ public:
 
   TraceCostMapping* mapping() { return &_mapping; }
 
-  // memory pool for FixCost instances
+  // memory pools
   FixPool* fixPool();
+  DynPool* dynPool();
 
   // factories for object/file/class/function/line instances
   TraceObject* object(const QString& name);
@@ -1777,16 +1870,10 @@ public:
   TraceCost* search(TraceItem::CostType, QString,
 		    TraceCostType* ct = 0, TraceCost* parent = 0);
 
-  // same factories with support for compressed format
-  TraceObject* compressedObject(const QString& name);
-  TraceFile* compressedFile(const QString& name);
-  TraceFunction* compressedFunction(const QString& name,
-                                    TraceFile*, TraceObject*);
-
   // for pretty function names without signature if unique...
   TraceFunctionMap::Iterator functionIterator(TraceFunction*);
-  TraceFunctionMap::Iterator functionBeginIterator();
-  TraceFunctionMap::Iterator functionEndIterator();
+  TraceFunctionMap::ConstIterator functionBeginIterator() const;
+  TraceFunctionMap::ConstIterator functionEndIterator() const;
 
   TraceObjectMap& objectMap() { return _objectMap; }
   TraceFileMap& fileMap() { return _fileMap; }
@@ -1794,14 +1881,16 @@ public:
   TraceFunctionMap& functionMap() { return _functionMap; }
 
   const TraceFunctionCycleList& functionCycles() { return _functionCycles; }
+  
+  TraceCost* callMax() { return &_callMax; }
 
   void setCommand(const QString& command) { _command = command; }
   QString command() const { return _command; }
   TraceCost* totals() { return &_totals; }
   void setMaxThreadID(int tid) { _maxThreadID = tid; }
-  int maxThreadID() { return _maxThreadID; }
+  int maxThreadID() const { return _maxThreadID; }
   void setMaxPartNumber(int n) { _maxPartNumber = n; }
-  int maxPartNumber() { return _maxPartNumber; }
+  int maxPartNumber() const { return _maxPartNumber; }
 
   // reset all manually set directories for source files
   void resetSourceDirs();
@@ -1818,7 +1907,7 @@ public:
   void updateFileCycles();
   bool inFunctionCycleUpdate() { return _inFunctionCycleUpdate; }
 
-private:
+ private:
   void init();
   // add trace part: events from one trace file
   TracePart* addPart(const QString& dir, const QString& file);
@@ -1832,6 +1921,7 @@ private:
   TraceCostMapping _mapping;
 
   FixPool* _fixPool;
+  DynPool* _dynPool;
 
   // always the trace totals (not dependent on active parts)
   TraceCost _totals;
@@ -1845,10 +1935,9 @@ private:
   QString _command;
   QString _traceName;
 
-  // support for compressed format
-  QPtrVector<TraceObject> _objectVector;
-  QPtrVector<TraceFile> _fileVector;
-  QPtrVector<TraceFunction> _functionVector;
+  // Max of all costs of calls: This allows to see if the incl. cost can
+  // be hidden for a cost type, as it's always the same as self cost
+  TraceCost _callMax;
 
   // cycles
   TraceFunctionCycleList _functionCycles;
