@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -9,30 +14,30 @@
 
 #include "actor.h"
 
-UMLActor::UMLActor(const QString & name, int id) : UMLCanvasObject(name, id) {
+UMLActor::UMLActor(const QString & name, Uml::IDType id) : UMLCanvasObject(name, id) {
 	init();
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 UMLActor::~UMLActor() {}
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 void UMLActor::init() {
-	m_BaseType = ot_Actor;
+	m_BaseType = Uml::ot_Actor;
 }
 
-bool UMLActor::saveToXMI(QDomDocument& qDoc, QDomElement& qElement) {
-	QDomElement actorElement = qDoc.createElement("UML:Actor");
-	bool status = UMLObject::saveToXMI(qDoc, actorElement);
+UMLObject* UMLActor::clone() const {
+	UMLActor *clone = new UMLActor();
+	UMLObject::copyInto(clone);
+	return clone;
+}
+
+void UMLActor::saveToXMI(QDomDocument& qDoc, QDomElement& qElement) {
+	QDomElement actorElement = UMLObject::save("UML:Actor", qDoc);
 	qElement.appendChild(actorElement);
-	return status;
 }
 
-bool UMLActor::loadFromXMI(QDomElement& element) {
-	return UMLObject::loadFromXMI(element);
+bool UMLActor::load(QDomElement&) {
+	return true;
 }
 
 
-
-
-
-
 #include "actor.moc"
