@@ -530,6 +530,13 @@ public:
 	 */
 	void calculateEndingPoints();
 
+	/**
+	 * Overriding the method from WidgetBase because we need to do
+	 * something extra in case this AssociationWidget represents
+	 * an attribute of a classifier.
+	 */
+	void setUMLObject(UMLObject *obj);
+
 	/**
 	 * Saves this widget to the <assocwidget> XMI element.
 	 */
@@ -889,6 +896,14 @@ public slots:
 	 */
 	void slotClearAllSelected();
 
+	/**
+	 * Connected to UMLClassifier::attributeRemoved() in case this
+	 * AssociationWidget is linked to a classifer's attribute type.
+	 *
+	 * @param obj		The UMLAttribute removed.
+	 */
+	void slotAttributeRemoved(UMLObject* obj);
+
 	/**
 	 * Synchronize this widget from the UMLAssociation.
 	 */
