@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -17,6 +22,8 @@
 #include "statewidget.h"
 #include "activitywidget.h"
 
+using namespace Uml;
+
 AssocRules::AssocRules() {}
 
 AssocRules::~AssocRules() {}
@@ -28,7 +35,7 @@ bool allowAssociation( Association_Type/* assocType*/, const std::type_info/* &t
 }
 
 bool AssocRules::allowAssociation( Association_Type assocType, UMLWidget * widget ) {
-	UMLWidget_Type widgetType = widget -> getBaseType();
+	Widget_Type widgetType = widget -> getBaseType();
 	bool bValid = false;
 	for (int i = 0; i < m_nNumRules; i++) {
 		if (assocType == m_AssocRules[i].assoc_type) {
@@ -51,6 +58,7 @@ bool AssocRules::allowAssociation( Association_Type assocType, UMLWidget * widge
 		case at_Coll_Message:
 		case at_Generalization://can have many sub/super types
 		case at_Aggregation:
+		case at_Relationship:
 		case at_Composition:
 		case at_Containment:
 			return true;//doesn't matter whats already connected to widget
@@ -87,11 +95,11 @@ bool AssocRules::allowAssociation( Association_Type assocType, UMLWidget * widge
 	return false;
 }
 
-// when we know what we are going to connect both ends of the association to, we can 
+// when we know what we are going to connect both ends of the association to, we can
 // use this method.
 bool AssocRules::allowAssociation( Association_Type assocType, UMLWidget * widgetA, UMLWidget * widgetB , bool extendedCheck) {
-	UMLWidget_Type widgetTypeA = widgetA -> getBaseType();
-	UMLWidget_Type widgetTypeB = widgetB -> getBaseType();
+	Widget_Type widgetTypeA = widgetA->getBaseType();
+	Widget_Type widgetTypeB = widgetB->getBaseType();
 	bool bValid = false;
 	for (int i = 0; i < m_nNumRules; i++) {
 		if (assocType == m_AssocRules[i].assoc_type) {
@@ -104,8 +112,9 @@ bool AssocRules::allowAssociation( Association_Type assocType, UMLWidget * widge
 	}
 	// we can bail here for quick checks, as occur in loading files
 	// for paste or regular creation operations, we need to go further
-	if(!extendedCheck)
+	if (!extendedCheck) {
 		return bValid;
+	}
 
 	if (!bValid) {
 		return false;
@@ -120,6 +129,7 @@ bool AssocRules::allowAssociation( Association_Type assocType, UMLWidget * widge
 		case at_Dependency:
 		case at_Coll_Message:
 		case at_Aggregation:
+		case at_Relationship:
 		case at_Composition:
 		case at_Containment:
 			return true;//doesn't matter whats already connected to widget
@@ -209,7 +219,7 @@ bool AssocRules::allowRole( Association_Type assocType ) {
 	return false;
 }
 
-bool AssocRules::allowMultiplicity( Association_Type assocType, UMLWidget_Type widgetType ) {
+bool AssocRules::allowMultiplicity( Association_Type assocType, Widget_Type widgetType ) {
 	for( int i = 0; i < m_nNumRules; i++ )
 		if( assocType == m_AssocRules[ i ].assoc_type )
 			if( widgetType == m_AssocRules[ i ].widgetA_type || widgetType == m_AssocRules[ i ].widgetB_type )
@@ -217,7 +227,7 @@ bool AssocRules::allowMultiplicity( Association_Type assocType, UMLWidget_Type w
 	return false;
 }
 
-bool AssocRules::allowSelf( Association_Type assocType, UMLWidget_Type widgetType ) {
+bool AssocRules::allowSelf( Association_Type assocType, Widget_Type widgetType ) {
 	for( int i = 0; i < m_nNumRules; i++ )
 		if( assocType == m_AssocRules[ i ].assoc_type )
 			if( widgetType == m_AssocRules[ i ].widgetA_type || widgetType == m_AssocRules[ i ].widgetB_type )
@@ -227,8 +237,8 @@ bool AssocRules::allowSelf( Association_Type assocType, UMLWidget_Type widgetTyp
 }
 
 Association_Type AssocRules::isGeneralisationOrRealisation(UMLWidget* widgetA, UMLWidget* widgetB) {
-	UMLWidget_Type widgetTypeA = widgetA->getBaseType();
-	UMLWidget_Type widgetTypeB = widgetB->getBaseType();
+	Widget_Type widgetTypeA = widgetA->getBaseType();
+	Widget_Type widgetTypeB = widgetB->getBaseType();
 	for (int i = 0; i < m_nNumRules; i++) {
 		if (m_AssocRules[i].assoc_type == at_Realization &&
 		    widgetTypeA == m_AssocRules[i].widgetA_type &&
@@ -274,10 +284,12 @@ AssocRules::Assoc_Rule AssocRules::m_AssocRules []= {
             { at_Generalization,wt_Actor,	wt_Actor,	false,	false,	false,	false },
             { at_Aggregation,	wt_Class,	wt_Class,	true,	true,	false,	true  },
             { at_Aggregation,	wt_Class,	wt_Interface,	true,	true,	false,	false },
-            { at_Dependency,	wt_Class,	wt_Class,	false,	false,	false,	true },
-            { at_Dependency,	wt_UseCase,	wt_UseCase,	false,	false,	false,	false },
-            { at_Dependency,	wt_Actor,	wt_Actor,	false,	false,	false,	false },
-            { at_Dependency,	wt_Actor,	wt_UseCase,	false,	false,	false,	false },
+            { at_Aggregation,	wt_Class,	wt_Enum,	true,	true,	false,	false },
+            { at_Aggregation,	wt_Class,	wt_Datatype,	true,	true,	false,	false },
+            { at_Dependency,	wt_Class,	wt_Class,	true,	false,	false,	true },
+            { at_Dependency,	wt_UseCase,	wt_UseCase,	true,	false,	false,	false },
+            { at_Dependency,	wt_Actor,	wt_Actor,	true,	false,	false,	false },
+            { at_Dependency,	wt_Actor,	wt_UseCase,	true,	false,	false,	false },
             { at_Dependency,	wt_Package,	wt_Package,	true,	true,	true,	true  },
             { at_Dependency,	wt_Package,	wt_Class,	true,	true,	true,	true  },
             { at_Dependency,	wt_Class,	wt_Package,	true,	true,	true,	true  },
@@ -290,8 +302,8 @@ AssocRules::Assoc_Rule AssocRules::m_AssocRules []= {
             { at_Dependency,	wt_Class,	wt_Enum,	true,	true,	true,	true  },
             { at_Dependency,	wt_Component,	wt_Component,	true,	true,	true,	true  },
             { at_Dependency,	wt_Component,	wt_Interface,	true,	true,	true,	true  },
-            { at_Dependency,	wt_Component,	wt_Artifact,	false,	false,	false,	false },
-            { at_Dependency,	wt_Node,	wt_Component,	false,	false,	false,	false },
+            { at_Dependency,	wt_Component,	wt_Artifact,	true,	false,	false,	false },
+            { at_Dependency,	wt_Node,	wt_Component,	true,	false,	false,	false },
             { at_Realization,	wt_Class,	wt_Interface,	false,	false,	false,	false },
             { at_Realization,	wt_Interface,	wt_Package,	false,	false,	false,	false },
             { at_Realization,	wt_Interface,	wt_Interface,	false,	false,	false,	false },
@@ -324,6 +336,7 @@ AssocRules::Assoc_Rule AssocRules::m_AssocRules []= {
             { at_Anchor,	wt_Message,	wt_Note,	false,	false,	false,	false },
             { at_Anchor,	wt_State,	wt_Note,	false,	false,	false,	false },
             { at_Anchor,	wt_Activity,	wt_Note,	false,	false,	false,	false },
+            { at_Relationship,	wt_Entity,	wt_Entity,	true,	false,	true,	false },
         };
 
 int AssocRules::m_nNumRules = sizeof( m_AssocRules ) / sizeof( AssocRules::Assoc_Rule );
