@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2003-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -36,7 +41,7 @@ public:
 	 * @param view		The parent to this widget.
 	 * @param id		The ID to assign (-1 will prompt a new ID.)
 	 */
-	BoxWidget(UMLView * view, int id = -1);
+	BoxWidget(UMLView * view, Uml::IDType id = Uml::id_None);
 
 	/**
 	 * destructor
@@ -67,7 +72,7 @@ public:
 	void draw(QPainter & p, int offsetX, int offsetY);
 
 	/**
-	 * Saves the widget to the <UML:BoxWidget> XMI element.
+	 * Saves the widget to the <boxwidget> XMI element.
 	 * Note: For loading from XMI, the inherited parent method is used.
 	 */
 	void saveToXMI(QDomDocument& qDoc, QDomElement& qElement);
