@@ -16,6 +16,7 @@
 #include "umlclassifierlistitemlist.h"
 #include "classifierlistitem.h"
 #include "umltemplatelist.h"
+#include "model_utils.h"
 
 class IDChangeLog;
 
@@ -76,6 +77,31 @@ public:
 	 */
 	virtual UMLObject* clone() const = 0;
 
+	/**
+	 * Creates an operation in the current document.
+	 * The new operation is initialized with name, id, etc.
+	 * If a method with the given profile already exists in the classifier,
+	 * no new method is created and the existing operation is returned.
+	 * If no name is provided, or if the params are NULL, an Operation
+	 * Dialog is shown to ask the user for a name and parameters.
+	 * The operation's signature is checked for validity within the parent
+	 * classifier.
+	 *
+	 * @param name		The operation name (will be chosen internally if
+	 *			none given.)
+	 * @param isExistingOp	Optional pointer to bool. If supplied, the bool is
+	 *			set to true if an existing operation is returned.
+	 * @param params	Optional list of parameter names and types.
+	 *			If supplied, new operation parameters are
+	 *			constructed using this list.
+	 * @return The new operation, or NULL if the operation could not be
+	 *         created because for example, the user canceled the dialog
+	 *         or no appropriate name can be found.
+	*/
+	UMLOperation* createOperation( const QString &name = QString::null,
+				       bool *isExistingOp = NULL,
+				       Umbrello::NameAndType_List *params = NULL);
+
 	/**
 	 * Adds an operation to the classifier, at the given position.
 	 * If position is negative or too large, the attribute is added
@@ -132,9 +158,11 @@ public:
 	 * the returned operation (or to delete the operation)
 	 *
 	 * @param  o operation to take
+	 * @param wasAtIndex	if given, the index in m_List of the item taken
+	 *			is returned in the int pointed-to.
 	 * @return pointer to the operation or null if not found.
 	 */
-	UMLOperation* takeOperation(UMLOperation* o);
+	UMLOperation* takeOperation(UMLOperation* o, int *wasAtIndex = NULL);
 
 	/**
 	 * counts the number of operations in the Classifier.
@@ -151,6 +179,13 @@ public:
 	 */
 	UMLOperationList getOpList(bool includeInherited = false);
 
+	/**
+	 * Creates a template for the concept.
+	 *
+	 * @return	The UMLTemplate created
+	 */
+	UMLObject* createTemplate(QString name = QString::null);
+
 	/**
 	 * Adds a template to the class if it is not there yet.
 	 *
@@ -198,9 +233,11 @@ public:
 	 * the returned template (or to delete the template)
 	 *
 	 * @param t		template to take
+	 * @param wasAtIndex	if given, the index in m_List of the item taken
+	 *			is returned in the int pointed-to.
 	 * @return	pointer to the template or null if not found.
 	 */
-	UMLTemplate* takeTemplate(UMLTemplate* t);
+	UMLTemplate* takeTemplate(UMLTemplate* t, int *wasAtIndex = NULL);
 
 	/**
 	 * Seeks the template parameter of the given name.
@@ -258,7 +295,17 @@ public:
 	 *
 	 * @return	The object found.  Will return 0 if none found.
 	 */
-	virtual UMLObject* findChildObject(Uml::IDType id);
+	virtual UMLObject* findChildObject(Uml::IDType id, bool considerAncestors = false);
+
+	/**
+	 * Find an operation of the given name and parameter signature.
+	 *
+	 * @param name		The name of the operation to find.
+	 * @param params	The parameter descriptors of the operation to find.
+	 *
+	 * @return	The operation found.  Will return 0 if none found.
+	 */
+	UMLOperation* findOperation(QString name, Umbrello::NameAndType_List params);
 
 	/**
 	 * Returns a list of concepts which this concept inherits from.
