@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -20,7 +25,10 @@
  * Bugs and comments to uml-devel@lists.sf.net or http://bugs.kde.org
  */
 
-#include <qpointarray.h>
+#include <qstring.h>
+#include <qvaluevector.h>
+
+#include "../umlnamespace.h"
 
 class IDChangeLog {
 public:
@@ -52,7 +60,7 @@ public:
 	/**
 	 * Adds a new ID Change to the log.
 	 */
-	void addIDChange(int OldID, int NewID);
+	void addIDChange(Uml::IDType OldID, Uml::IDType NewID);
 
 	/**
 	 * Appends another IDChangeLog to this instance of IDChangeLog and
@@ -64,34 +72,58 @@ public:
 	 * Returns the new assigned ID of the object that had OldID as its
 	 * previous id.
 	 */
-	int findNewID(int OldID);
+	Uml::IDType findNewID(Uml::IDType OldID);
 
 	/**
 	 * Returns the old ID of an UMLobject given its new one.
 	 */
-	int findOldID(int NewID);
+	Uml::IDType findOldID(Uml::IDType NewID);
 
 	/**
 	 * Removes a change giving an New ID.
 	 */
-	void removeChangeByNewID( int OldID);
+	void removeChangeByNewID( Uml::IDType OldID);
 
 	enum SpecialIDs
 	{
 	    NullID = -1000 ///< An impossible id value.
 	};
 
-public:
-	/**
-	 * Each change is a QPoint (x=newID, y=oldID)
-	 */
-	QPointArray m_LogArray;
-
 private:
+	/**
+	 * Each change is a Point (x=newID, y=oldID)
+	 */
+	class Point {
+	public:
+		Point()
+		{}
+		Point(const Uml::IDType &x, const Uml::IDType &y)
+			: m_x(x), m_y(y)
+		{}
+		virtual ~Point() {}
+		void setX(const Uml::IDType &x) { m_x = x; }
+		Uml::IDType x() const { return m_x; }
+		void setY(const Uml::IDType &y) { m_y = y; }
+		Uml::IDType y() const { return m_y; }
+	private:
+		Uml::IDType m_x, m_y;
+	};
+	class PointArray : QValueVector<Point> {
+	public:
+		void  setPoint(uint i, const Uml::IDType &x, const Uml::IDType &y) {
+			Point point(x, y);
+			QValueVector<Point>::at(i) = point;
+		}
+		const Point& point( uint i ) const { return QValueVector<Point>::at(i); }
+		uint   size() const          { return QValueVector<Point>::size(); }
+		bool   resize( uint size )   { QValueVector<Point>::resize(size); return true; }
+	};
+	PointArray m_LogArray;
+
 	/**
 	 * Finds a specific change in the log.
 	 */
-	bool findIDChange(int OldID, int NewID, uint& pos);
+	bool findIDChange(Uml::IDType OldID, Uml::IDType NewID, uint& pos);
 };
 
 #endif
