@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,10 +15,12 @@
 #include <qdom.h>
 
 #include "umldrag.h"
+#include "idchangelog.h"
 #include "../uml.h"
 #include "../umldoc.h"
 #include "../umlview.h"
 #include "../umlobject.h"
+#include "../class.h"
 #include "../umlwidget.h"
 #include "../umllistview.h"
 #include "../umllistviewitem.h"
@@ -97,6 +104,7 @@ void UMLDrag::setSubType(const QCString& string, int index) {
 }
 
 void UMLDrag::setEncodedData(const QByteArray& encodedData, int index) {
+	encodedData.copy();
 	data->enc[index] = encodedData.copy();
 }
 
@@ -253,9 +261,8 @@ void UMLDrag::setUMLDataClip4(UMLObjectList& objects, UMLListViewItemList& umlLi
 
 	data->setType("image/PNG", 1);
 	long l_size = (pngImage.convertToImage()).numBytes();
-	char* data = new char[l_size];
 	QByteArray clipdata;
-	clipdata.setRawData(data, l_size);
+	clipdata.resize(l_size);
 	QDataStream clipstream(clipdata, IO_WriteOnly);
 	clipstream << pngImage;
 	setEncodedData(clipdata, 1);
@@ -302,7 +309,8 @@ bool UMLDrag::canDecode(const QMimeSource* mimeSource) {
 }
 
 bool UMLDrag::decodeClip1(const QMimeSource* mimeSource, UMLObjectList& objects,
-			  UMLListViewItemList& umlListViewItems, UMLDoc* doc) {
+			  UMLListViewItemList& umlListViewItems) {
+	UMLDoc* doc = UMLApp::app()->getDocument();
 	if ( !mimeSource->provides("application/x-uml-clip1") ) {
 		return false;
 	}
@@ -329,6 +337,8 @@ bool UMLDrag::decodeClip1(const QMimeSource* mimeSource, UMLObjectList& objects,
 		return false;
 	}
 
+	UMLListView *listView = UMLApp::app()->getListView();
+
 	//UMLObjects
 	QDomNode objectsNode = xmiClipNode.firstChild();
 	QDomNode objectElement = objectsNode.firstChild();
@@ -340,24 +350,45 @@ bool UMLDrag::decodeClip1(const QMimeSource* mimeSource, UMLObjectList& objects,
 	while ( !element.isNull() ) {
 		pObject = 0;
 		QString type = element.tagName();
-		if (type != "UML:Association") {
-			pObject = doc->makeNewUMLObject(type);
-
-			if( !pObject ) {
-				kdWarning() << "UMLDrag::decodeClip1: Given wrong type of umlobject to create: "
-					    << type << endl;
-				return false;
-			}
-			pObject->setInPaste( true );
-			if( !pObject->loadFromXMI( element ) ) {
-				kdWarning() << "UMLDrag::decodeClip1: failed to load object of type "
-					    << type << " from XMI" << endl;
-				delete pObject;
-				return false;
-			}
-			pObject->setInPaste( false );
-			objects.append(pObject);
+		if (type == "UML:Association") {
+			objectElement = objectElement.nextSibling();
+			element = objectElement.toElement();
+			continue;
 		}
+		pObject = UMLDoc::makeNewUMLObject(type);
+
+		if( !pObject ) {
+			kdWarning() << "UMLDrag::decodeClip1: Given wrong type of umlobject to create: "
+				    << type << endl;
+			return false;
+		}
+		pObject->setInPaste( true );
+		if( !pObject->loadFromXMI( element ) ) {
+			kdWarning() << "UMLDrag::decodeClip1: failed to load object of type "
+				    << type << " from XMI" << endl;
+			delete pObject;
+			return false;
+		}
+		pObject->setInPaste( false );
+		if (listView->startedCopy()) {
+                        /****************************************************************
+			 * If the clone() methods called IDChangeLog::addIDChange(),
+			 * we could do the following:
+                        UMLObject *newObj = pObject->clone();
+                        delete pObject;
+                        pObject = newObj;
+			 * but since that's not currently the case we do: */
+			if(!doc->assignNewIDs(pObject)) {
+				return false;
+			}
+                        Uml::Object_Type type = pObject->getBaseType();
+                        QString newName = doc->uniqObjectName(type, pObject->getName());
+                        pObject->setName(newName);
+			/****************************************************************/
+
+			pObject->resolveRef();
+		}
+		objects.append(pObject);
 		objectElement = objectElement.nextSibling();
 		element = objectElement.toElement();
 	}
@@ -370,10 +401,44 @@ bool UMLDrag::decodeClip1(const QMimeSource* mimeSource, UMLObjectList& objects,
 		kdWarning() << "no listitems in XMI clip" << endl;
 		return false;
 	}
-	UMLListView *listView = UMLApp::app()->getListView();
+	UMLListViewItem *currentItem = (UMLListViewItem*)listView->currentItem();
 	while ( !listItemElement.isNull() ) {
-		UMLListViewItem* itemData = new UMLListViewItem( listView );
-		itemData->loadFromXMI(listItemElement);
+		UMLListViewItem* itemData;
+		if (currentItem)
+			itemData = new UMLListViewItem( currentItem );
+		else
+			itemData = new UMLListViewItem( listView );
+		if (!itemData->loadFromXMI(listItemElement)) {
+			delete itemData;
+			return false;
+		}
+		if (listView->startedCopy()) {
+			IDChangeLog *idchanges = doc->getChangeLog();
+			Uml::IDType oldID = itemData->getID();
+			Uml::IDType newID = idchanges->findNewID(oldID);
+			UMLListViewItem *shouldNotExist = listView->findItem(newID);
+			if (shouldNotExist) {
+				kdDebug() << "UMLDrag::decodeClip1: new list view item "
+					  << ID2STR(newID) << " already exists" << endl;
+				listItems = listItems.nextSibling();
+				listItemElement = listItems.toElement();
+				continue;
+			}
+			UMLObject *newObj = doc->findObjectById(newID);
+			if (newObj) {
+				kdDebug() << "UMLDrag::decodeClip1: adjusting lvitem(" << ID2STR(oldID)
+					  << ") to new UMLObject(" << ID2STR(newID) << ")" << endl;
+				itemData->setUMLObject(newObj);
+				itemData->setText(newObj->getName());
+				if (! listView->loadChildrenFromXMI(itemData, listItemElement)) {
+					kdError() << "UMLDrag::decodeClip1: listView->loadChildrenFromXMI() failed"
+						  << endl;
+				}
+			} else {
+				kdDebug() << "UMLDrag::decodeClip1: no UMLObject found for lvitem "
+					  << ID2STR(newID) << endl;
+			}
+		}
 		umlListViewItems.append(itemData);
 		listItems = listItems.nextSibling();
 		listItemElement = listItems.toElement();
@@ -382,7 +447,7 @@ bool UMLDrag::decodeClip1(const QMimeSource* mimeSource, UMLObjectList& objects,
 }
 
 bool UMLDrag::decodeClip2(const QMimeSource* mimeSource, UMLObjectList& objects,
-			  UMLListViewItemList& umlListViewItems, UMLViewList& diagrams, UMLDoc* doc) {
+			  UMLListViewItemList& umlListViewItems, UMLViewList& diagrams) {
 	if ( !mimeSource->provides("application/x-uml-clip2") ) {
 		return false;
 	}
@@ -421,7 +486,7 @@ bool UMLDrag::decodeClip2(const QMimeSource* mimeSource, UMLObjectList& objects,
 		pObject = 0;
 		QString type = element.tagName();
 		if (type != "UML:Association") {
-			pObject = doc->makeNewUMLObject(type);
+			pObject = UMLDoc::makeNewUMLObject(type);
 
 			if( !pObject ) {
 				kdWarning() << "Given wrong type of umlobject to create:" << type << endl;
@@ -462,10 +527,17 @@ bool UMLDrag::decodeClip2(const QMimeSource* mimeSource, UMLObjectList& objects,
 		return false;
 	}
 	UMLListView *listView = UMLApp::app()->getListView();
+	UMLListViewItem *currentItem = (UMLListViewItem*)listView->currentItem();
 	while ( !listItemElement.isNull() ) {
-		UMLListViewItem* itemData = new UMLListViewItem( listView );
-		itemData->loadFromXMI(listItemElement);
-		umlListViewItems.append(itemData);
+		UMLListViewItem* itemData;
+		if (currentItem)
+			itemData = new UMLListViewItem( currentItem );
+		else
+			itemData = new UMLListViewItem( listView );
+		if ( itemData->loadFromXMI(listItemElement) )
+			umlListViewItems.append(itemData);
+		else
+			delete itemData;
 		listItems = listItems.nextSibling();
 		listItemElement = listItems.toElement();
 	}
@@ -522,7 +594,7 @@ bool UMLDrag::getClip3TypeAndID(const QMimeSource* mimeSource,
 		}
 		LvTypeAndID * pData = new LvTypeAndID;
 		pData->type = (Uml::ListView_Type)(typeStr.toInt());
-		pData->id = idStr.toInt();
+		pData->id = STR2ID(idStr);
 		typeAndIdList.append(pData);
 		listItems = listItems.nextSibling();
 		listItemElement = listItems.toElement();
@@ -578,9 +650,11 @@ bool UMLDrag::decodeClip3(const QMimeSource* mimeSource,
 		}
 		Uml::ListView_Type t = (Uml::ListView_Type)(type.toInt());
 		UMLListViewItem* parent = parentListView->determineParentItem(t);
-		UMLListViewItem* item = new UMLListViewItem(parent);
-		item->loadFromXMI(listItemElement);
-		umlListViewItems.append(item);
+		UMLListViewItem* itemData = new UMLListViewItem(parent);
+		if ( itemData->loadFromXMI(listItemElement) )
+			umlListViewItems.append(itemData);
+		else
+			delete itemData;
 		listItems = listItems.nextSibling();
 		listItemElement = listItems.toElement();
 	}
@@ -589,8 +663,8 @@ bool UMLDrag::decodeClip3(const QMimeSource* mimeSource,
 
 bool UMLDrag::decodeClip4(const QMimeSource* mimeSource, UMLObjectList& objects,
 			  UMLListViewItemList& umlListViewItems, UMLWidgetList& widgets,
-			  AssociationWidgetList& associations, Uml::Diagram_Type & dType,
-			  UMLDoc* doc) {
+			  AssociationWidgetList& associations, Uml::Diagram_Type & dType) {
+	UMLDoc* doc = UMLApp::app()->getDocument();
 	if ( !mimeSource->provides("application/x-uml-clip4") ) {
 		return false;
 	}
@@ -630,7 +704,7 @@ bool UMLDrag::decodeClip4(const QMimeSource* mimeSource, UMLObjectList& objects,
 		//FIXME associations don't load
 		if (type == "UML:Association")
 			continue;
-		pObject = doc->makeNewUMLObject(type);
+		pObject = UMLDoc::makeNewUMLObject(type);
 
 		if ( !pObject ) {
 			kdWarning() << "Given wrong type of umlobject to create: " << type << endl;
@@ -664,7 +738,8 @@ bool UMLDrag::decodeClip4(const QMimeSource* mimeSource, UMLObjectList& objects,
 	QDomNode associationWidgetNode = associationWidgetsNode.firstChild();
 	QDomElement associationWidgetElement = associationWidgetNode.toElement();
 	while ( !associationWidgetElement.isNull() ) {
-		AssociationWidget* associationWidget = new AssociationWidget(doc->getCurrentView());
+		UMLView *view = UMLApp::app()->getCurrentView();
+		AssociationWidget* associationWidget = new AssociationWidget(view);
 		if (associationWidget->loadFromXMI(associationWidgetElement, widgets))
 			associations.append(associationWidget);
 		else {
@@ -677,24 +752,28 @@ bool UMLDrag::decodeClip4(const QMimeSource* mimeSource, UMLObjectList& objects,
 
 	//listviewitems
 	UMLListView *listView = UMLApp::app()->getListView();
+	UMLListViewItem *currentItem = (UMLListViewItem*)listView->currentItem();
 	QDomNode listItemNode = associationWidgetsNode.nextSibling();
 	QDomNode listItems = listItemNode.firstChild();
 	QDomElement listItemElement = listItems.toElement();
 	while ( !listItemElement.isNull() ) {
-		UMLListViewItem* itemData = new UMLListViewItem( listView );
-		itemData->loadFromXMI(listItemElement);
-		umlListViewItems.append(itemData);
+		UMLListViewItem* itemData;
+		if (currentItem)
+			itemData = new UMLListViewItem( currentItem );
+		else
+			itemData = new UMLListViewItem( listView );
+		if ( itemData->loadFromXMI(listItemElement) )
+			umlListViewItems.append(itemData);
+		else
+			delete itemData;
 		listItems = listItems.nextSibling();
 		listItemElement = listItems.toElement();
 	}
 	return true;
 }
 
-#ifdef __GNUC__
-#warning "decodeClip5 needs fixing"
-#endif
-bool UMLDrag::decodeClip5(const QMimeSource* mimeSource, UMLObjectList& /* objects */,
-			  UMLListViewItemList& umlListViewItems, UMLDoc* /*doc*/) {
+bool UMLDrag::decodeClip5(const QMimeSource* mimeSource, UMLObjectList& objects,
+			  UMLListViewItemList& umlListViewItems, UMLClassifier* newParent) {
 	if ( !mimeSource->provides("application/x-uml-clip5") ) {
 		return false;
 	}
@@ -728,12 +807,11 @@ bool UMLDrag::decodeClip5(const QMimeSource* mimeSource, UMLObjectList& /* objec
 	if ( element.isNull() ) {
 		return false;//return ok as it means there is no umlobjects
 	}
-/*
 	UMLObject* pObject = 0;
 	while ( !element.isNull() ) {
 		pObject = 0;
 		QString type = element.tagName();
-		pObject = doc->makeNewClassifierObject(type, element);
+		pObject = UMLDoc::makeNewClassifierObject(type, newParent);
 
 		if( !pObject ) {
 			kdWarning() << "Given wrong type of umlobject to create:" << type << endl;
@@ -743,12 +821,12 @@ bool UMLDrag::decodeClip5(const QMimeSource* mimeSource, UMLObjectList& /* objec
 			kdWarning() << "failed to load object from XMI" << endl;
 			return false;
 		}
+		pObject->resolveRef();
 		objects.append(pObject);
 		objectElement = objectElement.nextSibling();
 		element = objectElement.toElement();
 	}
-*/
-
+/*
 	//listviewitems
 	QDomNode listItemNode = objectsNode.nextSibling();
 	QDomNode listItems = listItemNode.firstChild();
@@ -758,13 +836,21 @@ bool UMLDrag::decodeClip5(const QMimeSource* mimeSource, UMLObjectList& /* objec
 		return false;
 	}
 	UMLListView *listView = UMLApp::app()->getListView();
+	UMLListViewItem *currentItem = (UMLListViewItem*)listView->currentItem();
 	while ( !listItemElement.isNull() ) {
-		UMLListViewItem* itemData = new UMLListViewItem( listView );
-		itemData->loadFromXMI(listItemElement);
-		umlListViewItems.append(itemData);
+		UMLListViewItem* itemData;
+		if (currentItem)
+			itemData = new UMLListViewItem( currentItem );
+		else
+			itemData = new UMLListViewItem( listView );
+		if ( itemData->loadFromXMI(listItemElement) )
+			umlListViewItems.append(itemData);
+		else
+			delete itemData;
 		listItems = listItems.nextSibling();
 		listItemElement = listItems.toElement();
 	}
+ */
 	return true;
 }
 
