@@ -35,7 +35,14 @@ class AdaWriter : public SimpleCodeGenerator {
         Q_OBJECT
 public:
 
+	/**
+	 * Basic Constructor
+	 */
 	AdaWriter (UMLDoc * parent = 0, const char* name = 0);
+
+	/**
+	 * Empty Destructor
+	 */
 	virtual ~AdaWriter ();
 
 	/**
@@ -44,11 +51,33 @@ public:
 	 */
 	virtual void writeClass (UMLClassifier *c);
 
+	/**
+	 * checks whether type is "AdaWriter"
+	 *
+	 * @param type
+	 */
 	virtual bool isType (QString & type);
-        virtual QString getLanguage();
+
+	/**
+	 * returns "Ada"
+	 */
+	virtual QString getLanguage();
 
 	void createDefaultDatatypes();
 
+	/**
+	 * Check whether the given string is a reserved word for the
+	 * language of this code generator
+	 *
+	 * @param rPossiblyReservedKeyword  The string to check.
+	 */
+	virtual bool isReservedKeyword(const QString & rPossiblyReservedWord);
+
+	/**
+	 * get list of reserved keywords
+	 */
+	virtual const QStringList reservedKeywords() const;
+
 private:
 
 	/**
@@ -65,11 +94,8 @@ private:
 	QString qualifiedName
 	(UMLClassifier *c, bool withType = false, bool byValue = false);
 
-	QString spc ();
-
 	static const QString defaultPackageSuffix;
 
-	int indentlevel;
 };
 
 #endif // ADAWRITER_H
