@@ -2,7 +2,7 @@
                           pythonwriter.h  -  description
                              -------------------
     begin                : Sat Dec 21 2002
-    author               : Vincent Decorges
+    copyright            : Vincent Decorges
     email                : vincent.decorges@eivd.ch
  ***************************************************************************/
 
@@ -35,29 +35,17 @@
 #include "../umlnamespace.h"
 
 PythonWriter::PythonWriter( UMLDoc *parent, const char *name ) :
-  	SimpleCodeGenerator( parent, name) {}
+	SimpleCodeGenerator( parent, name) {
+}
 
 PythonWriter::~PythonWriter() {}
 
-void PythonWriter::setSpaceIndent(int number) {
-	spaceIndent = "";
-	for (int i = 0; i < number; i++) {
-		spaceIndent.append(' ');
-	}
-}
-
-int PythonWriter::getSpaceIndent(void) {
-	return spaceIndent.length();
-}
-
 void PythonWriter::writeClass(UMLClassifier *c) {
 	if(!c) {
-		kdDebug()<<"Cannot write class of NULL concept!\n";
+		kdDebug()<<"Cannot write class of NULL concept!" << endl;
 		return;
 	}
 
-	spaceIndent = "    ";
-
 	QString classname = cleanName(c->getName());
 	QString fileName = c->getName();
 
@@ -97,7 +85,7 @@ void PythonWriter::writeClass(UMLClassifier *c) {
 	if(!str.isEmpty()) {
 		str.replace(QRegExp("%filename%"), fileName+".py");
 		str.replace(QRegExp("%filepath%"), fileh.name());
-		h<<str<<endl;
+		h<<str<<m_endl;
 	}
 
 
@@ -111,10 +99,10 @@ void PythonWriter::writeClass(UMLClassifier *c) {
 			first = headerName.at(0);
 			first = first.upper();
 			headerName = headerName.replace(0, 1, first);
-			h<<"from "<<headerName<<" import *"<<endl;
+			h<<"from "<<headerName<<" import *"<<m_endl;
 		}
 	}
-	h<<endl;
+	h<<m_endl;
 
 	h<<"class "<<classname<<(superclasses.count() > 0 ? " (":"");
 	int i = superclasses.count();
@@ -126,21 +114,21 @@ void PythonWriter::writeClass(UMLClassifier *c) {
 	}
 
 
-	h<<(superclasses.count() > 0 ? ")":"")<<":"<<endl<<endl;
+	h<<(superclasses.count() > 0 ? ")":"")<<":"<<m_endl<<m_endl;
 
 	if(forceDoc() || !c->getDoc().isEmpty()) {
-		h<<spaceIndent<<"\"\"\""<<endl;
-		h<<spaceIndent<<c->getDoc()<<endl;
-		h<<spaceIndent<<":version:"<<endl;
-		h<<spaceIndent<<":author:"<<endl;
-		h<<spaceIndent<<"\"\"\""<<endl<<endl;
+		h<<m_indentation<<"\"\"\""<<m_endl;
+		h<<m_indentation<<c->getDoc()<<m_endl;
+		h<<m_indentation<<":version:"<<m_endl;
+		h<<m_indentation<<":author:"<<m_endl;
+		h<<m_indentation<<"\"\"\""<<m_endl<<m_endl;
 	}
 
 	//operations
 	writeOperations(c,h);
 
 	//finish files
-	h<<endl<<endl;
+	h<<m_endl<<m_endl;
 
 	//close files and notfiy we are done
 	fileh.close();
@@ -154,7 +142,6 @@ void PythonWriter::writeClass(UMLClassifier *c) {
 void PythonWriter::writeOperations(UMLClassifier *c,QTextStream &h) {
 
 	//Lists to store operations  sorted by scope
-	UMLOperationList *opl;
 	UMLOperationList oppub,opprot,oppriv;
 
 	oppub.setAutoDelete(false);
@@ -162,8 +149,8 @@ void PythonWriter::writeOperations(UMLClassifier *c,QTextStream &h) {
 	oppriv.setAutoDelete(false);
 
 	//sort operations by scope first and see if there are abstract methods
-	opl = c->getFilteredOperationsList();
-	for(UMLOperation *op = opl->first(); op ; op = opl->next()) {
+	UMLOperationList opl(c->getOpList());
+	for(UMLOperation *op = opl.first(); op ; op = opl.next()) {
 		switch(op->getScope()) {
 			case Uml::Public:
 				oppub.append(op);
@@ -223,7 +210,7 @@ void PythonWriter::writeOperations(QString /*classname*/, UMLOperationList &opLi
 		for(at = atl->first(); at ; at = atl -> next())
 			writeDoc |= !at->getDoc().isEmpty();
 
-		h<< spaceIndent << "def "<< sAccess + cleanName(op->getName()) << "(self";
+		h<< m_indentation << "def "<< sAccess + cleanName(op->getName()) << "(self";
 
 		int j=0;
 		for( at = atl->first(); at ;at = atl->next(),j++) {
@@ -233,43 +220,192 @@ void PythonWriter::writeOperations(QString /*classname*/, UMLOperationList &opLi
 			    QString(""));
 		}
 
-		h<<"):"<<endl;
+		h<<"):"<<m_endl;
 
 		if( writeDoc )  //write method documentation
 		{
-			h<<spaceIndent<<spaceIndent<<"\"\"\""<<endl;
-			h<<spaceIndent<<spaceIndent<<op->getDoc()<<endl<<endl;
+			h<<m_indentation<<m_indentation<<"\"\"\""<<m_endl;
+			h<<m_indentation<<m_indentation<<op->getDoc()<<m_endl<<m_endl;
 
 			for(at = atl->first(); at ; at = atl -> next())  //write parameter documentation
 			{
 				if(forceDoc() || !at->getDoc().isEmpty()) {
-					h<<spaceIndent<<spaceIndent<<"@param "<<at->getTypeName()<<
+					h<<m_indentation<<m_indentation<<"@param "<<at->getTypeName()<<
 						" " << cleanName(at->getName());
-					h<<" : "<<at->getDoc()<<endl;
+					h<<" : "<<at->getDoc()<<m_endl;
 				}
 			}//end for : write parameter documentation
-			h<<spaceIndent<<spaceIndent<<"@return " + op->getReturnType()<<" :"<<endl;
-			h<<spaceIndent<<spaceIndent<<"@since"<<endl;
-			h<<spaceIndent<<spaceIndent<<"@author"<<endl;
-			h<<spaceIndent<<spaceIndent<<"\"\"\""<<endl;
+			h<<m_indentation<<m_indentation<<"@return " + op->getTypeName()<<" :"<<m_endl;
+			h<<m_indentation<<m_indentation<<"@since"<<m_endl;
+			h<<m_indentation<<m_indentation<<"@author"<<m_endl;
+			h<<m_indentation<<m_indentation<<"\"\"\""<<m_endl;
 		}
-		h<<spaceIndent<<spaceIndent<<"pass"<<endl<<endl;
+		h<<m_indentation<<m_indentation<<"pass"<<m_endl<<m_endl;
 
 	}//end for
 }
 
+/**
+ * returns "Python"
+ */
 QString PythonWriter::getLanguage() {
-        return "Python";
+	return "Python";
 }
 
+/**
+ * checks whether type is "PythonWriter"
+ *
+ * @param type
+ */
 bool PythonWriter::isType (QString & type)
 {
    if(type == "PythonWriter")
-        return true;
+	return true;
    return false;
 }
 
+const QStringList PythonWriter::reservedKeywords() const {
 
+  static QStringList keywords;
 
+  if (keywords.isEmpty()) {
+    keywords << "abs"
+	     << "and"
+	     << "apply"
+	     << "ArithmeticError"
+	     << "assert"
+	     << "AssertionError"
+	     << "AttributeError"
+	     << "break"
+	     << "buffer"
+	     << "callable"
+	     << "chr"
+	     << "class"
+	     << "classmethod"
+	     << "cmp"
+	     << "coerce"
+	     << "compile"
+	     << "complex"
+	     << "continue"
+	     << "def"
+	     << "del"
+	     << "delattr"
+	     << "DeprecationWarning"
+	     << "dict"
+	     << "dir"
+	     << "divmod"
+	     << "elif"
+	     << "Ellipsis"
+	     << "else"
+	     << "EnvironmentError"
+	     << "EOFError"
+	     << "eval"
+	     << "except"
+	     << "Exception"
+	     << "exec"
+	     << "execfile"
+	     << "file"
+	     << "filter"
+	     << "finally"
+	     << "float"
+	     << "FloatingPointError"
+	     << "for"
+	     << "from"
+	     << "getattr"
+	     << "global"
+	     << "globals"
+	     << "hasattr"
+	     << "hash"
+	     << "hex"
+	     << "id"
+	     << "if"
+	     << "import"
+	     << "__import__"
+	     << "ImportError"
+	     << "in"
+	     << "IndentationError"
+	     << "IndexError"
+	     << "input"
+	     << "int"
+	     << "intern"
+	     << "IOError"
+	     << "is"
+	     << "isinstance"
+	     << "issubclass"
+	     << "iter"
+	     << "KeyboardInterrupt"
+	     << "KeyError"
+	     << "lambda"
+	     << "len"
+	     << "list"
+	     << "locals"
+	     << "long"
+	     << "LookupError"
+	     << "map"
+	     << "max"
+	     << "MemoryError"
+	     << "min"
+	     << "NameError"
+	     << "None"
+	     << "not"
+	     << "NotImplemented"
+	     << "NotImplementedError"
+	     << "object"
+	     << "oct"
+	     << "open"
+	     << "or"
+	     << "ord"
+	     << "OSError"
+	     << "OverflowError"
+	     << "OverflowWarning"
+	     << "pass"
+	     << "pow"
+	     << "print"
+	     << "property"
+	     << "raise"
+	     << "range"
+	     << "raw_input"
+	     << "reduce"
+	     << "ReferenceError"
+	     << "reload"
+	     << "repr"
+	     << "return"
+	     << "round"
+	     << "RuntimeError"
+	     << "RuntimeWarning"
+	     << "setattr"
+	     << "slice"
+	     << "StandardError"
+	     << "staticmethod"
+	     << "StopIteration"
+	     << "str"
+	     << "super"
+	     << "SyntaxError"
+	     << "SyntaxWarning"
+	     << "SystemError"
+	     << "SystemExit"
+	     << "TabError"
+	     << "try"
+	     << "tuple"
+	     << "type"
+	     << "TypeError"
+	     << "UnboundLocalError"
+	     << "unichr"
+	     << "unicode"
+	     << "UnicodeError"
+	     << "UserWarning"
+	     << "ValueError"
+	     << "vars"
+	     << "Warning"
+	     << "while"
+	     << "WindowsError"
+	     << "xrange"
+	     << "yield"
+	     << "ZeroDivisionError"
+	     << "zip";
+  }
+
+  return keywords;
+}
 
 #include "pythonwriter.moc"
