@@ -1,6 +1,4 @@
 /***************************************************************************
-                          sqlwriter.cpp  -  description
-                             -------------------
     begin                : 10.02.2003
     copyright            : (C) 2003 Nikolaus Gradwohl
     email                : guru@local-guru.net
@@ -32,7 +30,8 @@
 #include "../attribute.h"
 
 SQLWriter::SQLWriter( UMLDoc *parent, const char *name )
-		:SimpleCodeGenerator( parent, name) {}
+	:SimpleCodeGenerator( parent, name) {
+}
 
 SQLWriter::~SQLWriter() {}
 
@@ -70,18 +69,18 @@ void SQLWriter::writeClass(UMLClassifier *c) {
 	if(!str.isEmpty()) {
 		str.replace(QRegExp("%filename%"),fileName);
 		str.replace(QRegExp("%filepath%"),file.name());
-		sql<<str<<endl;
+		sql<<str<<m_endl;
 	}
 
 	//Write class Documentation if there is somthing or if force option
 	if(forceDoc() || !c->getDoc().isEmpty()) {
-		sql << m_newLineEndingChars << "--" << m_newLineEndingChars;
-		sql<<"-- TABLE: "<<classname<<endl;
+		sql << m_endl << "--" << m_endl;
+		sql<<"-- TABLE: "<<classname<<m_endl;
 		sql<<formatDoc(c->getDoc(),"-- ");
-		sql << "--  " << m_newLineEndingChars << m_newLineEndingChars;
+		sql << "--  " << m_endl << m_endl;
 	}
 
-	sql << "CREATE TABLE "<< classname << " ( " << endl;
+	sql << "CREATE TABLE "<< classname << " ( " << m_endl;
 
 	if(myClass)
 		writeAttributes(myClass,sql);
@@ -89,17 +88,18 @@ void SQLWriter::writeClass(UMLClassifier *c) {
 	UMLAssociationList aggregations = c->getAggregations();
 	if( forceSections() || !aggregations.isEmpty() ) {
 		for(UMLAssociation* a = aggregations.first(); a; a = aggregations.next()) {
-			if( a->getObject(A)->getID() != c->getID() ) {
+			if( a->getObject(Uml::A)->getID() != c->getID() ) {
 
-				sql << m_indentation << "," << m_newLineEndingChars << m_indentation
+				sql << m_indentation << "," << m_endl << m_indentation
 					 << "CONSTRAINT " << a->getName() << " FOREIGN KEY ("
-					 << a->getRoleName(B) << ") REFERENCES "
-					 << a->getObject(A)->getName() << " (" << a->getRoleName(A) << ")";
+					 << a->getRoleName(Uml::B) << ") REFERENCES "
+					 << a->getObject(Uml::A)->getName()
+					 << " (" << a->getRoleName(Uml::A) << ")";
 			}
 		}
 	}
 
-	sql << m_newLineEndingChars << ");" << m_newLineEndingChars;
+	sql << m_endl << ");" << m_endl;
 
 	file.close();
 	emit codeGenerated(c, true);
@@ -107,15 +107,14 @@ void SQLWriter::writeClass(UMLClassifier *c) {
 
 
 void SQLWriter::writeAttributes(UMLClass *c, QTextStream &sql) {
-	UMLAttributeList* atl;
 	UMLAttributeList atpub, atprot, atpriv;
 	atpub.setAutoDelete(false);
 	atprot.setAutoDelete(false);
 	atpriv.setAutoDelete(false);
 
 	//sort attributes by scope and see if they have a default value
-	atl = c->getFilteredAttributeList();
-	for(UMLAttribute* at=atl->first(); at ; at=atl->next()) {
+	UMLAttributeList atl = c->getAttributeList();
+	for(UMLAttribute* at=atl.first(); at ; at=atl.next()) {
 		switch(at->getScope()) {
 		case Uml::Public:
 			atpub.append(at);
@@ -171,9 +170,9 @@ void SQLWriter::printAttributes(QTextStream& sql, UMLAttributeList attributeList
 		// print documentation/comment of last attribute at end of line
 		if (attrDoc.isEmpty() == false)
 		{
-			sql << " -- " << attrDoc << endl;
+			sql << " -- " << attrDoc << m_endl;
 		} else {
-			sql << endl;
+			sql << m_endl;
 		}
 
 		// write the attribute
@@ -187,16 +186,16 @@ void SQLWriter::printAttributes(QTextStream& sql, UMLAttributeList attributeList
 	// print documentation/comment at end of line
 	if (attrDoc.isEmpty() == false)
 	{
-		sql << " -- " << attrDoc << endl;
+		sql << " -- " << attrDoc << m_endl;
 	} else {
-		sql << endl;
+		sql << m_endl;
 	}
 
 	return;
 }
 
 QString SQLWriter::getLanguage() {
-        return "SQL";
+	return "SQL";
 }
 
 /**
@@ -206,164 +205,156 @@ QString SQLWriter::getLanguage() {
  */
 bool SQLWriter::isType (QString & type)
 {
-   if(type == "SQLWriter")
-        return true;
-   return false;
+	if(type == "SQLWriter")
+		return true;
+	return false;
 }
 
-/**
- * List of reserved keywords for this code generator.
- *
- * Just add new keywords, then mark all lines and
- * pipe it through the external 'sort' program.
- */
-static const char *ReservedWords[] = {
-  "access",
-  "add",
-  "all",
-  "alter",
-  "analyze",
-  "and",
-  "any",
-  "as",
-  "asc",
-  "audit",
-  "begin",
-  "between",
-  "boolean",
-  "by",
-  "char",
-  "character",
-  "check",
-  "cluster",
-  "column",
-  "comment",
-  "commit",
-  "compress",
-  "connect",
-  "create",
-  "current",
-  "cursor",
-  "date",
-  "decimal",
-  "default",
-  "delete",
-  "desc",
-  "distinct",
-  "drop",
-  "else",
-  "elsif",
-  "end",
-  "escape",
-  "exception",
-  "exclusive",
-  "execute",
-  "exists",
-  "explain",
-  "false",
-  "file",
-  "float",
-  "for",
-  "from",
-  "function",
-  "grant",
-  "group",
-  "having",
-  "identified",
-  "if",
-  "immediate",
-  "in",
-  "increment",
-  "index",
-  "initial",
-  "insert",
-  "integer",
-  "intersect",
-  "into",
-  "is",
-  "level",
-  "like",
-  "lock",
-  "long",
-  "loop",
-  "maxextents",
-  "minus",
-  "mlslabel",
-  "mode",
-  "modify",
-  "noaudit",
-  "nocompress",
-  "not",
-  "nowait",
-  "null",
-  "number",
-  "of",
-  "offline",
-  "on",
-  "online",
-  "option",
-  "or",
-  "order",
-  "out",
-  "pctfree",
-  "prior",
-  "privileges",
-  "procedure",
-  "public",
-  "raw",
-  "rename",
-  "resource",
-  "return",
-  "revoke",
-  "rollback",
-  "row",
-  "rowid",
-  "rowlabel",
-  "rownum",
-  "rows",
-  "savepoint",
-  "select",
-  "session",
-  "set",
-  "share",
-  "size",
-  "smallint",
-  "some",
-  "start",
-  "successful",
-  "synonym",
-  "sysdate",
-  "table",
-  "then",
-  "to",
-  "trigger",
-  "true",
-  "truncate",
-  "type",
-  "uid",
-  "union",
-  "unique",
-  "update",
-  "user",
-  "using",
-  "validate",
-  "values",
-  "varchar",
-  "varchar2",
-  "varray",
-  "view",
-  "whenever",
-  "where",
-  "with",
-  NULL
-};
+const QStringList SQLWriter::reservedKeywords() const {
 
-/**
- * get list of reserved keywords
- */
-const char **
-SQLWriter::getReservedKeywords() {
-  return ReservedWords;
+  static QStringList keywords;
+
+  if (keywords.isEmpty()) {
+    keywords << "access"
+	     << "add"
+	     << "all"
+	     << "alter"
+	     << "analyze"
+	     << "and"
+	     << "any"
+	     << "as"
+	     << "asc"
+	     << "audit"
+	     << "begin"
+	     << "between"
+	     << "boolean"
+	     << "by"
+	     << "char"
+	     << "character"
+	     << "check"
+	     << "cluster"
+	     << "column"
+	     << "comment"
+	     << "commit"
+	     << "compress"
+	     << "connect"
+	     << "create"
+	     << "current"
+	     << "cursor"
+	     << "date"
+	     << "decimal"
+	     << "default"
+	     << "delete"
+	     << "desc"
+	     << "distinct"
+	     << "drop"
+	     << "else"
+	     << "elsif"
+	     << "end"
+	     << "escape"
+	     << "exception"
+	     << "exclusive"
+	     << "execute"
+	     << "exists"
+	     << "explain"
+	     << "false"
+	     << "file"
+	     << "float"
+	     << "for"
+	     << "from"
+	     << "function"
+	     << "grant"
+	     << "group"
+	     << "having"
+	     << "identified"
+	     << "if"
+	     << "immediate"
+	     << "in"
+	     << "increment"
+	     << "index"
+	     << "initial"
+	     << "insert"
+	     << "integer"
+	     << "intersect"
+	     << "into"
+	     << "is"
+	     << "level"
+	     << "like"
+	     << "lock"
+	     << "long"
+	     << "loop"
+	     << "maxextents"
+	     << "minus"
+	     << "mlslabel"
+	     << "mode"
+	     << "modify"
+	     << "noaudit"
+	     << "nocompress"
+	     << "not"
+	     << "nowait"
+	     << "null"
+	     << "number"
+	     << "of"
+	     << "offline"
+	     << "on"
+	     << "online"
+	     << "option"
+	     << "or"
+	     << "order"
+	     << "out"
+	     << "pctfree"
+	     << "prior"
+	     << "privileges"
+	     << "procedure"
+	     << "public"
+	     << "raw"
+	     << "rename"
+	     << "resource"
+	     << "return"
+	     << "revoke"
+	     << "rollback"
+	     << "row"
+	     << "rowid"
+	     << "rowlabel"
+	     << "rownum"
+	     << "rows"
+	     << "savepoint"
+	     << "select"
+	     << "session"
+	     << "set"
+	     << "share"
+	     << "size"
+	     << "smallint"
+	     << "some"
+	     << "start"
+	     << "successful"
+	     << "synonym"
+	     << "sysdate"
+	     << "table"
+	     << "then"
+	     << "to"
+	     << "trigger"
+	     << "true"
+	     << "truncate"
+	     << "type"
+	     << "uid"
+	     << "union"
+	     << "unique"
+	     << "update"
+	     << "user"
+	     << "using"
+	     << "validate"
+	     << "values"
+	     << "varchar"
+	     << "varchar2"
+	     << "varray"
+	     << "view"
+	     << "whenever"
+	     << "where"
+	     << "with";
+  }
+
+  return keywords;
 }
 
 #include "sqlwriter.moc"
