@@ -1,6 +1,4 @@
 /***************************************************************************
-                          xmlschemawriter.cpp  -  description
-                             -------------------
     copyright            : (C) 2003 Brian Thomas brian.thomas@gsfc.nasa.gov
 ***************************************************************************/
 
@@ -30,7 +28,7 @@
 #include "../umlnamespace.h"
 
 // Constructor
-XMLSchemaWriter::XMLSchemaWriter( UMLDoc *doc, const char *name ) 
+XMLSchemaWriter::XMLSchemaWriter( UMLDoc *doc, const char *name )
    : SimpleCodeGenerator ( doc, name )
 {
 
@@ -38,24 +36,29 @@ XMLSchemaWriter::XMLSchemaWriter( UMLDoc *doc, const char *name )
 	packageNamespaceURI = "http://foo.example.com/";
 	schemaNamespaceTag = "xs";
 	schemaNamespaceURI = "http://www.w3.org/2001/XMLSchema";
-	indent = m_indentation;
-	indentLevel = 0;
-	startline = m_newLineEndingChars;
 }
 
 // form of..."the Destructor"!!
 XMLSchemaWriter::~XMLSchemaWriter() {
 }
 
+/**
+ * returns "XMLSchema"
+ */
 QString XMLSchemaWriter::getLanguage() {
-        return "XMLSchema";
+	return "XMLSchema";
 }
 
+/**
+ * checks whether type is "XMLSchemaWriter"
+ *
+ * @param type
+ */
 bool XMLSchemaWriter::isType (QString & type)
 {
-   if(type == "XMLSchemaWriter")
-        return true;
-   return false;
+	if(type == "XMLSchemaWriter")
+		return true;
+	return false;
 }
 
 // main method for invoking..
@@ -85,14 +88,14 @@ void XMLSchemaWriter::writeClass(UMLClassifier *c)
 	QTextStream XMLschema(&file);
 
 	// set package namespace tag appropriately
-        if(!c->getPackage().isEmpty())
+ 	if(!c->getPackage().isEmpty())
 	       packageNamespaceTag = c->getPackage();
 
 	// START WRITING
 
 	// 0. FIRST THING: open the xml processing instruction. This MUST be
 	// the first thing in the file
-	XMLschema<<"<?xml version=\"1.0\"?>"<<endl;
+	XMLschema<<"<?xml version=\"1.0\"?>"<<m_endl;
 
 	// 1. create the header
 	QString headerText = getHeadingFile(".xsd");
@@ -101,30 +104,30 @@ void XMLSchemaWriter::writeClass(UMLClassifier *c)
 		headerText.replace(QRegExp("%filepath%"),file.name());
 	}
 	if(!headerText.isEmpty())
-		XMLschema<<headerText<<endl;
+		XMLschema<<headerText<<m_endl;
 
 	// 2. Open schema element node with appropriate namespace decl
 	XMLschema<<"<"<<makeSchemaTag("schema");
 	// common namespaces we know will be in the file..
-	XMLschema<<" targetNamespace=\""<<packageNamespaceURI+packageNamespaceTag<<"\""<<endl;
+	XMLschema<<" targetNamespace=\""<<packageNamespaceURI+packageNamespaceTag<<"\""<<m_endl;
 	XMLschema<<" xmlns:"<<schemaNamespaceTag<<"=\""<<schemaNamespaceURI<<"\"";
 	XMLschema<<" xmlns:"<<packageNamespaceTag<<"=\""<<packageNamespaceURI+packageNamespaceTag<<"\"";
 
-	XMLschema<<">"<<endl; // close opening declaration
+	XMLschema<<">"<<m_endl; // close opening declaration
 
-	indentLevel++;
+	m_indentLevel++;
 
-        // 3? IMPORT statements -- do we need to do anything here? I suppose if
+	// 3? IMPORT statements -- do we need to do anything here? I suppose if
 	// our document has more than one package, which is possible, we are missing
 	// the correct import statements. Leave that for later at this time.
-        /*
-        //only import classes in a different package as this class
-        UMLClassifierList imports;
-        findObjectsRelated(c,imports);
-        for(UMLClassifier *con = imports.first(); con ; con = imports.next())
-                if(con->getPackage() != c->getPackage())
-                        XMLschema<<"import "<<con->getPackage()<<"."<<cleanName(con->getName())<<";"<<endl;
-        */
+	/*
+	//only import classes in a different package as this class
+	UMLClassifierList imports;
+	findObjectsRelated(c,imports);
+	for(UMLClassifier *con = imports.first(); con ; con = imports.next())
+		if(con->getPackage() != c->getPackage())
+			XMLschema<<"import "<<con->getPackage()<<"."<<cleanName(con->getName())<<";"<<m_endl;
+	*/
 
 	// 4. BODY of the schema.
 	// start the writing by sending this classifier, the "root" for this particular
@@ -133,12 +136,12 @@ void XMLSchemaWriter::writeClass(UMLClassifier *c)
 	writeClassifier(c, XMLschema);
 
 	// 5. What remains is to make the root node declaration
-	XMLschema<<endl;
+	XMLschema<<m_endl;
 	writeElementDecl(getElementName(c), getElementTypeName(c), XMLschema);
 
 	// 6. Finished: now we may close schema decl
-	indentLevel--;
-	XMLschema<<getIndent()<<"</"<<makeSchemaTag("schema")<<">"<<endl; // finished.. close schema node
+	m_indentLevel--;
+	XMLschema<<getIndent()<<"</"<<makeSchemaTag("schema")<<">"<<m_endl; // finished.. close schema node
 
 	// bookeeping for code generation
 	emit codeGenerated(c, true);
@@ -151,15 +154,15 @@ void XMLSchemaWriter::writeClass(UMLClassifier *c)
 	writtenClassifiers.clear();
 }
 
-void XMLSchemaWriter::writeElementDecl( QString elementName, QString elementTypeName, QTextStream &XMLschema)
+void XMLSchemaWriter::writeElementDecl( const QString &elementName, const QString &elementTypeName, QTextStream &XMLschema)
 {
-        if(forceDoc())
+	if(forceDoc())
 		writeComment(elementName+" is the root element, declared here.", XMLschema);
 
 	XMLschema<<getIndent()<<"<"<<makeSchemaTag("element")
 		<<" name=\""<<elementName<<"\""
 		<<" type=\""<<makePackageTag(elementTypeName)<<"\""
-		<<"/>"<<endl;
+		<<"/>"<<m_endl;
 
 }
 
@@ -170,10 +173,10 @@ void XMLSchemaWriter::writeClassifier (UMLClassifier *c, QTextStream &XMLschema)
 	if(hasBeenWritten(c))
 		return;
 
-	XMLschema<<endl;
+	XMLschema<<m_endl;
 
 	// write documentation for class, if any, first
-        if(forceDoc() || !c->getDoc().isEmpty())
+	if(forceDoc() || !c->getDoc().isEmpty())
 		writeComment(c->getDoc(),XMLschema);
 
 	if(c->getAbstract() || dynamic_cast<UMLInterface*>(c) )
@@ -185,14 +188,14 @@ void XMLSchemaWriter::writeClassifier (UMLClassifier *c, QTextStream &XMLschema)
 
 UMLAttributeList XMLSchemaWriter::findAttributes (UMLClassifier *c)
 {
-        // sort attributes by Scope
-        UMLAttributeList attribs;
-        attribs.setAutoDelete(false);
+	// sort attributes by Scope
+	UMLAttributeList attribs;
+	attribs.setAutoDelete(false);
 
 	UMLClass * myClass = dynamic_cast<UMLClass*>(c);
 	if(myClass) {
-	        UMLAttributeList *atl = myClass->getFilteredAttributeList();
-	        for(UMLAttribute *at=atl->first(); at ; at=atl->next()) {
+	        UMLAttributeList atl = myClass->getAttributeList();
+	        for(UMLAttribute *at=atl.first(); at ; at=atl.next()) {
 	                switch(at->getScope())
 	                {
 	                        case Uml::Public:
@@ -236,7 +239,7 @@ void XMLSchemaWriter::writeAbstractClassifier (UMLClassifier *c, QTextStream &XM
 	{
 
 		QString elementName = getElementName(c);
-        	UMLAttributeList attribs = findAttributes(c);
+		UMLAttributeList attribs = findAttributes(c);
 		QStringList attribGroups = findAttributeGroups(c);
 
 		writeAttributeGroupDecl(elementName, attribs, XMLschema);
@@ -261,23 +264,23 @@ void XMLSchemaWriter::writeGroupClassifierDecl (UMLClassifier *c,
 	QString elementTypeName = getElementGroupTypeName(c);
 
 	// start Writing node but only if it has subclasses? Nah..right now put in empty group
-	XMLschema<<getIndent()<<"<"<<makeSchemaTag("group")<<" name=\""<<elementTypeName<<"\">"<<endl;
-	indentLevel++;
+	XMLschema<<getIndent()<<"<"<<makeSchemaTag("group")<<" name=\""<<elementTypeName<<"\">"<<m_endl;
+	m_indentLevel++;
 
-	XMLschema<<getIndent()<<"<"<<makeSchemaTag("choice")<<">"<<endl;
-	indentLevel++;
+	XMLschema<<getIndent()<<"<"<<makeSchemaTag("choice")<<">"<<m_endl;
+	m_indentLevel++;
 
 	for(UMLClassifier *classifier = subclasses.first(); classifier; classifier = subclasses.next()) {
 		writeAssociationRoleDecl(classifier, "1", XMLschema);
 	}
 
-	indentLevel--;
-	XMLschema<<getIndent()<<"</"<<makeSchemaTag("choice")<<">"<<endl;
+	m_indentLevel--;
+	XMLschema<<getIndent()<<"</"<<makeSchemaTag("choice")<<">"<<m_endl;
 
-	indentLevel--;
+	m_indentLevel--;
 
 	// finish node
-	XMLschema<<getIndent()<<"</"<<makeSchemaTag("group")<<">"<<endl;
+	XMLschema<<getIndent()<<"</"<<makeSchemaTag("group")<<">"<<m_endl;
 
 }
 
@@ -311,25 +314,25 @@ void XMLSchemaWriter::writeComplexTypeClassifierDecl (UMLClassifier *c,
 	if(hasAssociations || hasAttributes || hasSuperclass)
 	{
 
-		XMLschema<<">"<<endl;
+		XMLschema<<">"<<m_endl;
 
-		indentLevel++;
+		m_indentLevel++;
 
 		if(hasSuperclass)
 		{
 			QString superClassName = getElementTypeName(superclasses.first());
-			XMLschema<<getIndent()<<"<"<<makeSchemaTag("complexContent")<<">"<<endl;
+			XMLschema<<getIndent()<<"<"<<makeSchemaTag("complexContent")<<">"<<m_endl;
 
 			//PROBLEM: we only treat ONE superclass for inheritence.. bah.
-			indentLevel++;
+			m_indentLevel++;
 			XMLschema<<getIndent()<<"<"<<makeSchemaTag("extension")<<" base=\""<<makePackageTag(superClassName)
 				<<"\"";
 			if(hasAssociations || hasAttributes )
-				XMLschema<<">"<<endl;
+				XMLschema<<">"<<m_endl;
 			else
-				XMLschema<<"/>"<<endl;
+				XMLschema<<"/>"<<m_endl;
 
-			indentLevel++;
+			m_indentLevel++;
 		}
 
 		if(hasAssociations)
@@ -342,8 +345,8 @@ void XMLSchemaWriter::writeComplexTypeClassifierDecl (UMLClassifier *c,
 			didFirstOne = writeAssociationDecls(compositions, false, didFirstOne, c->getID(), XMLschema);
 
 			if (didFirstOne) {
-				indentLevel--;
-				XMLschema<<getIndent()<<"</"<<makeSchemaTag("sequence")<<">"<<endl;
+				m_indentLevel--;
+				XMLschema<<getIndent()<<"</"<<makeSchemaTag("sequence")<<">"<<m_endl;
 			}
 		}
 
@@ -354,26 +357,26 @@ void XMLSchemaWriter::writeComplexTypeClassifierDecl (UMLClassifier *c,
 			writeAttributeDecls(attribs, XMLschema);
 			for(uint i= 0; i < attribGroups.count(); i++)
 				XMLschema<<getIndent()<<"<"<<makeSchemaTag("attributeGroup")<<" ref=\""
-					<<makePackageTag(attribGroups[i])<<"\"/>"<<endl;
+					<<makePackageTag(attribGroups[i])<<"\"/>"<<m_endl;
 		}
 
 		if(hasSuperclass)
 		{
-			indentLevel--;
+			m_indentLevel--;
 
 			if(hasAssociations || hasAttributes )
-				XMLschema<<getIndent()<<"</"<<makeSchemaTag("extension")<<">"<<endl;
+				XMLschema<<getIndent()<<"</"<<makeSchemaTag("extension")<<">"<<m_endl;
 
-			indentLevel--;
-			XMLschema<<getIndent()<<"</"<<makeSchemaTag("complexContent")<<">"<<endl;
+			m_indentLevel--;
+			XMLschema<<getIndent()<<"</"<<makeSchemaTag("complexContent")<<">"<<m_endl;
 		}
 
 		// close this element decl
-		indentLevel--;
-		XMLschema<<getIndent()<<"</"<<makeSchemaTag("complexType")<<">"<<endl;
+		m_indentLevel--;
+		XMLschema<<getIndent()<<"</"<<makeSchemaTag("complexType")<<">"<<m_endl;
 
 	} else
-		XMLschema<<"/>"<<endl; // empty node. just close this element decl
+		XMLschema<<"/>"<<m_endl; // empty node. just close this element decl
 
 }
 
@@ -422,8 +425,8 @@ QStringList XMLSchemaWriter::findAttributeGroups (UMLClassifier *c)
 			// only classes have attributes..
 			UMLClass * myClass = dynamic_cast<UMLClass*>(classifier);
 			if(myClass) {
-				UMLAttributeList* attribs = myClass->getFilteredAttributeList();
-				if (attribs->count() > 0)
+				UMLAttributeList attribs = myClass->getAttributeList();
+				if (attribs.count() > 0)
 					list.append(getElementName(classifier)+"AttribGroupType");
 			}
 		}
@@ -489,7 +492,7 @@ void XMLSchemaWriter::writeAttributeDecl(UMLAttribute *attrib, QTextStream &XMLs
 
 	XMLschema<<getIndent()<<"<"<<makeSchemaTag("attribute")
        		<<" name=\""<<cleanName(attrib->getName())<<"\""
-               	<<" type=\""<<typeName<<"\"";
+		<<" type=\""<<typeName<<"\"";
 
 	// default value?
 	if(!initialValue.isEmpty())
@@ -503,11 +506,11 @@ void XMLSchemaWriter::writeAttributeDecl(UMLAttribute *attrib, QTextStream &XMLs
 	}
 
 	// finish decl
-	XMLschema<<"/>"<<endl;
+	XMLschema<<"/>"<<m_endl;
 
 }
 
-void XMLSchemaWriter::writeAttributeGroupDecl (QString elementName, UMLAttributeList &attribs, QTextStream &XMLschema )
+void XMLSchemaWriter::writeAttributeGroupDecl (const QString &elementName, UMLAttributeList &attribs, QTextStream &XMLschema )
 {
 
 	if (attribs.count()> 0) {
@@ -516,23 +519,23 @@ void XMLSchemaWriter::writeAttributeGroupDecl (QString elementName, UMLAttribute
 		writeComment("attributes for element "+elementName,XMLschema);
 
 		// open attribute group
-		XMLschema<<getIndent()<<"<"<<makeSchemaTag("attributeGroup")<<" name=\""<<elementName+"AttribGroupType"<<"\">"<<endl;
+		XMLschema<<getIndent()<<"<"<<makeSchemaTag("attributeGroup")<<" name=\""<<elementName+"AttribGroupType"<<"\">"<<m_endl;
 
-		indentLevel++;
+		m_indentLevel++;
 
 		for( UMLAttribute *at=attribs.first(); at; at=attribs.next())
 		{
 			writeAttributeDecl(at,XMLschema);
 		}
 
-		indentLevel--;
+		m_indentLevel--;
 
 		// close attrib group node
-		XMLschema<<getIndent()<<"</"<<makeSchemaTag("attributeGroup")<<">"<<endl;
+		XMLschema<<getIndent()<<"</"<<makeSchemaTag("attributeGroup")<<">"<<m_endl;
 	}
 }
 
-void XMLSchemaWriter::writeComment(QString comment, QTextStream &XMLschema)
+void XMLSchemaWriter::writeComment( const QString &comment, QTextStream &XMLschema )
 {
 	// in the case we have several line comment..
 	// NOTE: this part of the method has the problem of adopting UNIX newline,
@@ -540,16 +543,16 @@ void XMLSchemaWriter::writeComment(QString comment, QTextStream &XMLschema)
 	QString indent = getIndent();
 	XMLschema<<indent<<"<!-- ";
 	if (comment.contains(QRegExp("\n"))) {
-		XMLschema<<endl;
+		XMLschema<<m_endl;
 		QStringList lines = QStringList::split( "\n", comment);
 		for(uint i= 0; i < lines.count(); i++)
-			XMLschema<<indent<<"     "<<lines[i]<<endl;
+			XMLschema<<indent<<"     "<<lines[i]<<m_endl;
 
-		XMLschema<<indent<<"-->"<<endl;
+		XMLschema<<indent<<"-->"<<m_endl;
 	} else {
 		// this should be more fancy in the future, breaking it up into 80 char
 		// lines so that it doesnt look too bad
-		XMLschema<<comment<<" -->"<<endl;
+		XMLschema<<comment<<" -->"<<m_endl;
 	}
 }
 
@@ -560,7 +563,7 @@ void XMLSchemaWriter::writeComment(QString comment, QTextStream &XMLschema)
 // badly for someone using a plain association between 2 different classes. THAT should
 // be done, but isnt yet (this is why I have left role b code in for now). -b.t.
 bool XMLSchemaWriter::writeAssociationDecls(UMLAssociationList associations,
-		bool noRoleNameOK, bool didFirstOne, int id, QTextStream &XMLschema)
+		bool noRoleNameOK, bool didFirstOne, Uml::IDType id, QTextStream &XMLschema)
 {
 
 	if( !associations.isEmpty() )
@@ -572,10 +575,10 @@ bool XMLSchemaWriter::writeAssociationDecls(UMLAssociationList associations,
 			// it may seem counter intuitive, but you want to insert the role of the
 			// *other* class into *this* class.
 
-			if (a->getRoleId(A) == id && a->getVisibility(B) != Uml::Private)
+			if (a->getObjectId(Uml::A) == id && a->getVisibility(Uml::B) != Uml::Private)
 				printRoleB = true;
 
-			if (a->getRoleId(B) == id && a->getVisibility(A) != Uml::Private)
+			if (a->getObjectId(Uml::B) == id && a->getVisibility(Uml::A) != Uml::Private)
 				printRoleA = true;
 
 			// First: we insert documentaion for association IF it has either role
@@ -587,8 +590,8 @@ bool XMLSchemaWriter::writeAssociationDecls(UMLAssociationList associations,
 			if(!didFirstOne && (printRoleA || printRoleB))
 			{
 				didFirstOne = true;
-				XMLschema<<getIndent()<<"<"<<makeSchemaTag("sequence")<<">"<<endl;
-				indentLevel++;
+				XMLschema<<getIndent()<<"<"<<makeSchemaTag("sequence")<<">"<<m_endl;
+				m_indentLevel++;
 			}
 
 			// print RoleB decl
@@ -611,12 +614,12 @@ bool XMLSchemaWriter::writeAssociationDecls(UMLAssociationList associations,
 			// print RoleA decl
 			if (printRoleA)
 			{
-				UMLClassifier *classifierA = dynamic_cast<UMLClassifier*>(a->getObject(A));
+				UMLClassifier *classifierA = dynamic_cast<UMLClassifier*>(a->getObject(Uml::A));
 				if (classifierA) {
 					// ONLY write out IF there is a rolename given
 					// otherwise its not meant to be declared
-					if (!a->getRoleName(A).isEmpty() || noRoleNameOK )
-						writeAssociationRoleDecl(classifierA, a->getMulti(A), XMLschema);
+					if (!a->getRoleName(Uml::A).isEmpty() || noRoleNameOK )
+						writeAssociationRoleDecl(classifierA, a->getMulti(Uml::A), XMLschema);
 				}
 			}
 		}
@@ -628,26 +631,26 @@ bool XMLSchemaWriter::writeAssociationDecls(UMLAssociationList associations,
 
 UMLObjectList XMLSchemaWriter::findChildObjsInAssociations (UMLClassifier *c, UMLAssociationList associations)
 {
-	int id = c->getID();
+	Uml::IDType id = c->getID();
 	UMLObjectList list;
 	for(UMLAssociation *a = associations.first(); a; a = associations.next())
 	{
-		if (a->getRoleId(A) == id
-				&& a->getVisibility(B) != Uml::Private
-				&& !a->getRoleName(B).isEmpty()
+		if (a->getObjectId(Uml::A) == id
+				&& a->getVisibility(Uml::B) != Uml::Private
+				&& !a->getRoleName(Uml::B).isEmpty()
 		   )
-			list.append(a->getObject(B));
+			list.append(a->getObject(Uml::B));
 
-		if (a->getRoleId(B) == id
-				&& a->getVisibility(A) != Uml::Private
-				&& !a->getRoleName(A).isEmpty()
+		if (a->getObjectId(Uml::B) == id
+				&& a->getVisibility(Uml::A) != Uml::Private
+				&& !a->getRoleName(Uml::A).isEmpty()
 		   )
-			list.append(a->getObject(A));
+			list.append(a->getObject(Uml::A));
 	}
 	return list;
 }
 
-void XMLSchemaWriter::writeAssociationRoleDecl( UMLClassifier *c, QString multi, QTextStream &XMLschema)
+void XMLSchemaWriter::writeAssociationRoleDecl( UMLClassifier *c, const QString &multi, QTextStream &XMLschema)
 {
 
 	bool isAbstract = c->getAbstract();
@@ -737,7 +740,7 @@ void XMLSchemaWriter::writeAssociationRoleDecl( UMLClassifier *c, QString multi,
 		XMLschema<<" maxOccurs=\""<<maxOccurs<<"\"";
 
 	// tidy up the node
-	XMLschema<<"/>"<<endl;
+	XMLschema<<"/>"<<m_endl;
 
 }
 
@@ -752,7 +755,7 @@ QString XMLSchemaWriter::fixTypeName(QString string)
 	return string;
 }
 
-QString XMLSchemaWriter::fixInitialStringDeclValue(QString value, QString type)
+QString XMLSchemaWriter::fixInitialStringDeclValue(QString value, const QString &type)
 {
 	// check for strings only
 	if (!value.isEmpty() && type == "xs:string") {
@@ -764,24 +767,8 @@ QString XMLSchemaWriter::fixInitialStringDeclValue(QString value, QString type)
 	return value;
 }
 
-QString XMLSchemaWriter::lowerFirstLetterCase(QString string)
-{
-	QChar firstChar = string.at(0);
-	string.replace( 0, 1, firstChar.lower());
-	return string;
-}
-
-QString XMLSchemaWriter::getIndent ()
-{
-	QString myIndent = "";
-	for (int i = 0 ; i < indentLevel ; i++)
-		myIndent.append(indent);
-	return myIndent;
-}
-
 QString XMLSchemaWriter::getElementName(UMLClassifier *c)
 {
-	// return cleanName(lowerFirstLetterCase(c->getName()));
 	return cleanName(c->getName());
 }
 
@@ -807,4 +794,30 @@ QString XMLSchemaWriter::makeSchemaTag (QString tagName) {
 	return tagName;
 }
 
+const QStringList XMLSchemaWriter::reservedKeywords() const {
+
+  static QStringList keywords;
+
+  if (keywords.isEmpty()) {
+    keywords << "ATTLIST"
+	     << "CDATA"
+	     << "DOCTYPE"
+	     << "ELEMENT"
+	     << "ENTITIES"
+	     << "ENTITY"
+	     << "ID"
+	     << "IDREF"
+	     << "IDREFS"
+	     << "NMTOKEN"
+	     << "NMTOKENS"
+	     << "NOTATION"
+	     << "PUBLIC"
+	     << "SHORTREF"
+	     << "SYSTEM"
+	     << "USEMAP";
+  }
+
+  return keywords;
+}
+
 #include "xmlschemawriter.moc"
