@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2003-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -12,6 +17,8 @@
 
 #include "classifier.h"
 
+class UMLObject;
+
 /**
  * This class contains the non-graphical information required for a UML
  * Datatype.
@@ -35,7 +42,7 @@ public:
 	 * @param name		The name of the Concept.
 	 * @param id		The unique id of the Concept.
 	 */
-	UMLDatatype(const QString & name = "", int id = -1);
+	UMLDatatype(const QString & name = "", Uml::IDType id = Uml::id_None);
 
 	/**
 	 * Standard deconstructor.
@@ -68,6 +75,26 @@ public:
 	 */
 	virtual bool isInterface();
 
+	/**
+	 * Set the origin type (in case of e.g. typedef)
+	 */
+	void setOriginType(UMLClassifier *origType);
+
+	/**
+	 * Get the origin type (in case of e.g. typedef)
+	 */
+	UMLClassifier * originType();
+
+	/**
+	 * Set the m_isRef flag (true when dealing with a pointer type)
+	 */
+	void setIsReference(bool isRef = true);
+
+	/**
+	 * Get the m_isRef flag.
+	 */
+	bool isReference();
+
 protected:
 	/**
 	 * Loads the <UML:DataType> XMI element (empty.)
@@ -78,6 +105,8 @@ protected:
 	 * Initializes key variables of the class.
 	 */
 	virtual void init(); // doesnt seem to be any reason for this to be public
+
+	bool m_isRef;
 };
 
 #endif // DATATYPE_H
