@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -31,7 +36,7 @@ ClassWizard::ClassWizard( UMLDoc * pDoc ) : KWizard( (QWidget*)pDoc -> parent(),
 	QString newName = name;
 	QString num = "";
 	int i = 0;
-	m_pClass = new UMLClass( newName, m_pDoc -> getUniqueID() );
+	m_pClass = new UMLClass( newName );
 	do {
 		m_pClass -> setName( newName );
 		pTemp = m_pDoc -> findUMLObject( newName );
@@ -52,11 +57,11 @@ void ClassWizard::setupPages() {
 	setHelpEnabled(m_pGenPage, false);
 
 	//Setup Attribute Page
-	m_pAttPage = new ClassifierListPage(this, m_pClass, m_pDoc, ot_Attribute);
+	m_pAttPage = new ClassifierListPage(this, m_pClass, m_pDoc, Uml::ot_Attribute);
 	addPage( m_pAttPage, i18n("Class Attributes") );
 
 	//Setup Operation Page
-	m_pOpPage = new ClassifierListPage(this, m_pClass, m_pDoc, ot_Operation);
+	m_pOpPage = new ClassifierListPage(this, m_pClass, m_pDoc, Uml::ot_Operation);
 	addPage( m_pOpPage, i18n("Class Operations") );
 }
 
@@ -91,13 +96,13 @@ void ClassWizard::accept() {
 	m_pDoc->signalUMLObjectCreated(m_pClass);
 
 	UMLListView *listView = UMLApp::app()->getListView();
-	UMLClassifierListItemList* attributes = m_pClass->getAttList();
-	for ( UMLClassifierListItem* attribute = attributes->first(); attribute; attribute = attributes->next() )  {
+	UMLClassifierListItemList attributes = m_pClass->getFilteredList(Uml::ot_Attribute);
+	for ( UMLClassifierListItem* attribute = attributes.first(); attribute; attribute = attributes.next() )  {
 		listView->childObjectAdded(attribute, m_pClass);
 	}
 
-	UMLClassifierListItemList operations = m_pClass->getOpList();
-	for ( UMLClassifierListItem* operation = operations.first(); operation; operation = operations.next() )  {
+	UMLOperationList operations = m_pClass->getOpList();
+	for ( UMLOperation* operation = operations.first(); operation; operation = operations.next() )  {
 		listView->childObjectAdded(operation, m_pClass);
 	}
 
