@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,10 +26,10 @@
 // app includes
 #include "codegenerationoptionspage.h"
 #include "codevieweroptionspage.h"
-#include "dialog_utils.h"
+#include "../dialog_utils.h"
 
 SettingsDlg::SettingsDlg( QWidget * parent, Settings::OptionState *state,
-			  QDict<GeneratorInfo> ldict, QString activeLanguage, CodeGenerator * gen)
+			  QString activeLanguage, CodeGenerator * gen)
 	: KDialogBase( IconList, i18n("Umbrello Setup"),
         Help | Default | Apply | Ok | Cancel, Ok, parent, 0, true, true ) {
 	m_bChangesApplied = false;
@@ -34,7 +39,7 @@ SettingsDlg::SettingsDlg( QWidget * parent, Settings::OptionState *state,
 	setupFontPage();
 	setupUIPage();
 	setupClassPage();
-	setupCodeGenPage(gen, ldict, activeLanguage);
+	setupCodeGenPage(gen, activeLanguage);
 	setupCodeViewerPage(state->codeViewerState);
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -99,7 +104,7 @@ void SettingsDlg::setupGeneralPage() {
 	// Set up undo setting
 	m_GeneralWidgets.miscGB = new QGroupBox( i18n("Miscellaneous"), page );
 
-	QGridLayout * miscLayout = new QGridLayout( m_GeneralWidgets.miscGB, 1, 1 );
+	QGridLayout * miscLayout = new QGridLayout( m_GeneralWidgets.miscGB, 2, 2 );
 	miscLayout -> setSpacing( spacingHint() );
 	miscLayout -> setMargin( fontMetrics().height() );
 
@@ -107,6 +112,14 @@ void SettingsDlg::setupGeneralPage() {
 	m_GeneralWidgets.undoCB -> setChecked( m_pOptionState->generalState.undo );
 	miscLayout -> addWidget( m_GeneralWidgets.undoCB, 0, 0 );
 
+	m_GeneralWidgets.tabdiagramsCB = new QCheckBox( i18n("Use tabbed diagrams"), m_GeneralWidgets.miscGB );
+	m_GeneralWidgets.tabdiagramsCB -> setChecked( m_pOptionState->generalState.tabdiagrams );
+	miscLayout -> addWidget( m_GeneralWidgets.tabdiagramsCB, 0, 1 );
+
+	m_GeneralWidgets.newcodegenCB = new QCheckBox( i18n("Use new C++/Java generators"), m_GeneralWidgets.miscGB );
+	m_GeneralWidgets.newcodegenCB -> setChecked( m_pOptionState->generalState.newcodegen );
+	miscLayout -> addWidget( m_GeneralWidgets.newcodegenCB, 1, 0 );
+
 	//setup autosave settings
 
 	m_GeneralWidgets.autosaveGB = new QGroupBox( i18n("Autosave"), page );
@@ -128,7 +141,7 @@ void SettingsDlg::setupGeneralPage() {
 
 	// 2004-05-17 Achim Spangler: Allow definition of Suffix for autosave
 	// ( default: ".xmi" )
-	Dialog_Utils::makeLabeledEditField( m_GeneralWidgets.autosaveGB, autosaveLayout, 2,
+	Umbrello::makeLabeledEditField( m_GeneralWidgets.autosaveGB, autosaveLayout, 2,
 					    m_GeneralWidgets.autosaveSuffixL, i18n("Set autosave suffix:"),
 					    m_GeneralWidgets.autosaveSuffixT, m_pOptionState->generalState.autosavesuffix );
 
@@ -162,10 +175,13 @@ void SettingsDlg::setupGeneralPage() {
 				i18n("Collaboration Diagram"), i18n("State Diagram"),
 				i18n("Activity Diagram"), i18n("Component Diagram"),
 				i18n("Deployment Diagram") };
-	for( int i=0; i<9; i++)
-		m_GeneralWidgets.diagramKB -> insertItem( diagrams[i] );
 
-	m_GeneralWidgets.diagramKB -> setCurrentItem( (int) m_pOptionState->generalState.diagram );
+	//start at 1 because we don't allow No Diagram any more
+	for (int i=1; i<9; i++) {
+		m_GeneralWidgets.diagramKB->insertItem( diagrams[i] );
+	}
+
+	m_GeneralWidgets.diagramKB->setCurrentItem( (int)m_pOptionState->generalState.diagram-1 );
 	connect( m_GeneralWidgets.autosaveCB, SIGNAL(clicked()), this, SLOT(slotAutosaveCBClicked()) );
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -236,10 +252,10 @@ void SettingsDlg::setupClassPage() {
 
 }
 
-void SettingsDlg::setupCodeGenPage(CodeGenerator *gen, QDict<GeneratorInfo> ldict, QString activeLanguage) {
+void SettingsDlg::setupCodeGenPage(CodeGenerator *gen, const QString &activeLanguage) {
 	//setup code generation settings page
 	QVBox * page = addVBoxPage( i18n("Code Generation"), i18n("Code Generation Settings"), DesktopIcon( "source") );
-	m_pCodeGenPage = new CodeGenerationOptionsPage(gen, ldict, activeLanguage, page);
+	m_pCodeGenPage = new CodeGenerationOptionsPage(gen, activeLanguage, page);
 	connect( m_pCodeGenPage, SIGNAL(languageChanged()), this, SLOT(slotApply()) );
 }
 
@@ -324,6 +340,8 @@ void SettingsDlg::applyPage( Settings::Page page ) {
 	switch( page ) {
 		case Settings::page_general:
 			m_pOptionState->generalState.undo = m_GeneralWidgets.undoCB -> isChecked();
+			m_pOptionState->generalState.tabdiagrams = m_GeneralWidgets.tabdiagramsCB->isChecked();
+			m_pOptionState->generalState.newcodegen = m_GeneralWidgets.newcodegenCB->isChecked();
 			m_pOptionState->generalState.autosave = m_GeneralWidgets.autosaveCB -> isChecked();
 			m_pOptionState->generalState.autosavetime = m_GeneralWidgets.timeISB -> value();
 			// 2004-05-17 Achim Spangler: retrieve Suffix setting from dialog entry
@@ -331,7 +349,7 @@ void SettingsDlg::applyPage( Settings::Page page ) {
 			m_pOptionState->generalState.logo = m_GeneralWidgets.logoCB -> isChecked();
 			m_pOptionState->generalState.tip = m_GeneralWidgets.tipCB -> isChecked();
 			m_pOptionState->generalState.loadlast = m_GeneralWidgets.loadlastCB -> isChecked();
-			m_pOptionState->generalState.diagram  = ( Settings::Diagram ) m_GeneralWidgets.diagramKB -> currentItem();
+			m_pOptionState->generalState.diagram  = (Settings::Diagram)(m_GeneralWidgets.diagramKB->currentItem() + 1);
 			break;
 
 		case Settings::page_font:
