@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -45,7 +50,7 @@ class SettingsDlg : public KDialogBase {
 
 public:
 	SettingsDlg( QWidget * parent, Settings::OptionState *state,
-		     QDict<GeneratorInfo> ldict, QString activeLanguage, CodeGenerator * gen);
+		     QString activeLanguage, CodeGenerator * gen);
 	~SettingsDlg();
 
 	//public methods
@@ -86,6 +91,8 @@ private:
 		KComboBox * diagramKB;
 
 		QCheckBox * undoCB;
+		QCheckBox * tabdiagramsCB;
+		QCheckBox * newcodegenCB;
 		QCheckBox * autosaveCB;
 		QCheckBox * logoCB;
 		QCheckBox * tipCB;
@@ -133,7 +140,7 @@ private:
 	void setupUIPage();
 	void setupGeneralPage();
 	void setupClassPage();
-  	void setupCodeGenPage( CodeGenerator *gen, QDict<GeneratorInfo> ldict, QString activeLanguage);
+  	void setupCodeGenPage( CodeGenerator *gen, const QString &activeLanguage);
 	void setupCodeViewerPage(Settings::CodeViewerState options);
 	void applyPage( Settings::Page page );
 
