@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -25,7 +30,7 @@
 //local includes
 #include "../umlview.h"
 #include "../statewidget.h"
-#include "dialog_utils.h"
+#include "../dialog_utils.h"
 
 StateDialog::StateDialog( UMLView * pView, StateWidget * pWidget )
   : KDialogBase(IconList, i18n("Properties"), Ok | Apply | Cancel | Help, Ok, pView, "_STATEDIALOG_", true, true) {
@@ -90,12 +95,12 @@ void StateDialog::setupGeneralPage() {
 	generalLayout -> setSpacing( spacingHint() );
 	generalLayout -> setMargin(  fontMetrics().height()  );
 
-	Dialog_Utils::makeLabeledEditField( m_GenPageWidgets.generalGB, generalLayout, 0,
+	Umbrello::makeLabeledEditField( m_GenPageWidgets.generalGB, generalLayout, 0,
 					    m_GenPageWidgets.typeL, i18n("State type:"),
 					    m_GenPageWidgets.typeLE, types[ (int)type ] );
 	m_GenPageWidgets.typeLE -> setEnabled( false );
 
-	Dialog_Utils::makeLabeledEditField( m_GenPageWidgets.generalGB, generalLayout, 1,
+	Umbrello::makeLabeledEditField( m_GenPageWidgets.generalGB, generalLayout, 1,
 					    m_GenPageWidgets.nameL, i18n("State name:"),
 					    m_GenPageWidgets.nameLE );
 
