@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2003-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -38,7 +43,7 @@ public:
 	 * @param name		The name of the Enum.
 	 * @param id		The unique id of the Enum.
 	 */
-	UMLEnum(const QString& name = "", int id = -1);
+	UMLEnum(const QString& name = "", Uml::IDType id = Uml::id_None);
 
 	/**
 	 * Standard deconstructor.
@@ -61,21 +66,24 @@ public:
 	 */
 	virtual UMLObject* clone() const;
 
-	/**
-	 * Creates the <UML:Enum> element including its enumliterals.
-	 */
-	virtual void saveToXMI( QDomDocument & qDoc, QDomElement & qElement );
-
 	virtual bool isInterface () { return false; }
 
+	/**
+	 * Creates a literal for the enum.
+	 *
+	 * @return	The UMLEnum created
+	 */
+	UMLObject* createEnumLiteral();
+
  	/**
   	 * Adds an enumliteral to the enum.
   	 *
   	 * @param name		The name of the enumliteral.
-	 * @param id		The id of the enumliteral.
+	 * @param id		The id of the enumliteral (optional.)
+	 *			If omitted a new ID is assigned internally.
 	 * @return	Pointer to the UMLEnumliteral created.
 	 */
-	UMLObject* addEnumLiteral(QString name, int id);
+	UMLObject* addEnumLiteral(const QString &name, Uml::IDType id = Uml::id_None);
 
 	/**
 	 * Adds an already created enumliteral.
@@ -113,9 +121,11 @@ public:
 	 * the returned enumliteral (or to delete the enumliteral)
 	 *
 	 * @param  el enumliteral to tkae
+	 * @param wasAtIndex	if given, the index in m_List of the item taken
+	 *			is returned in the int pointed-to.
 	 * @return pointer to the enumliteral or null if not found.
 	 */
-	UMLEnumLiteral* takeEnumLiteral(UMLEnumLiteral* el);
+	UMLEnumLiteral* takeEnumLiteral(UMLEnumLiteral* el, int *wasAtIndex = NULL);
 
 	/**
 	 * Returns the number of enumliterals for the class.
@@ -124,39 +134,19 @@ public:
 	 */
 	int enumLiterals();
 
-	/**
-	 * Return the list of enumliterals for the class.
-	 *
-	 * @return	Pointer to the list of enumliterals for the class.
-	 */
-	UMLClassifierListItemList* getEnumLiteralList();
-
-	/**
-	 * Returns the entries in m_EnumLiteralList that are actually enumliterals.
-	 *
-	 * @return	The list of true enumliterals for the class.
-	 */
-	UMLEnumLiteralList getFilteredEnumLiteralList();
-
 	/**
 	 * Find a list of enumliterals with the given name.
 	 *
 	 * @param t		The type to find.
 	 * @param n		The name of the object to find.
-	 * @param seekStereo	Set this true if we should look at the object's
-	 *			stereotype instead of its name.
 	 * @return	List of objects found.  Will be empty if none found.
 	 */
-	 virtual UMLObjectList findChildObject(UMLObject_Type t, QString n,
-	 				       bool seekStereo = false);
+	 virtual UMLObjectList findChildObject(Uml::Object_Type t, const QString &n);
 
 	/**
-	 * Find an enumliteral
-	 *
-	 * @param id		The id of the object to find.
-	 * @return	Pointer to the object found; NULL if not found.
+	 * Creates the <UML:Enum> element including its enumliterals.
 	 */
-	virtual UMLObject* findChildObject(int id);
+	virtual void saveToXMI( QDomDocument & qDoc, QDomElement & qElement );
 
 signals:
 	void enumLiteralAdded(UMLObject*);
@@ -168,10 +158,6 @@ protected:
 	 */
 	bool load( QDomElement & element );
 
-	/**
-	 * List of all the enumliterals in this class.
-	 */
-	UMLClassifierListItemList m_EnumLiteralList;
 private:
 	/**
 	 * Initializes key variables of the class.
