@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2003-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,9 +15,12 @@
 #include "enumwidget.h"
 #include "enum.h"
 #include "enumliteral.h"
+#include "classifier.h"
+#include "umlclassifierlistitemlist.h"
 #include "classifierlistitem.h"
 #include "umlview.h"
 #include "umldoc.h"
+#include "uml.h"
 #include "listpopupmenu.h"
 
 #include <kdebug.h>
@@ -20,22 +28,21 @@
 
 EnumWidget::EnumWidget(UMLView* view, UMLObject* o) : UMLWidget(view, o) {
 	init();
-	setSize(100,30);
-	calculateSize();
-}
-////////////////////////////////////////////////////////////////////////////////////////////////////
-EnumWidget::EnumWidget(UMLView* view) : UMLWidget(view) {
-	init();
-	setSize(100,30);
+	if ( ! UMLApp::app()->getDocument()->loading() )
+	{ // set default size - but only if we aren't loading a XMI file at the
+		// moment - then just recreate the saved settings
+		setSize(100,30);
+		calculateSize();
+	}
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 void EnumWidget::init() {
-	UMLWidget::setBaseType(wt_Enum);
+	UMLWidget::setBaseType(Uml::wt_Enum);
 	m_pMenu = 0;
 	//set defaults from m_pView
 	if (m_pView) {
 		//check to see if correct
-		const SettingsDlg::OptionState& ops = m_pView->getOptionState();
+		const Settings::OptionState& ops = m_pView->getOptionState();
 		m_bShowPackage = ops.classState.showPackage;
 	} else {
 		// For completeness only. Not supposed to happen.
@@ -46,7 +53,7 @@ void EnumWidget::init() {
 EnumWidget::~EnumWidget() {}
 /////////////////////////////////////////////////////////////////////////////////////////////////////
 void EnumWidget::draw(QPainter& p, int offsetX, int offsetY) {
-	p.setPen(UMLWidget::getLineColour());
+	UMLWidget::draw(p, offsetX, offsetY);
 	if(UMLWidget::getUseFillColour())
 		p.setBrush(UMLWidget::getFillColour());
 	else
@@ -59,7 +66,7 @@ void EnumWidget::draw(QPainter& p, int offsetX, int offsetY) {
 	int fontHeight  = fm.lineSpacing();
 	QString name;
 	if ( m_bShowPackage ) {
-		name = m_pObject -> getPackage() + "::" + this -> getName();
+		name = m_pObject -> getPackage() + "." + this -> getName();
 	} else {
 		name = this -> getName();
 	}
@@ -72,7 +79,7 @@ void EnumWidget::draw(QPainter& p, int offsetX, int offsetY) {
 	p.setFont(font);
 	p.drawText(offsetX + ENUM_MARGIN, offsetY,
 		   w - ENUM_MARGIN * 2,fontHeight,
-		   AlignCenter, "" + m_pObject -> getStereotype() + "");
+		   AlignCenter, m_pObject -> getStereotype());
 
 	font.setItalic( m_pObject -> getAbstract() );
 	p.setFont(font);
@@ -84,15 +91,16 @@ void EnumWidget::draw(QPainter& p, int offsetX, int offsetY) {
 
 	int y = fontHeight * 2;
 
-	p.setPen( UMLWidget::getLineColour() );
+	UMLWidget::draw(p, offsetX, offsetY);
 
 	p.drawLine(offsetX, offsetY + y, offsetX + w - 1, offsetY + y);
 
 	QFontMetrics fontMetrics(font);
+	UMLClassifier *classifier = (UMLClassifier*)m_pObject;
 	UMLClassifierListItem* enumLiteral = 0;
-	UMLClassifierListItemList* list = ((UMLEnum*)m_pObject)->getEnumLiteralList();
-	for(enumLiteral=list->first(); enumLiteral != 0; enumLiteral=list->next()) {
-		QString text = enumLiteral->toString( st_NoSig );
+	UMLClassifierListItemList list = classifier->getFilteredList(Uml::ot_EnumLiteral);
+	for (enumLiteral = list.first(); enumLiteral; enumLiteral = list.next()) {
+		QString text = enumLiteral->getName();
 		p.setPen( QPen(black) );
 		p.drawText(offsetX + ENUM_MARGIN, offsetY + y,
 			   fontMetrics.width(text), fontHeight, AlignVCenter, text);
@@ -136,7 +144,7 @@ void EnumWidget::calculateSize() {
 	//now set the width of the concept
 	//set width to name to start with
 	if (m_bShowPackage)  {
-		width = getFontMetrics(FT_BOLD_ITALIC).boundingRect(m_pObject->getPackage() + "::" + getName()).width();
+		width = getFontMetrics(FT_BOLD_ITALIC).boundingRect(m_pObject->getPackage() + "." + getName()).width();
 	} else {
 		width = getFontMetrics(FT_BOLD_ITALIC).boundingRect(getName()).width();
 	}
@@ -145,10 +153,11 @@ void EnumWidget::calculateSize() {
 
 	width = w > width?w:width;
 
-	UMLClassifierListItemList* list = ((UMLEnum*)m_pObject)->getEnumLiteralList();
+	UMLClassifier *classifier = (UMLClassifier*)m_pObject;
+	UMLClassifierListItemList list = classifier->getFilteredList(Uml::ot_EnumLiteral);
 	UMLClassifierListItem* listItem = 0;
-	for (listItem = list->first();listItem != 0; listItem = list->next()) {
-		int w = fm.width( listItem->toString(st_NoSig) );
+	for (listItem = list.first(); listItem; listItem = list.next()) {
+		int w = fm.width( listItem->getName() );
 		width = w > width?w:width;
 	}
 
@@ -162,8 +171,8 @@ void EnumWidget::calculateSize() {
 void EnumWidget::slotMenuSelection(int sel) {
 	switch(sel) {
 		case ListPopupMenu::mt_EnumLiteral:
-			if ( m_pView->getDocument()->createChildObject(m_pObject, ot_EnumLiteral) )  {
-				m_pView->getDocument()->setModified();
+			if ( UMLApp::app()->getDocument()->createChildObject(m_pObject, Uml::ot_EnumLiteral) )  {
+				UMLApp::app()->getDocument()->setModified();
 			}
 			break;
 	}
@@ -188,14 +197,12 @@ bool EnumWidget::activate(IDChangeLog* ChangeLog /* = 0 */) {
 	return status;
 }
 
-bool EnumWidget::saveToXMI( QDomDocument& qDoc, QDomElement& qElement ) {
+void EnumWidget::saveToXMI( QDomDocument& qDoc, QDomElement& qElement ) {
 	QDomElement conceptElement = qDoc.createElement("enumwidget");
-	bool status = UMLWidget::saveToXMI(qDoc, conceptElement);
+	UMLWidget::saveToXMI(qDoc, conceptElement);
 
 	conceptElement.setAttribute("showpackage", m_bShowPackage);
 	qElement.appendChild(conceptElement);
-
-	return status;
 }
 
 bool EnumWidget::loadFromXMI( QDomElement & qElement ) {
