@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -15,25 +20,29 @@
 #include <kdebug.h>
 
 // local includes
-#include "associationwidget.h"
 #include "association.h"
 #include "umlview.h"
 #include "umldoc.h"
 #include "uml.h"
 #include "classifier.h"
-#include "messagewidget.h"
 #include "listpopupmenu.h"
 #include "operation.h"
+#include "model_utils.h"
 #include "inputdialog.h"
 #include "dialogs/assocpropdlg.h"
 #include "dialogs/selectopdlg.h"
 
-FloatingText::FloatingText(UMLView * view, Text_Role role, QString text, int id)
+FloatingText::FloatingText(UMLView * view, Uml::Text_Role role, QString text, Uml::IDType id)
   : UMLWidget(view, id)
 {
 	init();
 	m_Text = text;
 	m_Role = role;
+	if ( ! UMLApp::app()->getDocument()->loading() ) {
+		calculateSize();
+		setZ( 10 );//make sure always on top.
+		update();
+	}
 }
 
 void FloatingText::init() {
@@ -41,17 +50,10 @@ void FloatingText::init() {
 	m_PreText = "";
 	m_Text = "";
 	m_PostText = "";
-	//m_SeqNum = "";
-	//m_Operation = "";
 	m_Role = Uml::tr_Floating;
 	m_Type = Uml::wt_Text;
 	// initialize non-saved (i.e. volatile) data
 	m_pLink = NULL;
-	if ( ! UMLApp::app()->getDocument()->loading() ) {
-		calculateSize();
-		setZ( 10 );//make sure always on top.
-		update();
-	}
 }
 
 FloatingText::~FloatingText() {
@@ -70,45 +72,6 @@ void FloatingText::draw(QPainter & p, int offsetX, int offsetY) {
 
 void FloatingText::resizeEvent(QResizeEvent * /*re*/) {}
 
-void FloatingText::setLinePos(int x, int y) {
-	bool xIsValid = (x >= restrictPositionMin && x <= restrictPositionMax);
-	bool yIsValid = (y >= restrictPositionMin && y <= restrictPositionMax);
-	if (xIsValid && yIsValid) { // fine
-		setX(x);
-		setY(y);
-	} else { // something is broken
-		kdDebug() << "FloatingText::setLinePos( " << x << " , " << y << " ) "
-			<< "- was blocked because at least one value is out of bounds: ["
-			<< restrictPositionMin << "..." << restrictPositionMax << "]  "
-			<< "origX: " << getX() << ", origY: " << getY()
-			<< endl;
-		// Let's just leave them at their original values.
-	}
-}
-
-void FloatingText::setLinePositionRelatively(int newX, int newY, int oldX, int oldY) {
-	int myNewX = getX() + (newX-oldX);
-	int myNewY = getY() + (newY-oldY);
-	bool xIsValid = (myNewX >= restrictPositionMin && myNewX <= restrictPositionMax);
-	bool yIsValid = (myNewY >= restrictPositionMin && myNewY <= restrictPositionMax);
-	if (xIsValid && yIsValid) { // fine
-		bool oldIgnoreSnapToGrid = m_bIgnoreSnapToGrid;
-		m_bIgnoreSnapToGrid = true;
-		setX(myNewX);
-		setY(myNewY);
-		m_bIgnoreSnapToGrid = oldIgnoreSnapToGrid;
-	} else { // something is broken
-		kdDebug() << "FloatingText::setLinePositionRelatively( " << myNewX
-			<< " , " << myNewY << " ) - was Blocked because at least one value is out of bounds: ["
-			<< restrictPositionMin << "..." << restrictPositionMax << "]\n"
-			<< "ToX: " << newX << ", ToY: " << newY
-			<< "FromX: " << oldX << ", FromY: " << oldY
-			<< "CurrentPointX: " << getX() << ", CurrentPointY: " << getY()
-			<< endl;
-		// Let's just leave them at their original values.
-	}
-}
-
 void FloatingText::calculateSize() {
 	QFontMetrics &fm = getFontMetrics(FT_NORMAL);
 	int h = fm.lineSpacing();
@@ -122,40 +85,10 @@ void FloatingText::slotMenuSelection(int sel) {
 		mouseDoubleClickEvent((QMouseEvent *)0);
 		break;
 
-	case ListPopupMenu::mt_Delete_Association:
-		// UNEXPECTED - should not be here
-		// Remove this branch when confident that it does not happen
-		kdError() << "UNEXPECTED: "
-			  << "FloatingText::slotMenuSelection(mt_Delete_Association)"
-			  << " is called." << endl;
-		break;
-
 	case ListPopupMenu::mt_Delete:
 		m_pView -> removeWidget(this);
 		break;
 
-	case ListPopupMenu::mt_Delete_Message:
-		// UNEXPECTED - should not be here
-		// Remove this branch when confident that it does not happen
-		kdDebug() << "FloatingText::slotMenuSelection(mt_Delete_Message)"
-			  << " is called" << endl;
-		if (m_pLink == NULL)
-			return;
-		if (m_Role == tr_Seq_Message || m_Role == tr_Seq_Message_Self) {
-			//here to delete this from a seq. diagram.
-			kdDebug() << "  --- for Seq_Message." << endl;
-			MessageWidget *m = static_cast<MessageWidget*>(m_pLink);
-			m_pView->removeWidget( m );
-		} else {
-			if (m_pView->getType() != dt_Collaboration)
-				return;
-			kdDebug() << "  --- for association." << endl;
-			//here to delete assoc. on collab diagram.
-			AssociationWidget *a = static_cast<AssociationWidget*>(m_pLink);
-			m_pView->removeAssoc( a );
-		}
-		break;
-
 	case ListPopupMenu::mt_Operation:
 		{
 			kdDebug() << "FloatingText::slotMenuSelection(mt_Operation) is called."
@@ -165,24 +98,20 @@ void FloatingText::slotMenuSelection(int sel) {
 					  << "m_pLink is NULL" << endl;
 				return;
 			}
-			UMLClassifier* c = m_pLink->getOperationOwner(this);
+			UMLClassifier* c = m_pLink->getOperationOwner();
 			if (c == NULL) {
 				bool ok = false;
 				QString opText = KInputDialog::getText(i18n("Name"),
 								       i18n("Enter operation name:"),
 								       getText(), &ok, m_pView);
 				if (ok)
-					m_pLink->setOperationText(this, opText);
+					m_pLink->setCustomOpText(opText);
 				return;
 			}
 			UMLObject* umlObj = UMLApp::app()->getDocument()->createChildObject(c, Uml::ot_Operation);
 			if (umlObj) {
 				UMLOperation* newOperation = static_cast<UMLOperation*>( umlObj );
-				// It should perhaps be configurable whether the
-				// operation is displayed with or without its signature.
-				// For the time being, we don't display the signature.
-				QString opText = newOperation->toString(st_NoSigNoScope);
-				m_pLink->setOperationText(this, opText);
+				m_pLink->setOperation(newOperation);
 			}
 		}
 		break;
@@ -199,10 +128,10 @@ void FloatingText::slotMenuSelection(int sel) {
 		{
 			QFont font = getFont();
 			if( KFontDialog::getFont( font, false, m_pView ) ) {
-				if( m_Role == tr_Floating || m_Role == tr_Seq_Message ) {
+				if( m_Role == Uml::tr_Floating || m_Role == Uml::tr_Seq_Message ) {
 					setFont( font );
 				} else if (m_pLink) {
-					m_pLink->setFont(font);
+					m_pLink->lwSetFont(font);
 				}
 			}
 		}
@@ -221,18 +150,18 @@ void FloatingText::slotMenuSelection(int sel) {
 
 void FloatingText::handleRename() {
 	QString t;
-	if( m_Role == tr_RoleAName || m_Role == tr_RoleBName ) {
+	if( m_Role == Uml::tr_RoleAName || m_Role == Uml::tr_RoleBName ) {
 		t = i18n("Enter role name:");
-	} else if (m_Role == tr_MultiA || m_Role == tr_MultiB) {
+	} else if (m_Role == Uml::tr_MultiA || m_Role == Uml::tr_MultiB) {
 		t = i18n("Enter multiplicity:");
 		/*
 		// NO! shouldnt be allowed
-		} else if( m_Role == tr_ChangeA || m_Role == tr_ChangeB ) {
+		} else if( m_Role == Uml::tr_ChangeA || m_Role == Uml::tr_ChangeB ) {
 		t = i18n("Enter changeability");
 		*/
-	} else if (m_Role == tr_Name) {
+	} else if (m_Role == Uml::tr_Name) {
 		t = i18n("Enter association name:");
-	} else if (m_Role == tr_Floating) {
+	} else if (m_Role == Uml::tr_Floating) {
 		t = i18n("Enter new text:");
 	} else {
 		t = i18n("ERROR");
@@ -243,13 +172,13 @@ void FloatingText::handleRename() {
 		return;
 	bool valid = isTextValid(newText);
 	if (!valid || newText == getText()) {
-		if (!valid && m_Role == tr_Floating)
+		if (!valid && m_Role == Uml::tr_Floating)
 			m_pView -> removeWidget(this);
 		calculateSize();
 		update();
 		return;
 	}
-	if (m_pLink && m_Role != tr_Seq_Message && m_Role != tr_Seq_Message_Self) {
+	if (m_pLink && m_Role != Uml::tr_Seq_Message && m_Role != Uml::tr_Seq_Message_Self) {
 		m_pLink->setText(this, newText);
 	} else {
 		setText( newText );
@@ -260,8 +189,8 @@ void FloatingText::handleRename() {
 	update();
 }
 
-void FloatingText::setText(QString t) {
-	if (m_Role == tr_Seq_Message || m_Role == tr_Seq_Message_Self) {
+void FloatingText::setText(const QString &t) {
+	if (m_Role == Uml::tr_Seq_Message || m_Role == Uml::tr_Seq_Message_Self) {
 		QString seqNum, op;
 		m_pLink->getSeqNumAndOp(this, seqNum, op);
 		if (seqNum.length() > 0 || op.length() > 0)
@@ -274,14 +203,14 @@ void FloatingText::setText(QString t) {
 	update();
 }
 
-void FloatingText::setPreText (QString t)
+void FloatingText::setPreText (const QString &t)
 {
 	m_PreText = t;
 	calculateSize();
 	update();
 }
 
-void FloatingText::setPostText(QString t) {
+void FloatingText::setPostText(const QString &t) {
 	m_PostText = t;
 	calculateSize();
 	update();
@@ -304,10 +233,10 @@ void FloatingText::changeTextDlg() {
 void FloatingText::mouseDoubleClickEvent(QMouseEvent * /* me*/) {
 	if(m_pView -> getCurrentCursor() != WorkToolBar::tbb_Arrow)
 		return;
-	if (m_Role == tr_Coll_Message || m_Role == tr_Coll_Message_Self ||
-	    m_Role == tr_Seq_Message || m_Role == tr_Seq_Message_Self) {
+	if (m_Role == Uml::tr_Coll_Message || m_Role == Uml::tr_Coll_Message_Self ||
+	    m_Role == Uml::tr_Seq_Message || m_Role == Uml::tr_Seq_Message_Self) {
 		showOpDlg();
-	} else if (m_Role == tr_Floating) {
+	} else if (m_Role == Uml::tr_Floating) {
 		// double clicking on a text line opens the dialog to change the text
 		handleRename();
 	} else if (m_pLink) {
@@ -320,8 +249,8 @@ void FloatingText::showOpDlg() {
 		kdError() << "FloatingText::showOpDlg: m_pLink is NULL" << endl;
 		return;
 	}
-	QString seqNum, op;
-	UMLClassifier* c = m_pLink->getSeqNumAndOp(this, seqNum, op);
+	QString seqNum, opText;
+	UMLClassifier* c = m_pLink->getSeqNumAndOp(this, seqNum, opText);
 	if (c == NULL) {
 		kdError() << "FloatingText::showOpDlg: "
 			  << "m_pLink->getSeqNumAndOp() returns a NULL classifier"
@@ -329,40 +258,55 @@ void FloatingText::showOpDlg() {
 		return;
 	}
 
-	SelectOpDlg selectDlg((QWidget*)m_pView, c);
+	SelectOpDlg selectDlg(m_pView, c);
 	selectDlg.setSeqNumber( seqNum );
-	selectDlg.setCustomOp( op );
+	if (m_pLink->getOperation() == NULL)
+		selectDlg.setCustomOp( opText );
 	int result = selectDlg.exec();
 	if(!result) {
 		return;
 	}
 	seqNum = selectDlg.getSeqNumber();
-	// It should perhaps be configurable whether the operation
-	// is displayed with or without its signature.
-	// Here, we just discard the signature.
-	op = selectDlg.getOpText();
-	op.replace( QRegExp("\\(.*\\)"), "()" );
-	m_pLink->setSeqNumAndOp(seqNum, op);
+	opText = selectDlg.getOpText();
+	Umbrello::OpDescriptor od;
+	Umbrello::Parse_Status st = Umbrello::parseOperation(opText, od, c);
+	if (st == Umbrello::PS_OK) {
+		UMLClassifierList selfAndAncestors = c->findSuperClassConcepts();
+		selfAndAncestors.prepend(c);
+		UMLOperation *op = NULL;
+		for (UMLClassifier *cl = selfAndAncestors.first(); cl; cl = selfAndAncestors.next()) {
+			op = cl->findOperation(od.m_name, od.m_args);
+			if (op != NULL)
+				break;
+		}
+		if (op == NULL) {
+			// The op does not yet exist. Create a new one.
+			UMLObject *o = c->createOperation(od.m_name, NULL, &od.m_args);
+			op = static_cast<UMLOperation*>(o);
+		}
+		if (od.m_pReturnType)
+			op->setType(od.m_pReturnType);
+		m_pLink->setOperation(op);
+		opText = QString::null;
+	} else {
+		m_pLink->setOperation(NULL);
+	}
+	m_pLink->setSeqNumAndOp(seqNum, opText);
 	setMessageText();
 }
 
 void FloatingText::mouseMoveEvent(QMouseEvent* me) {
 	if (!m_bMouseDown && me->button() != LeftButton)
 		return;
-	if (m_Role == tr_Seq_Message_Self)
+	if (m_Role == Uml::tr_Seq_Message_Self)
 		return;
 	QPoint newPosition = doMouseMove(me);
 	int newX = newPosition.x();
 	int newY = newPosition.y();
 
 	//implement specific rules for a sequence diagram
-	if (m_Role == tr_Seq_Message) {
-		MessageWidget *pMessage = static_cast<MessageWidget*>(m_pLink);
-		const int minHeight = pMessage->getMinHeight();
-		newY = newY < minHeight ? minHeight : newY;
-		const int maxHeight = pMessage->getMaxHeight() - height() - 5;
-		newY = newY < maxHeight ? newY : maxHeight;
-		pMessage->setY( newY + height() );
+	if (m_Role == Uml::tr_Seq_Message || m_Role == Uml::tr_Seq_Message_Self) {
+		m_pLink->constrainTextPos(newX, newY, width(), height(), m_Role);
 	}
 	m_nOldX = newX;
 	m_nOldY = newY;
@@ -386,8 +330,8 @@ QString FloatingText::getText() const {
 	//and the actual message widget
 	// hmm. this section looks like it could have been avoided by using pre-, post- text
 	// instead of storing in the main body of the text -b.t.
-	if(m_Role == tr_Seq_Message || m_Role == tr_Seq_Message_Self ||
-	   m_Role == tr_Coll_Message || m_Role == tr_Coll_Message_Self) {
+	if(m_Role == Uml::tr_Seq_Message || m_Role == Uml::tr_Seq_Message_Self ||
+	   m_Role == Uml::tr_Coll_Message || m_Role == Uml::tr_Coll_Message_Self) {
 		if( m_Text.length() <= 1 || m_Text == ": " )
 			return "";
 	}
@@ -409,25 +353,15 @@ bool FloatingText::activate( IDChangeLog* ChangeLog /*= 0 */) {
 	return status;
 }
 
-// God this is bad. Why not use the assocList in the
-// parent umlwidget class? At the very least, we should
-// keep them synced.
-// Also, there seems to be some issues with setting id to the association
-// XMI id. There CAN be more than one floating text widget with this id
-// but a findWidget will come up with only the first one.
 void FloatingText::setLink(LinkWidget * l) {
-	if (m_pLink)  // remove pre-existing link
-		m_pLink->cleanupBeforeFTsetLink(this);
 	m_pLink = l;
-	if (m_pLink)
-		m_pLink->setupAfterFTsetLink(this);
 }
 
 LinkWidget * FloatingText::getLink() {
 	return m_pLink;
 }
 
-void FloatingText::setRole(Text_Role role) {
+void FloatingText::setRole(Uml::Text_Role role) {
 	m_Role = role;
 }
 
@@ -435,7 +369,7 @@ Uml::Text_Role FloatingText::getRole() const {
 	return m_Role;
 }
 
-bool FloatingText::isTextValid(QString text) {
+bool FloatingText::isTextValid( const QString &text ) {
 	int length = text.length();
 	if(length < 1)
 		return false;
@@ -445,32 +379,7 @@ bool FloatingText::isTextValid(QString text) {
 	return false;
 }
 
-/*
-void FloatingText::setSeqNum(QString sn)
-{
-	m_SeqNum = sn;
-	update();
-}
-
-QString FloatingText::getSeqNum() const {
-	return m_SeqNum;
-}
-
-void FloatingText::setOperation(QString op) {
-	m_Operation = op;
-}
-
-QString FloatingText::getOperation() const {
-	return m_Operation;
-}
-*/
-
 void FloatingText::setSelected(bool _select) {
-	if( m_nOldID == -10 )//used to stop a recursive call
-	{
-		m_nOldID = 0;
-		return;
-	}
 	UMLWidget::setSelected( _select );
 }
 
@@ -506,7 +415,7 @@ bool FloatingText::loadFromXMI( QDomElement & qElement ) {
 	// CAVEAT: The caller should not interpret the false return value
 	//  as an indication of failure since previous umbrello versions
 	//  saved lots of these empty FloatingTexts.
-	bool isDummy = (m_Text == "" && m_PreText == "" && m_PostText == "");
+	bool isDummy = (m_Text.isEmpty() && m_PreText.isEmpty() && m_PostText.isEmpty());
 	return !isDummy;
 }
 
@@ -517,3 +426,6 @@ void FloatingText::setMessageText() {
 	calculateSize();
 }
 
+
+#include "floatingtext.moc"
+
