@@ -13,6 +13,9 @@
    Usage:   #include "inputdialog.h" instead of
 	    #include <klineeditdlg.h> or <kinputdialog.h>
  */
+#ifndef INPUTDIALOG_H
+#define INPUTDIALOG_H
+
 #include <kdeversion.h>
 #if !KDE_IS_VERSION(3,1,90)
 # include <klineeditdlg.h>  //deprecated
@@ -21,3 +24,4 @@
 # include <kinputdialog.h>
 #endif
 
+#endif // INPUTDIALOG_H
