@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2003-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -12,6 +17,7 @@
 
 #include "umlcanvasobject.h"
 #include "umlclassifierlist.h"
+#include "umlclasslist.h"
 #include "umlinterfacelist.h"
 
 /**
@@ -40,7 +46,7 @@ public:
 	 * @param name		The name of the Concept.
 	 * @param id		The unique id of the Concept.
 	 */
-	UMLPackage(const QString & Name = "", int id = -1);
+	UMLPackage(const QString & Name = "", Uml::IDType id = Uml::id_None);
 
 	/**
 	 * Empty deconstructor.
@@ -88,7 +94,7 @@ public:
 	 * @param name		The name to seek.
 	 * @return	Pointer to the UMLObject found or NULL if not found.
 	 */
-	UMLObject * findObject(QString name);
+	UMLObject * findObject(const QString &name);
 
 	/**
 	 * Find the object of the given ID in the list of contained objects.
@@ -96,18 +102,7 @@ public:
 	 * @param id		The ID to seek.
 	 * @return	Pointer to the UMLObject found or NULL if not found.
 	 */
-	UMLObject * findObject(int id);
-
-	/**
-	 * Find the object of the given (non-numeric) auxiliary ID
-	 * in the list of contained objects. The auxiliary ID is the
-	 * ID returned by UMLObject::getAuxId() and is currently
-	 * only used for loading foreign XMI files.
-	 *
-	 * @param idStr		The ID to seek.
-	 * @return	Pointer to the UMLObject found or NULL if not found.
-	 */
-	UMLObject* findObjectByIdStr(QString idStr);
+	UMLObject * findObjectById(Uml::IDType id);
 
 	/**
 	 * Append all classifiers from this package (and those from
@@ -120,6 +115,16 @@ public:
 	void appendClassifiers( UMLClassifierList& classifiers,
 				bool includeNested = true );
 
+	/**
+	 * Append all classes from this package (and those from
+	 * nested packages) to the given UMLClassList.
+	 *
+	 * @param classes		The list to append to.
+	 * @param includeNested		Whether to include the classes from
+	 *				nested packages (default: true.)
+	 */
+	void appendClasses( UMLClassList& classes, bool includeNested = true );
+
 	/**
 	 * Append all classes and interfaces from this package (and those
 	 * from nested packages) to the given UMLClassifierList.
@@ -142,6 +147,16 @@ public:
 	void appendInterfaces(UMLInterfaceList& interfaces,
 				bool includeNested = true );
 
+	/**
+	 * Resolve types. Required when dealing with foreign XMI files.
+	 * Needs to be called after all UML objects are loaded from file.
+	 * Overrides the method from UMLObject.
+	 * Calls resolveRef() on each contained object.
+	 *
+	 * @return	True for overall success.
+	 */
+	virtual bool resolveRef();
+
 	/**
 	 * Creates the <UML:Package> XMI element.
 	 */
