@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2003-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -16,23 +21,19 @@
 #include <klocale.h>
 #include <kdebug.h>
 
-UMLStereotype::UMLStereotype(QString name, int id /* = -1 */)
+UMLStereotype::UMLStereotype(const QString &name, Uml::IDType id /* = Uml::id_None */)
   : UMLObject( name, id ) {
-	m_BaseType = ot_Stereotype;
+	m_BaseType = Uml::ot_Stereotype;
 	m_refCount = 0;
 }
 
 UMLStereotype::UMLStereotype() : UMLObject() {
-	m_BaseType = ot_Stereotype;
+	m_BaseType = Uml::ot_Stereotype;
 	m_refCount = 0;
 }
 
 UMLStereotype::~UMLStereotype() {}
 
-QString UMLStereotype::toString(Signature_Type /*sig*/) {
-	return "" + getName() + "";
-}
-
 bool UMLStereotype::operator==( UMLStereotype &rhs) {
 	if (this == &rhs) {
 		return true;
@@ -78,10 +79,6 @@ bool UMLStereotype::showPropertiesDialogue(QWidget* parent) {
 	return ok;
 }
 
-QString UMLStereotype::getShortName() {
-	return "" + getName() + "";
-}
-
 void UMLStereotype::incrRefCount() {
 	m_refCount++;
 }
