@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2003-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -32,7 +37,7 @@ public:
 	 * @param listType	The list which this stereotype is part of
 	 *			(attribute, operation etc)
 	 */
-	UMLStereotype(QString name, int id = -1);
+	UMLStereotype(const QString &name, Uml::IDType id = Uml::id_None);
 
 	/**
 	 * Sets up a stereotype.
@@ -62,13 +67,6 @@ public:
 	 */
 	virtual UMLObject* clone() const;
 
-	/**
-	 * Returns a string representation of the UMLStereotype.
-	 *
-	 * @return	Returns a string representation of the UMLStereotype.
-	 */
-	QString toString(Signature_Type sig = st_NoSig);
-
 	/**
 	 * Increments the reference count for this stereotype.
 	 */
@@ -95,11 +93,6 @@ public:
 	 */
 	bool showPropertiesDialogue(QWidget* parent);
 
-	/**
-	 * Return the stereotype name surrounded by double angle brackets.
-	 */
-	virtual QString getShortName();
-
 protected:
 	/**
 	 * Loads a <UML:StereoType> XMI element.
