@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2003-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -7,40 +12,45 @@
  *                                                                         *
  ***************************************************************************/
 
+// own header
 #include "template.h"
-#include "dialogs/umltemplatedialog.h"
+
+// qt/kde includes
+#include <qregexp.h>
 #include <kdebug.h>
 
-UMLTemplate::UMLTemplate(const UMLObject *parent, QString name, int id, QString type)
+// app includes
+#include "uml.h"
+#include "umldoc.h"
+#include "dialogs/umltemplatedialog.h"
+
+UMLTemplate::UMLTemplate(const UMLObject *parent, QString name, Uml::IDType id, QString type)
   : UMLClassifierListItem( parent, name, id ) {
-	m_TypeName = type;
-	m_BaseType = ot_Template;
+	setTypeName( type );
+	m_BaseType = Uml::ot_Template;
 }
-////////////////////////////////////////////////////////////////////////////////////////////////////
+
 UMLTemplate::UMLTemplate(const UMLObject *parent)
   : UMLClassifierListItem( parent ) {
-	m_BaseType = ot_Template;
-	m_TypeName = "";
+	m_BaseType = Uml::ot_Template;
 }
-////////////////////////////////////////////////////////////////////////////////////////////////////
+
 UMLTemplate::~UMLTemplate() {}
-////////////////////////////////////////////////////////////////////////////////////////////////////
-QString UMLTemplate::getTypeName() {
-	return m_TypeName;
-}
-////////////////////////////////////////////////////////////////////////////////////////////////////
-void UMLTemplate::setTypeName(QString type) {
-	m_TypeName = type;
-}
-////////////////////////////////////////////////////////////////////////////////////////////////////
-QString UMLTemplate::toString(Signature_Type /*sig = st_NoSig*/) {
-	if (m_TypeName == "" || m_TypeName == "class") {
+
+QString UMLTemplate::toString(Uml::Signature_Type /*sig = st_NoSig*/) {
+	if (m_pSecondary == NULL || m_pSecondary->getName() == "class") {
 		return getName();
 	} else {
-		return getName() + " : " + m_TypeName;
+		return getName() + " : " + m_pSecondary->getName();
 	}
 }
-////////////////////////////////////////////////////////////////////////////////////////////////////
+
+QString UMLTemplate::getTypeName() {
+	if (m_pSecondary == NULL)
+		return "class";
+	return m_pSecondary->getName();
+}
+
 bool UMLTemplate::operator==(UMLTemplate &rhs) {
 	if (this == &rhs) {
 		return true;
@@ -48,7 +58,7 @@ bool UMLTemplate::operator==(UMLTemplate &rhs) {
 	if ( !UMLObject::operator==( rhs ) ) {
 		return false;
 	}
-	if (m_TypeName != rhs.m_TypeName) {
+	if (m_pSecondary != rhs.m_pSecondary) {
 		return false;
 	}
 	return true;
@@ -57,8 +67,6 @@ bool UMLTemplate::operator==(UMLTemplate &rhs) {
 void UMLTemplate::copyInto(UMLTemplate *rhs) const
 {
 	UMLClassifierListItem::copyInto(rhs);
-
-	rhs->m_TypeName = m_TypeName;
 }
 
 UMLObject* UMLTemplate::clone() const
@@ -70,16 +78,16 @@ UMLObject* UMLTemplate::clone() const
 }
 
 
-
 void UMLTemplate::saveToXMI(QDomDocument& qDoc, QDomElement& qElement) {
 	//FIXME: uml13.dtd compliance
-	QDomElement attributeElement = UMLObject::save("template", qDoc);
-	attributeElement.setAttribute("type", m_TypeName);
+	QDomElement attributeElement = UMLObject::save("UML:TemplateParameter", qDoc);
+	if (m_pSecondary)
+		attributeElement.setAttribute("type", ID2STR(m_pSecondary->getID()));
 	qElement.appendChild(attributeElement);
 }
 
 bool UMLTemplate::load(QDomElement& element) {
-	m_TypeName = element.attribute("type", "");
+	m_SecondaryId = element.attribute("type", "");
 	return true;
 }
 
