@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2003-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -33,7 +38,8 @@ public:
 	 * @param id		The unique id given to this UMLTemplate.
 	 * @param type		The type of this UMLTemplate.
 	 */
-	UMLTemplate(const UMLObject *parent, QString name, int id, QString type = "class");
+	UMLTemplate(const UMLObject *parent, QString name, Uml::IDType id = Uml::id_None,
+		    QString type = "class");
 
 	/**
 	 * Sets up a template.
@@ -63,27 +69,23 @@ public:
 	 */
 	virtual ~UMLTemplate();
 
-	/**
-	 * Returns the type of the UMLTemplate.
-	 *
-	 * @return	The type of the UMLTemplate.
-	 */
-	QString getTypeName();
-
-	/**
-	 * Sets the type of the UMLTemplate.
-	 *
-	 * @param type		The type of the UMLTemplate.
-	 */
-	void setTypeName(QString type);
-
 	/**
 	 * Returns a string representation of the UMLTemplate.
 	 *
 	 * @param sig		Currently unused.
 	 * @return	Returns a string representation of the UMLTemplate.
 	 */
-	QString toString(Signature_Type sig = st_NoSig);
+	QString toString(Uml::Signature_Type sig = Uml::st_NoSig);
+
+	/**
+	 * Overrides method from UMLClassifierListItem.
+	 * Returns the type name of the UMLTemplate.
+	 * If the template parameter is a class, there is no separate
+	 * type object. In this case, getTypeName() returns "class".
+	 *
+	 * @return	The type name of the UMLClassifierListItem.
+	 */
+	virtual QString getTypeName();
 
 	/**
 	 * Display the properties configuration dialogue for the template.
@@ -93,21 +95,16 @@ public:
 	bool showPropertiesDialogue(QWidget* parent);
 
 	/**
-	 * Write the <UML:Template> XMI element.
+	 * Writes the <UML:TemplateParameter> XMI element.
 	 */
 	void saveToXMI(QDomDocument & qDoc, QDomElement & qElement);
 
 protected:
 	/**
-	 * Loads the <UML:Template> XMI element.
+	 * Loads the <UML:TemplateParameter> XMI element.
 	 */
 	bool load(QDomElement & element);
 
-private:
-	/**
-	 * The type of the template, usually just "class"
-	 */
-	QString m_TypeName;
 };
 
 #endif
