@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,6 +15,7 @@
 #ifndef TOOLBARSTATEMESSAGES_H
 #define TOOLBARSTATEMESSAGES_H
 
+#include <qpoint.h>
 #include "toolbarstatepool.h"
 
 
@@ -40,6 +46,7 @@ protected:
 protected:
 	UMLWidget* m_pSelectedWidget;
 	QCanvasLine* m_pLine;
+	QPoint m_FirstMousePos;
 };
 
 #endif //TOOLBARSTATEMESSAGES_H
