@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -15,6 +20,7 @@
 #endif
 
 #include "optionstate.h"
+#include "generatorinfo.h"
 
 #include <qmap.h>
 #include <qdict.h>
@@ -26,10 +32,10 @@
 
 // forward declaration of the UML classes
 class AlignToolBar;
+class ClassImport;
 class CodeDocument;
 class CodeGenerator;
 class CodeGenerationPolicy;
-class GeneratorInfo;
 class DocWindow;
 class UMLClassifier;
 class UMLDoc;
@@ -47,16 +53,17 @@ class KRecentFilesAction;
 class KStatusBarLabel;
 class KToggleAction;
 class KDockWidget;
+class KTabWidget;
+class KToolBarButton;
+class KPopupMenu;
 
 // Qt forward declarations
 class QWidgetStack;
 class QMenuData;
 class QClipboard;
+class QToolButton;
 
-namespace Umbrello
-{
 class RefactoringAssistant;
-}
 
 /**
  * The base class for UML application windows. It sets up the main
@@ -212,7 +219,13 @@ public:
 	 * @return	Pointer to the CodeGenerator created.
   	 */
 	CodeGenerator* createGenerator();
-	
+
+	/**
+	 * Returns the dictionary of available languages.
+	 *
+	 * @return	Pointer to the dictionary of available languages.
+	 */
+	GeneratorDict& generatorDict();
 
 	/**
 	 * Call the refactoring assistant on a classifier.
@@ -237,7 +250,6 @@ public:
 
 	/**
 	 * Returns the widget used as the parent for UMLViews.
-	 *
 	 * @return	The main view widget.
 	 */
 	QWidget* getMainViewWidget();
@@ -251,6 +263,19 @@ public:
 	 */
 	void setCurrentView(UMLView* view = 0);
 
+	/**
+	 * Get the current view.
+	 * This may return a null pointer (when no view was previously
+	 * specified.)
+	 *
+	 */
+	UMLView* getCurrentView();
+
+	/**
+	 * Return the ClassImport object.
+	 */
+	ClassImport * classImport();
+
 	/**
 	 * Sets the default mimetype for all diagrams that are exported as 
 	 * images.
@@ -278,9 +303,19 @@ public:
 	 */
 	bool editCutCopy( bool bFromView );
 
+	/**
+	 * Return the tab widget.
+	 */
+	KTabWidget *tabWidget();
+
 protected:
 	virtual void keyPressEvent(QKeyEvent* e);
 	virtual void keyReleaseEvent(QKeyEvent* e);
+	
+	/**
+	 * Helper method for handling cursor key release events (refactoring).
+	 */
+	 void handleCursorKeyReleaseEvent(QKeyEvent* e);
 
 	/**
 	 * Save general Options like all bar positions and status
@@ -306,13 +341,6 @@ protected:
 	 */
 	void initStatusBar();
 
-	/**
-	 * Initializes the document object of the main window
-	 * that is connected to the view in initView().
-	 * @see initView();
-	 */
-	void initDocument();
-
 	/**
 	 * Creates the centerwidget of the KMainWindow instance and
 	 * sets it as the view.
@@ -517,6 +545,10 @@ public slots:
 	 * Create this view.
 	 */
 	void slotDeploymentDiagram();
+	/**
+	 * Create this view.
+	 */
+	void slotEntityRelationshipDiagram();
 
 	/**
 	 * Notification of changed clipboard data.
@@ -562,7 +594,7 @@ public slots:
 	 *
 	 * @param activeLanguage The name of the language to set
 	 */
-	void setActiveLanguage(QString activeLanguage);
+	void setActiveLanguage( const QString &activeLanguage );
 
 	/**
 	 * Menu selection for clear current view.
@@ -677,7 +709,32 @@ public slots:
 	 * @param menu	The QPopupMenu or QMenuBar to search through.
 	 * @param name	The name of the menu to search for (name, not text)
 	 */
-	QPopupMenu* findMenu(QMenuData* menu, QString name);
+	QPopupMenu* findMenu(QMenuData* menu, const QString &name);
+
+	/**
+	 * called when the tab has changed
+	 */
+	void slotTabChanged(QWidget* view);
+
+	/**
+	 * make the tab on the left of the current one the active one
+	 */
+	void slotChangeTabLeft();
+
+	/**
+	 * make the tab on the right of the current one the active one
+	 */
+	void slotChangeTabRight();
+
+	/**
+	 * Move the current tab left, not implemented
+	 */
+	void slotMoveTabLeft();
+
+	/**
+	 * Move the current tab right, not implemented
+	 */
+	void slotMoveTabRight();
 
 private:
 	static UMLApp* s_instance;
@@ -685,24 +742,29 @@ private:
 	/**
 	 * For selecting the active language.
 	 */
-	QPopupMenu *langSelect;
+	QPopupMenu *m_langSelect;
 	
 	/**
 	* Popup menu for zoom selection.
 	*/
-	QPopupMenu *zoomSelect;
+	QPopupMenu *m_zoomSelect;
 
 	/**
 	 * Active language.
 	 */
-	QString activeLanguage;
+	QString m_activeLanguage;
 
 	/**
 	 * Dictionary containing all the info needed to load a Code Generator
 	 * at run time:
 	 * Language (so that the user can choose), library and object name.
 	 */
-	QDict<GeneratorInfo> generatorDict;
+	GeneratorDict m_generatorDict;
+
+	/**
+	 * Reverse engineering object.
+	 */
+	ClassImport * m_classImporter;
 
 	/**
 	 * Reads from the config file the options state.
@@ -727,23 +789,22 @@ private:
 	/**
 	 * The configuration object of the application.
 	 */
-	KConfig* config;
+	KConfig* m_config;
 
 	/**
-	 * View is the main widget which represents your working
-	 * area. The View class should handle all events of the view
-	 * widget.  It is kept empty so you can create your view
-	 * according to your application's needs by changing the view
-	 * class.
+	 * View is the main widget which represents your working area.
+	 * The View class should handle all events of the view widget.
+	 * It is kept empty so you can create your view according to your
+	 * application's needs by changing the view class.
 	 */
-	UMLView* view;
+	UMLView* m_view;
 
 	/**
 	 * doc represents your actual document and is created only once.
 	 * It keeps information such as filename and does the loading and
 	 * saving of your files.
 	 */
-	UMLDoc* doc;
+	UMLDoc* m_doc;
 
 	/**
 	 * Listview shows the current open file.
@@ -771,7 +832,7 @@ private:
 	DocWindow* m_pDocWindow;
 	
 	/** Refactoring assistant. */
-	Umbrello::RefactoringAssistant* m_refactoringAssist;
+	RefactoringAssistant* m_refactoringAssist;
 
 	//KAction pointers to enable/disable actions
 	KAction* fileNew;
@@ -800,6 +861,7 @@ private:
 	KAction* activityDiagram;
 	KAction* componentDiagram;
 	KAction* deploymentDiagram;
+	KAction* entityRelationshipDiagram;
 	KAction* viewClearDiagram;
 		
 	KToggleAction* viewSnapToGrid;
@@ -816,11 +878,18 @@ private:
 	KAction* classWizard;
 	KAction* deleteSelectedWidget;
 	KAction* deleteDiagram;
-//Remove these once we stop supporting KDE 3.1
-// #if !KDE_IS_VERSION(3,1,90)
+
+	KAction* changeTabLeft;
+	KAction* changeTabRight;
+	KAction* moveTabLeft;
+	KAction* moveTabRight;
+#if KDE_IS_VERSION(3,1,90)
+	KToolBarButton* m_newSessionButton;
+	KPopupMenu* m_diagramMenu;
+	QToolButton* m_closeDiagramButton;
+#endif
 	KToggleAction* viewToolBar;
 	KToggleAction* viewStatusBar;
-// #endif
 	WorkToolBar* toolsbar;
 	QTimer* m_clipTimer;
 	QTimer* m_copyTimer;
@@ -832,8 +901,8 @@ private:
 	/**
 	 * True if the application is opening an existing document
 	 */
-	bool loading;
-	Settings::OptionState optionState;
+	bool m_loading;
+	Settings::OptionState m_optionState;
 
 	/**
 	 * Blank widget, displayed when there are no diagrams
@@ -841,9 +910,16 @@ private:
 	InfoWidget* blankWidget;
 
 	/**
-	 * Shows, and is parent of, all the UMLViews (diagrams).
+	 * Shows, and is parent of, all the UMLViews (diagrams)
+	 * if tabbed diagrams are not enabled.
 	 */
-	QWidgetStack* viewStack;
+	QWidgetStack* m_viewStack;
+
+	/**
+	 * Shows, and is parent of, all the UMLViews (diagrams)
+	 * if tabbed diagrams are enabled.
+	 */
+	KTabWidget* m_tabWidget;
 		
 
 	/**
@@ -851,13 +927,14 @@ private:
 	 */
 	QString m_imageMimetype;
 
-public:
 	/**
 	 * the global UML settings dialogue
 	 */
-	SettingsDlg* dlg;
+	SettingsDlg* m_dlg;
+
+public:
 	Settings::OptionState getOptionState() {
-		return optionState;
+		return m_optionState;
 	}
 
 signals:
