@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -18,16 +23,19 @@
 
 // app includes
 #include "class.h"
+#include "template.h"
+#include "attribute.h"
 #include "operation.h"
 #include "umldoc.h"
 #include "umllistview.h"
 #include "umllistviewitem.h"
 #include "umlobjectlist.h"
 #include "umlview.h"
+#include "model_utils.h"
 
 UMLListView* UMLListViewItem::s_pListView = 0;
 
-UMLListViewItem::UMLListViewItem( UMLListView * parent, QString name,
+UMLListViewItem::UMLListViewItem( UMLListView * parent, const QString &name,
 				  Uml::ListView_Type t, UMLObject* o)
   : QListViewItem(parent, name) {
 	init();
@@ -55,7 +63,7 @@ UMLListViewItem::UMLListViewItem(UMLListViewItem * parent)
 	init();
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
-UMLListViewItem::UMLListViewItem(UMLListViewItem * parent, QString name, Uml::ListView_Type t,UMLObject*o)
+UMLListViewItem::UMLListViewItem(UMLListViewItem * parent, const QString &name, Uml::ListView_Type t,UMLObject*o)
   : QListViewItem(parent, name) {
 	if (s_pListView == NULL) {
 		kdDebug() << "UMLListViewItem internal error 1: s_pListView is NULL" << endl;
@@ -65,7 +73,7 @@ UMLListViewItem::UMLListViewItem(UMLListViewItem * parent, QString name, Uml::Li
 	m_Type = t;
 	m_pObject = o;
 	if( !o ) {
-		m_nId = -1;
+		m_nId = Uml::id_None;
 		updateFolder();
 	} else {
 		updateObject();
@@ -81,7 +89,7 @@ UMLListViewItem::UMLListViewItem(UMLListViewItem * parent, QString name, Uml::Li
 	setText( name );
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
-UMLListViewItem::UMLListViewItem(UMLListViewItem * parent, QString name, Uml::ListView_Type t,int id)
+UMLListViewItem::UMLListViewItem(UMLListViewItem * parent, const QString &name, Uml::ListView_Type t,Uml::IDType id)
   : QListViewItem(parent, name) {
 	if (s_pListView == NULL) {
 		kdDebug() << "UMLListViewItem internal error 2: s_pListView is NULL" << endl;
@@ -91,15 +99,32 @@ UMLListViewItem::UMLListViewItem(UMLListViewItem * parent, QString name, Uml::Li
 	m_Type = t;
 	m_nId = id;
 	switch (m_Type) {
-		case lvt_Collaboration_Diagram: setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_Collaboration ) ); break;
-		case lvt_Class_Diagram: setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_Class ) ); break;
-		case lvt_State_Diagram: setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_State ) ); break;
-		case lvt_Activity_Diagram: setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_Activity ) ); break;
-		case lvt_Sequence_Diagram: setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_Sequence ) ); break;
-		case lvt_Component_Diagram: setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_Component ) ); break;
-		case lvt_Deployment_Diagram: setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_Deployment ) ); break;
-		case lvt_UseCase_Diagram: setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_Usecase ) ); break;
-		default: setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram ) );
+		case Uml::lvt_Collaboration_Diagram:
+			setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_Collaboration ) );
+			break;
+		case Uml::lvt_Class_Diagram:
+			setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_Class ) );
+			break;
+		case Uml::lvt_State_Diagram:
+			setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_State ) );
+			break;
+		case Uml::lvt_Activity_Diagram:
+			setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_Activity ) );
+			break;
+		case Uml::lvt_Sequence_Diagram:
+			setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_Sequence ) );
+			break;
+		case Uml::lvt_Component_Diagram:
+			setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_Component ) );
+			break;
+		case Uml::lvt_Deployment_Diagram:
+			setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_Deployment ) );
+			break;
+		case Uml::lvt_UseCase_Diagram:
+			setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram_Usecase ) );
+			break;
+		default:
+			setPixmap(0, s_pListView->getPixmap( UMLListView::it_Diagram ) );
 	}
 	/*
 		Constructor also used by folder so just make sure we don't need to
@@ -116,7 +141,7 @@ void UMLListViewItem::init() {
 	m_Type = Uml::lvt_Unknown;
 	m_bCreating = false;
 	m_pObject = NULL;
-	m_nId = -1;
+	m_nId = Uml::id_None;
 	m_nChildren = 0;
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -124,143 +149,163 @@ Uml::ListView_Type UMLListViewItem::getType() const {
 	return m_Type;
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
-int UMLListViewItem::getID() const {
+Uml::IDType UMLListViewItem::getID() const {
 	if (m_pObject)
 		return m_pObject->getID();
 	return m_nId;
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
-void UMLListViewItem::setID(int id) {
+void UMLListViewItem::setID(Uml::IDType id) {
 	if (m_pObject) {
-		int oid = m_pObject->getID();
-		if (id != -1 && oid != id)
-			kdDebug() << "UMLListViewItem::setID: new id " << id
-				  << " does not agree with object id " << oid << endl;
+		Uml::IDType oid = m_pObject->getID();
+		if (id != Uml::id_None && oid != id)
+			kdDebug() << "UMLListViewItem::setID: new id " << ID2STR(id)
+				  << " does not agree with object id " << ID2STR(oid)
+				  << endl;
 	}
 	m_nId = id;
 }
-////////////////////////////////////////////////////////////////////////////////////////////////////
+
+bool UMLListViewItem::isOwnParent(Uml::IDType listViewItemID) {
+	QListViewItem *lvi = (QListViewItem*)s_pListView->findItem(listViewItemID);
+	if (lvi == NULL) {
+		kdError() << "UMLListViewItem::isOwnParent: ListView->findItem("
+			  << ID2STR(listViewItemID) << ") returns NULL" << endl;
+		return true;
+	}
+	for (QListViewItem *self = (QListViewItem*)this; self; self = self->parent()) {
+		if (lvi == self)
+			return true;
+	}
+	return false;
+}
+
 void UMLListViewItem::updateObject() {
 	if( m_pObject == NULL )
 		return;
 
 	Uml::Scope scope = m_pObject->getScope();
-	setText( m_pObject->getName() );
+	Uml::Object_Type ot = m_pObject->getBaseType();
+	QString modelObjText = m_pObject->getName();
+	if (ot == Uml::ot_Operation || ot == Uml::ot_Attribute || ot == Uml::ot_Template) {
+		UMLClassifierListItem *pNarrowed = static_cast<UMLClassifierListItem*>(m_pObject);
+		modelObjText = pNarrowed->toString(Uml::st_SigNoScope);
+	}
+	setText(modelObjText);
 
-	switch( m_pObject->getBaseType() ) {
+	UMLListView::Icon_Type icon = UMLListView::it_Home;
+	switch (ot) {
 		case Uml::ot_Actor:
-			setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Actor ) );
+			icon = UMLListView::it_Actor;
 			break;
 
 		case Uml::ot_UseCase:
-			setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_UseCase ) );
+			icon = UMLListView::it_UseCase;
 			break;
 
 		case Uml::ot_Class:
-			setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Class ) );
+			icon = UMLListView::it_Class;
 			break;
 
 		case Uml::ot_Template:
-			setPixmap( 0, s_pListView->getPixmap(UMLListView::it_Template) );
+			icon = UMLListView::it_Template;
 			break;
 
 		case Uml::ot_Package:
-			setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Package ) );
+			icon =  UMLListView::it_Package;
 			break;
 
 		case Uml::ot_Component:
-			setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Component ) );
+			icon =  UMLListView::it_Component;
 			break;
 
 		case Uml::ot_Node:
-			setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Node ) );
+			icon =  UMLListView::it_Node;
 			break;
 
 		case Uml::ot_Artifact:
-			setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Artifact ) );
+			icon =  UMLListView::it_Artifact;
 			break;
 
 		case Uml::ot_Interface:
-			setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Interface ) );
+			icon =  UMLListView::it_Interface;
 			break;
 
 		case Uml::ot_Datatype:
-			setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Datatype ) );
+			icon =  UMLListView::it_Datatype;
 			break;
 
 		case Uml::ot_Enum:
-			setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Enum ) );
+			icon =  UMLListView::it_Enum;
+			break;
+
+		case Uml::ot_Entity:
+			icon =  UMLListView::it_Entity;
 			break;
 
 		case Uml::ot_Operation:
 			if( scope == Uml::Public )
-				setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Public_Method ) );
+				icon =  UMLListView::it_Public_Method;
 			else if( scope == Uml::Private )
-				setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Private_Method ) );
+				icon =  UMLListView::it_Private_Method;
 			else
-				setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Protected_Method ) );
+				icon =  UMLListView::it_Protected_Method;
 			break;
 
 		case Uml::ot_Attribute:
 			if( scope == Uml::Public )
-				setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Public_Attribute ) );
+				icon =  UMLListView::it_Public_Attribute;
 			else if( scope == Uml::Private )
-				setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Private_Attribute ) );
+				icon =  UMLListView::it_Private_Attribute;
 			else
-				setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Protected_Attribute ) );
+				icon =  UMLListView::it_Protected_Attribute;
 			break;
 		default:
 			break;
 	}//end switch
+	if (icon)
+		setPixmap(0, s_pListView->getPixmap(icon) );
 }
 
 void UMLListViewItem::updateFolder() {
+	UMLListView::Icon_Type icon = UMLListView::it_Home;
 	switch( m_Type ) {
 		case Uml::lvt_UseCase_View:
 		case Uml::lvt_UseCase_Folder:
-			if( isOpen() )
-				setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Folder_Grey_Open ) );
-			else
-				setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Folder_Grey ) );
+			icon = UMLListView::it_Folder_Grey;
 			break;
 
 		case Uml::lvt_Logical_View:
 		case Uml::lvt_Logical_Folder:
-			if( isOpen() )
-				setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Folder_Green_Open ) );
-			else
-				setPixmap( 0, s_pListView -> getPixmap( UMLListView::it_Folder_Green ) );
+			icon = UMLListView::it_Folder_Green;
 			break;
 
 		case Uml::lvt_Datatype_Folder:
-			if ( isOpen() )  {
-				setPixmap( 0, s_pListView->getPixmap(UMLListView::it_Folder_Orange_Open) );
-			} else {
-				setPixmap( 0, s_pListView->getPixmap(UMLListView::it_Folder_Orange) );
-			}
+			icon = UMLListView::it_Folder_Orange;
 			break;
 
 		case Uml::lvt_Component_View:
 		case Uml::lvt_Component_Folder:
-			if ( isOpen() ) {
-				setPixmap(0, s_pListView->getPixmap(UMLListView::it_Folder_Red_Open) );
-			} else {
-				setPixmap(0, s_pListView->getPixmap(UMLListView::it_Folder_Red) );
-			}
+			icon = UMLListView::it_Folder_Red;
 			break;
 
 		case Uml::lvt_Deployment_View:
 		case Uml::lvt_Deployment_Folder:
-			if ( isOpen() ) {
-				setPixmap(0, s_pListView->getPixmap(UMLListView::it_Folder_Violet_Open) );
-			} else {
-				setPixmap(0, s_pListView->getPixmap(UMLListView::it_Folder_Violet) );
-			}
+			icon = UMLListView::it_Folder_Violet;
+			break;
+
+		case Uml::lvt_EntityRelationship_Model:
+		case Uml::lvt_EntityRelationship_Folder:
+			icon = UMLListView::it_Folder_Cyan;
 			break;
 
 		default:
 			break;
 	}
+	if (icon) {
+		icon = (UMLListView::Icon_Type)((int)icon + (int)isOpen());
+		setPixmap(0, s_pListView->getPixmap(icon) );
+	}
 }
 
 void UMLListViewItem::setOpen( bool open ) {
@@ -268,7 +313,7 @@ void UMLListViewItem::setOpen( bool open ) {
 	updateFolder();
 }
 
-void UMLListViewItem::setText(QString newText) {
+void UMLListViewItem::setText(const QString &newText) {
 	m_Label = newText;
 	QListViewItem::setText(0, newText);
 }
@@ -279,22 +324,21 @@ QString UMLListViewItem::getText() const {
 
 void UMLListViewItem::okRename( int col ) {
 	QListViewItem::okRename( col );
+	UMLDoc* doc = s_pListView->getDocument();
 	if (m_bCreating) {
 		m_bCreating = false;
-		if ( s_pListView -> slotItemRenamed( this, col ) ) {
+		if ( s_pListView->slotItemRenamed( this, col ) ) {
 			m_Label = text(col);
+			doc->setModified(true);
 		}
 		return;
 	}
 	QString newText = text( col );
-        if ( newText == m_Label )
+        if ( newText == m_Label ) {
             return;
-	UMLDoc * doc = s_pListView -> getDocument();
+        }
 	if( newText.isEmpty() ) {
-		KMessageBox::error( kapp->mainWidget() ,
-				    i18n("The name you entered was invalid.\nRenaming process has been canceled."),
-				    i18n("Name Not Valid") );
-		setText( m_Label );
+		cancelRenameWithMsg();
 		return;
 	}
 	switch( m_Type ) {
@@ -310,6 +354,7 @@ void UMLListViewItem::okRename( int col ) {
 				return;
 			}
 			m_pObject -> setName( newText );
+			doc->setModified(true);
 			m_Label = newText;
 			break;
 
@@ -321,17 +366,45 @@ void UMLListViewItem::okRename( int col ) {
 			}
 			UMLOperation *op = static_cast<UMLOperation*>(m_pObject);
 			UMLClassifier *parent = static_cast<UMLClassifier *>( op -> parent() );
-			//see if op already has that name and not the op/att we are renaming
-			//then give a warning about the name being the same
-			UMLObjectList list = parent -> findChildObject( op->getBaseType(), newText );
-			if(list.isEmpty() || (!list.isEmpty() && KMessageBox::warningYesNo( kapp -> mainWidget() ,
-					      i18n( "The name you entered was not unique.\nIs this what you wanted?" ),
-					      i18n( "Name Not Unique" ) ) == KMessageBox::Yes )) {
-				op->setName( newText );
-				m_Label = newText;
+			Umbrello::OpDescriptor od;
+			Umbrello::Parse_Status st = Umbrello::parseOperation(newText, od, parent);
+			if (st == Umbrello::PS_OK) {
+			// TODO: Check that no operation with the exact same profile exists.
+				op->setName( od.m_name );
+				op->setType( od.m_pReturnType );
+				UMLAttributeList* parmList = op->getParmList();
+				unsigned i = 0;
+				if (parmList->count() > od.m_args.count()) {
+					for (i = od.m_args.count(); i < parmList->count(); i++) {
+						parmList->remove(i);
+					}
+					i = 0;
+				}
+				for (Umbrello::NameAndType_ListIt lit = od.m_args.begin();
+				     lit != od.m_args.end(); ++lit, ++i) {
+					const Umbrello::NameAndType& nm_tp = *lit;
+					UMLAttribute *a;
+					if (i < parmList->count()) {
+						a = parmList->at(i);
+						a->setName(nm_tp.m_name);
+						a->setType(nm_tp.m_type);
+						a->setInitialValue(nm_tp.m_initialValue);
+					} else {
+						a = new UMLAttribute(op);
+						a->setID( doc->getUniqueID() );
+						a->setName(nm_tp.m_name);
+						a->setType(nm_tp.m_type);
+						a->setInitialValue(nm_tp.m_initialValue);
+						op->addParm(a);
+					}
+				}
+				m_Label = op->toString(Uml::st_SigNoScope);
 			} else {
-				QListViewItem::setText(0, m_Label);
+				KMessageBox::error( kapp->mainWidget(),
+						    Umbrello::psText(st),
+						    i18n("Rename canceled") );
 			}
+			QListViewItem::setText(0, m_Label);
 			break;
 		}
 
@@ -342,13 +415,53 @@ void UMLListViewItem::okRename( int col ) {
 				return;
 			}
 			UMLClass *parent = static_cast<UMLClass*>( m_pObject -> parent() );
-			UMLObjectList list = parent -> findChildObject( m_pObject -> getBaseType(), newText );
-			if (! list.isEmpty()) {
+			Umbrello::NameAndType nt;
+			Umbrello::Parse_Status st = Umbrello::parseAttribute(newText, nt, parent);
+			if (st == Umbrello::PS_OK) {
+				UMLObjectList list = parent->findChildObject( m_pObject->getBaseType(), newText );
+				if (! list.isEmpty()) {
+					cancelRenameWithMsg();
+					return;
+				}
+				m_pObject->setName(nt.m_name);
+				UMLAttribute *pAtt = static_cast<UMLAttribute*>(m_pObject);
+				pAtt->setType(nt.m_type);
+				pAtt->setInitialValue(nt.m_initialValue);
+				m_Label = pAtt->toString(Uml::st_SigNoScope);
+			} else {
+				KMessageBox::error( kapp->mainWidget(),
+						    Umbrello::psText(st),
+						    i18n("Rename canceled") );
+			}
+			QListViewItem::setText(0, m_Label);
+			break;
+		}
+
+		case Uml::lvt_Template:
+		{
+			if (m_pObject == NULL) {
 				cancelRenameWithMsg();
 				return;
 			}
-			m_pObject -> setName( newText );
-			m_Label = newText;
+			UMLClassifier *parent = static_cast<UMLClassifier*>(m_pObject->parent());
+			Umbrello::NameAndType nt;
+			Umbrello::Parse_Status st = Umbrello::parseTemplate(newText, nt, parent);
+			if (st == Umbrello::PS_OK) {
+				UMLObjectList list = parent->findChildObject( m_pObject->getBaseType(), newText );
+				if (! list.isEmpty()) {
+					cancelRenameWithMsg();
+					return;
+				}
+				m_pObject->setName(nt.m_name);
+				UMLTemplate *tmpl = static_cast<UMLTemplate*>(m_pObject);
+				tmpl->setType(nt.m_type);
+				m_Label = tmpl->toString(Uml::st_SigNoScope);
+			} else {
+				KMessageBox::error( kapp->mainWidget(),
+						    Umbrello::psText(st),
+						    i18n("Rename canceled") );
+			}
+			QListViewItem::setText(0, m_Label);
 			break;
 		}
 
@@ -385,9 +498,13 @@ void UMLListViewItem::okRename( int col ) {
 			m_Label = newText;
 			break;
 		default:
-			cancelRenameWithMsg();
+			KMessageBox::error( kapp->mainWidget() ,
+			    i18n("Renaming an item of listview type %1 is not yet implemented.").arg(m_Type),
+			    i18n("Function Not Implemented") );
+			QListViewItem::setText(0, m_Label);
 			break;
 	}
+	doc->setModified(true);
 }
 
 void UMLListViewItem::cancelRenameWithMsg() {
@@ -407,12 +524,42 @@ void UMLListViewItem::cancelRename(int col) {
 // sort the listview items by type and alphabetically
 int UMLListViewItem::compare(QListViewItem *other, int col, bool ascending) const
 {
+	UMLListViewItem *ulvi = static_cast<UMLListViewItem*>(other);
 	Uml::ListView_Type ourType = getType();
-	Uml::ListView_Type otherType = static_cast<UMLListViewItem*>( other )->getType();
-
-	if ( ourType == otherType )
-		return key( col, ascending ).compare( other->key( col, ascending) );
+	Uml::ListView_Type otherType = ulvi->getType();
 
+	if ( ourType == otherType ) {
+		UMLObject *otherObj = ulvi->getUMLObject();
+		if (m_pObject == NULL || otherObj == NULL)
+			return key( col, ascending ).compare( other->key( col, ascending) );
+		if (ourType == Uml::lvt_Attribute) {
+			UMLClass *ourParent = dynamic_cast<UMLClass*>(m_pObject->parent());
+			UMLClass *otherParent = dynamic_cast<UMLClass*>(otherObj->parent());
+			if (ourParent == NULL || otherParent == NULL || ourParent != otherParent) {
+				kdError() << "UMLListViewItem::compare(UMLAttribute): ourParent="
+					  << ourParent << ", otherParent=" << otherParent << endl;
+				return key( col, ascending ).compare( other->key( col, ascending) );
+			}
+			UMLAttributeList atts = ourParent->getAttributeList();
+			int myIndex = atts.findRef( static_cast<UMLAttribute*>(m_pObject) );
+			int otherIndex = atts.findRef( static_cast<UMLAttribute*>(otherObj) );
+			return (myIndex < otherIndex ? -1 : myIndex > otherIndex ? 1 : 0);
+		} else if (ourType == Uml::lvt_Operation) {
+			UMLClassifier *ourParent = dynamic_cast<UMLClassifier*>(m_pObject->parent());
+			UMLClassifier *otherParent = dynamic_cast<UMLClassifier*>(otherObj->parent());
+			if (ourParent == NULL || otherParent == NULL || ourParent != otherParent) {
+				kdError() << "UMLListViewItem::compare(UMLOperation): ourParent="
+					  << ourParent << ", otherParent=" << otherParent << endl;
+				return key( col, ascending ).compare( other->key( col, ascending) );
+			}
+			UMLOperationList ops = ourParent->getOpList();
+			int myIndex = ops.findRef( static_cast<UMLOperation*>(m_pObject) );
+			int otherIndex = ops.findRef( static_cast<UMLOperation*>(otherObj) );
+			return (myIndex < otherIndex ? -1 : myIndex > otherIndex ? 1 : 0);
+		} else {
+			return key( col, ascending ).compare( other->key( col, ascending) );
+		}
+	}
 	if ( ourType < otherType )
 		return -1;
 	if ( ourType > otherType )
@@ -451,7 +598,7 @@ UMLListViewItem* UMLListViewItem::findUMLObject(UMLObject *o) {
 	return NULL;
 }
 
-UMLListViewItem * UMLListViewItem::findItem(int id) {
+UMLListViewItem * UMLListViewItem::findItem(Uml::IDType id) {
 	if (getID() == id)
 		return this;
 	UMLListViewItem *childItem = static_cast<UMLListViewItem*>(firstChild());
@@ -466,13 +613,18 @@ UMLListViewItem * UMLListViewItem::findItem(int id) {
 
 void UMLListViewItem::saveToXMI( QDomDocument & qDoc, QDomElement & qElement ) {
 	QDomElement itemElement = qDoc.createElement( "listitem" );
-	itemElement.setAttribute( "id", getID() );
+	Uml::IDType id = getID();
+	QString idStr = ID2STR(id);
+	//kdDebug() << "UMLListViewItem::saveToXMI: id = " << idStr
+	//	  << ", type = " << m_Type << endl;
+	if (id != Uml::id_None)
+		itemElement.setAttribute( "id", idStr );
 	itemElement.setAttribute( "type", m_Type );
 	if (m_pObject == NULL) {
 		// The predefined listview items such as "Logical View" etc. do
 		// not have a model counterpart thus their label is saved here.
 		itemElement.setAttribute( "label", m_Label );
-	} else if (m_pObject->getID() == -1) {
+	} else if (m_pObject->getID() == Uml::id_None) {
 		if (m_Label.isEmpty()) {
 			kdDebug() << "UMLListViewItem::saveToXMI(): Skipping empty item"
 				  << endl;
@@ -486,7 +638,7 @@ void UMLListViewItem::saveToXMI( QDomDocument & qDoc, QDomElement & qElement ) {
 	UMLListViewItem * childItem = static_cast<UMLListViewItem *> ( firstChild() );
 	while( childItem ) {
 		childItem -> saveToXMI( qDoc, itemElement );
-		childItem = static_cast<UMLListViewItem *> ( childItem -> nextSibling() );
+		childItem = dynamic_cast<UMLListViewItem *> ( childItem -> nextSibling() );
 	}
 	qElement.appendChild( itemElement );
 }
@@ -506,10 +658,12 @@ bool UMLListViewItem::loadFromXMI(QDomElement& qElement) {
 
 	m_nChildren = qElement.childNodes().count();
 
-	m_nId = id.toInt();
-	if (m_nId != -1)
-		m_pObject = s_pListView->getDocument()->findUMLObject( m_nId );
+	m_nId = STR2ID(id);
+	if (m_nId != Uml::id_None)
+		m_pObject = s_pListView->getDocument()->findObjectById( m_nId );
 	m_Type = (Uml::ListView_Type)(type.toInt());
+	if (m_pObject)
+		updateObject();
 	setOpen( (bool)open.toInt() );
 	return true;
 }
