@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -14,7 +19,9 @@
 #include <qdom.h>
 #include "umlnamespace.h"
 
+// forward declarations
 class UMLListView;
+class UMLObject;
 
 /**
  * Items used by the class @ref UMLListView.  This is needed as the type
@@ -36,7 +43,7 @@ public:
 	 * @param t		The type of this instance.
 	 * @param o		The object it represents.
 	 */
-	UMLListViewItem(UMLListView * parent, QString name, Uml::ListView_Type t, UMLObject*o=0);
+	UMLListViewItem(UMLListView * parent, const QString &name, Uml::ListView_Type t, UMLObject*o=0);
 
 	/**
 	 * Sets up an instance for subsequent loadFromXMI().
@@ -60,7 +67,7 @@ public:
 	 * @param t		The type of this instance.
 	 * @param o		The object it represents.
 	 */
-	UMLListViewItem(UMLListViewItem * parent, QString name, Uml::ListView_Type t, UMLObject*o=0);
+	UMLListViewItem(UMLListViewItem * parent, const QString &name, Uml::ListView_Type t, UMLObject*o=0);
 
 	/**
 	 * Sets up an instance.
@@ -70,7 +77,7 @@ public:
 	 * @param t		The type of this instance.
 	 * @param id		The id of this instance.
 	 */
-	UMLListViewItem(UMLListViewItem * parent, QString name, Uml::ListView_Type t, int id);
+	UMLListViewItem(UMLListViewItem * parent, const QString &name, Uml::ListView_Type t, Uml::IDType id);
 
 	/**
 	 * Standard deconstructor.
@@ -91,14 +98,14 @@ public:
 	 *
 	 * @return	The id this class represents.
 	 */
-	void setID(int id);
+	void setID(Uml::IDType id);
 
 	/**
 	 * Returns the id this class represents.
 	 *
 	 * @return	The id this class represents.
 	 */
-	int getID() const;
+	Uml::IDType getID() const;
 
 	/**
 	 * Set the UMLObject associated with this instance.
@@ -118,6 +125,12 @@ public:
 		return m_pObject;
 	}
 
+	/**
+	 * Returns true if the UMLListViewItem of the given ID is a parent of
+	 * this UMLListViewItem.
+	 */
+	bool isOwnParent(Uml::IDType listViewItemID);
+
 	/**
 	 * Updates the representation of the object.
 	 */
@@ -137,7 +150,7 @@ public:
 	/**
 	 * Changes the current text and updates the tooltip.
 	 */
-	void setText( QString text );
+	void setText( const QString &text );
 
 	/**
 	 * Returns the current text.
@@ -191,7 +204,7 @@ public:
 	 * @param id		The ID to search for.
 	 * @return	The item with the given ID or NULL if not found.
 	 */
-	UMLListViewItem * findItem(int id);
+	UMLListViewItem * findItem(Uml::IDType id);
 
 	/**
 	 * saves the listview item to a <listitem> tag
@@ -231,7 +244,8 @@ protected:
 	bool m_bCreating;
 
 	Uml::ListView_Type m_Type;
-	int m_nId, m_nChildren;
+	Uml::IDType m_nId;
+	int m_nChildren;
 	UMLObject * m_pObject;
 	QString m_Label;
 };
