@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,21 +15,16 @@
 #ifndef UMLNAMESPACE_H
 #define UMLNAMESPACE_H
 
+#include <string>
 #include <qstring.h>
 
-//uncomment for use with Luis' diagram code
-//namespace Umbrello
-//{
-//enum Visibility {Public = 200, Private, Protected };
-//}
-
 
 /**
  *@author Paul Hensgen
  * Bugs and comments to uml-devel@lists.sf.net or http://bugs.kde.org
  */
 namespace Uml {
-	enum UMLObject_Type
+	enum Object_Type
 	{
 	    ot_UMLObject  = 100,
 	    ot_Actor,
@@ -42,7 +42,10 @@ namespace Uml {
 	    ot_Component,
 	    ot_Artifact,
 	    ot_Node,
-	    ot_Stereotype
+	    ot_Stereotype,
+	    ot_Role,
+	    ot_Entity,
+	    ot_EntityAttribute
 	};
 
 	enum Scope
@@ -52,7 +55,7 @@ namespace Uml {
 	    Protected
 	};
 
-	enum UMLWidget_Type
+	enum Widget_Type
 	{
 	    wt_UMLWidget  =  300,	// does not have UMLObject representation
 	    wt_Actor,			// has UMLObject representation
@@ -61,6 +64,7 @@ namespace Uml {
 	    wt_Interface,		// has UMLObject representation
 	    wt_Datatype,		// has UMLObject representation
 	    wt_Enum,			// has UMLObject representation
+	    wt_Entity,			// has UMLObject representation
 	    wt_Package,			// has UMLObject representation
 	    wt_Object,			// has UMLObject representation
 	    wt_Note,			// does not have UMLObject representation
@@ -87,6 +91,7 @@ namespace Uml {
 	    dt_Activity,
 	    dt_Component,
 	    dt_Deployment,
+	    dt_EntityRelationship,
 	    dt_Undefined = -1
 	};
 
@@ -108,6 +113,7 @@ namespace Uml {
 	    at_Anchor,
 	    at_State,
 	    at_Activity,
+	    at_Relationship,
 	    at_Unknown  =  - 1
 	};
 
@@ -172,6 +178,11 @@ namespace Uml {
 	    lvt_Datatype,
 	    lvt_Datatype_Folder,
 	    lvt_Enum,
+	    lvt_Entity,
+	    lvt_EntityAttribute,
+	    lvt_EntityRelationship_Diagram,
+	    lvt_EntityRelationship_Folder,
+	    lvt_EntityRelationship_Model,
 	    lvt_Unknown = -1
 	};
 
@@ -186,7 +197,16 @@ namespace Uml {
 	{
 		//This is saved out to the file so only add new entries at the end
 		sequence_message_synchronous = 1000,
-		sequence_message_asynchronous
+		sequence_message_asynchronous,
+		sequence_message_creation
+	};
+
+	enum DBIndex_Type
+	{
+	    None  =  1100,
+	    Primary,
+	    Index,
+	    Unique
 	};
 
 	/**
@@ -201,7 +221,21 @@ namespace Uml {
 	 *   inout = operation both reads and writes the parameter
 	 * The numeric values of this enum are not currently saved to file.
 	 */
-	enum Parameter_Kind { pk_In, pk_InOut, pk_Out };
+	enum Parameter_Direction { pd_In, pd_InOut, pd_Out };
+
+	/**
+	 * The data type used for unique IDs.
+	 */
+	typedef std::string IDType;
+	/**
+	 * Reserved value for uninitialized/illegal ID.
+	 */
+	const IDType id_None = "-1";
+	const IDType id_Reserved = "0";
+
+#	define STR2ID(id)  id.ascii()
+#	define ID2STR(id)  QString(id.c_str())
+
 
 	/**
 	 * Function for comparing tags in XMI files.
