@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -16,66 +21,72 @@
 #include "umllistviewitem.h"
 #include "package.h"
 #include "stereotype.h"
+#include "model_utils.h"
 
-UMLObject::UMLObject(const UMLObject * parent, const QString &name, int id)
+UMLObject::UMLObject(const UMLObject * parent, const QString &name, Uml::IDType id)
   : QObject(const_cast<UMLObject*>(parent), "UMLObject" ) {
 	init();
-	m_nId = id;
+	if (id == Uml::id_None)
+		m_nId = UMLApp::app()->getDocument()->getUniqueID();
+	else
+		m_nId = id;
 	m_Name = name;
 }
 
-UMLObject::UMLObject(const QString &name, int id)
+UMLObject::UMLObject(const QString &name, Uml::IDType id)
   :  QObject(UMLApp::app()->getDocument(), "UMLObject") {
 	init();
-	m_nId = id;
+	if (id == Uml::id_None)
+		m_nId = UMLApp::app()->getDocument()->getUniqueID();
+	else
+		m_nId = id;
 	m_Name = name;
 }
 
-////////////////////////////////////////////////////////////////////////////////////////////////////
 UMLObject::UMLObject(const UMLObject * parent)
   : QObject(const_cast<UMLObject*>(parent)) {
 	init();
 }
 
-////////////////////////////////////////////////////////////////////////////////////////////////////
 UMLObject::~UMLObject() {
 }
 
 void UMLObject::init() {
-	m_BaseType = ot_UMLObject;
-	m_nId = -1;
+	m_BaseType = Uml::ot_UMLObject;
+	m_nId = Uml::id_None;
 	m_pUMLPackage = NULL;
 	m_Name = "";
-	m_Scope = Public;
+	m_Scope = Uml::Public;
 	m_pStereotype = NULL;
 	m_Doc = "";
 	m_bAbstract = false;
 	m_bStatic = false;
 	m_bInPaste = false;
+	m_bCreationWasSignalled = false;
+	m_pSecondary = NULL;
 }
 
-////////////////////////////////////////////////////////////////////////////////////////////////////
 
 bool UMLObject::acceptAssociationType(Uml::Association_Type)
 {// A UMLObject accepts nothing. This should be reimplemented by the subclasses
 	return false;
 }
-////////////////////////////////////////////////////////////////////////////////////////////////////
-void UMLObject::setID(int NewID) {
+
+void UMLObject::setID(Uml::IDType NewID) {
 	m_nId = NewID;
 	emit modified();
 }
-////////////////////////////////////////////////////////////////////////////////////////////////////
-void UMLObject::setName(QString strName) {
+
+void UMLObject::setName(const QString &strName) {
 	m_Name = strName;
 	emit modified();
 }
-////////////////////////////////////////////////////////////////////////////////////////////////////
+
 QString UMLObject::getName() const {
 	return m_Name;
 }
 
-QString UMLObject::getFullyQualifiedName(QString separator /* = "::" */) const {
+QString UMLObject::getFullyQualifiedName(const QString &separator /* = "." */) const {
 	QString fqn;
 	if (m_pUMLPackage) {
 		fqn = m_pUMLPackage->getFullyQualifiedName(separator);
@@ -85,7 +96,6 @@ QString UMLObject::getFullyQualifiedName(QString separator /* = "::" */) const {
 	return fqn;
 }
 
-////////////////////////////////////////////////////////////////////////////////////////////////////
 bool UMLObject::operator==(UMLObject & rhs ) {
 	if( this == &rhs )
 		return true;
@@ -184,7 +194,7 @@ bool UMLObject::getStatic() const
 	return m_bStatic;
 }
 /** Sets the value for m_bStatic. */
-void UMLObject::setStatic(const bool bStatic)
+void UMLObject::setStatic(bool bStatic)
 {
 	m_bStatic = bStatic;
 	emit modified();
@@ -195,16 +205,16 @@ void UMLObject::emitModified()
 	emit modified();
 }
 
-void UMLObject::setDoc(QString d) {
+void UMLObject::setDoc(const QString &d) {
 	m_Doc = d;
 	emit modified();
 }
 
-UMLObject_Type UMLObject::getBaseType() const {
+Uml::Object_Type UMLObject::getBaseType() const {
 	return m_BaseType;
 }
 
-int UMLObject::getID() const {
+Uml::IDType UMLObject::getID() const {
 	return m_nId;
 }
 
@@ -212,11 +222,11 @@ QString UMLObject::getDoc() const {
 	return m_Doc;
 }
 
-Scope UMLObject::getScope() const {
+Uml::Scope UMLObject::getScope() const {
 	return m_Scope;
 }
 
-void UMLObject::setScope(Scope s) {
+void UMLObject::setScope(Uml::Scope s) {
 	m_Scope = s;
 	emit modified();
 }
@@ -239,7 +249,7 @@ void UMLObject::setUMLStereotype(UMLStereotype *stereo) {
 	emit modified();
 }
 
-void UMLObject::setStereotype(QString _name) {
+void UMLObject::setStereotype(const QString &_name) {
 	if (_name.isEmpty()) {
 		setUMLStereotype(NULL);
 		return;
@@ -249,21 +259,21 @@ void UMLObject::setStereotype(QString _name) {
 	setUMLStereotype(s);
 }
 
-void UMLObject::setPackage(QString _name) {
+void UMLObject::setPackage(const QString &_name) {
 	// TBD: Resolve nested packages given in _name (e.g. A::B::C)
 	UMLObject *pkgObj = NULL;
-	if (_name != "") {
+	if (!_name.isEmpty()) {
 		UMLDoc* umldoc = UMLApp::app()->getDocument();
 		if (umldoc == NULL) {
 			kdError() << "UMLObject::setPackage: cannot set package name on "
 				  << m_Name << endl;
 			return;
 		}
-		pkgObj = umldoc->findUMLObject(_name, ot_Package);
+		pkgObj = umldoc->findUMLObject(_name, Uml::ot_Package);
 		if (pkgObj == NULL) {
 			kdDebug() << "UMLObject::setPackage: creating UMLPackage "
 				  << _name << " for " << m_Name << endl;
-			pkgObj = umldoc->createUMLObject(ot_Package, _name);
+			pkgObj = umldoc->createUMLObject(Uml::ot_Package, _name);
 		}
 	}
 	setUMLPackage( static_cast<UMLPackage *>(pkgObj) );
@@ -278,13 +288,16 @@ const UMLStereotype * UMLObject::getUMLStereotype() {
 	return m_pStereotype;
 }
 
-QString UMLObject::getStereotype() {
+QString UMLObject::getStereotype(bool includeAdornments /* = true */) {
 	if (m_pStereotype == NULL)
 		return "";
-	return m_pStereotype->getName();
+	QString name = m_pStereotype->getName();
+	if (includeAdornments)
+		name = "" + name + "";
+	return name;
 }
 
-QString UMLObject::getPackage(QString separator /* ="::" */) {
+QString UMLObject::getPackage(const QString &separator /* ="." */) {
 	if (m_pUMLPackage == NULL)
 		return "";
 	QStringList pkgList;
@@ -310,44 +323,134 @@ UMLPackage* UMLObject::getUMLPackage() {
 	return m_pUMLPackage;
 }
 
-QString UMLObject::getAuxId() const {
-	return m_AuxId;
+QString UMLObject::getSecondaryId() const {
+	return m_SecondaryId;
 }
 
-QDomElement UMLObject::save( QString tag, QDomDocument & qDoc ) {
+void UMLObject::maybeSignalObjectCreated() {
+	if (!m_bCreationWasSignalled &&
+	    m_BaseType != Uml::ot_Stereotype &&
+	    m_BaseType != Uml::ot_Association &&
+	    m_BaseType != Uml::ot_Role) {
+		UMLDoc* umldoc = UMLApp::app()->getDocument();
+		umldoc->signalUMLObjectCreated(this);
+		m_bCreationWasSignalled = true;
+	}
+}
+
+bool UMLObject::resolveRef() {
+	if (m_pSecondary || m_SecondaryId.isEmpty()) {
+		maybeSignalObjectCreated();
+		return true;
+	}
+#ifdef VERBOSE_DEBUGGING
+	kdDebug() << "UMLObject::resolveRef(" << m_Name << "): m_SecondaryId is "
+		  << m_SecondaryId << endl;
+#endif
+	UMLDoc *pDoc = UMLApp::app()->getDocument();
+	// In the new, XMI standard compliant save format,
+	// the type is the xmi.id of a UMLClassifier.
+	Uml::IDType id = STR2ID(m_SecondaryId);
+	m_pSecondary = pDoc->findObjectById(id);
+	if (m_pSecondary != NULL) {
+		m_SecondaryId = "";
+		maybeSignalObjectCreated();
+		return true;
+	}
+	if (!pDoc->isNativeXMIFile() || !m_SecondaryId.contains(QRegExp("\\D"))) {
+		kdError() << "UMLObject::resolveRef(" << m_Name
+			  << "): cannot find type with id "
+			  << ID2STR(id) << endl;
+		return false;
+	}
+	// Assume we're dealing with the older Umbrello format where
+	// the type name was saved in the "type" attribute rather
+	// than the xmi.id of the model object of the attribute type.
+	m_pSecondary = pDoc->findUMLObject( m_SecondaryId, Uml::ot_UMLObject, this );
+	if (m_pSecondary) {
+		m_SecondaryId = "";
+		maybeSignalObjectCreated();
+		return true;
+	}
+	// Work around UMLDoc::createUMLObject()'s incapability
+	// of on-the-fly scope creation:
+	if (m_SecondaryId.contains("::")) {
+		m_SecondaryId.replace("::", ".");
+		m_pSecondary = pDoc->findUMLObject( m_SecondaryId, Uml::ot_UMLObject, this );
+		if (m_pSecondary) {
+			m_SecondaryId = "";
+			maybeSignalObjectCreated();
+			return true;
+		}
+	}
+	kdDebug() << "UMLObject::resolveRef: Creating new type for "
+		  << m_SecondaryId << endl;
+	// This is very C++ specific - we rely on  some '*' or
+	// '&' to decide it's a ref type. Plus, we don't recognize
+	// typedefs of ref types.
+	bool isReferenceType = ( m_SecondaryId.contains('*') ||
+				 m_SecondaryId.contains('&') );
+	Uml::Object_Type ot = Uml::ot_Class;
+	if (isReferenceType) {
+		ot = Uml::ot_Datatype;
+	} else {
+		// Make data type for easily identified cases
+		const int n_types = 12;
+		const char *types[] = {
+			"void", "bool",
+			"char", "unsigned char",
+			"short", "unsigned short",
+			"int", "unsigned int",
+			"long", "unsigned long",
+			"float", "double"
+		};
+		int i = 0;
+		for (; i < n_types; i++) {
+			if (m_SecondaryId == types[i])
+				break;
+		}
+		if (i < n_types)
+			ot = Uml::ot_Datatype;
+	}
+	m_pSecondary = pDoc->createUMLObject(ot, m_SecondaryId, NULL, true);
+	// The `prepend' flag is set true because we need to move the
+	// newly created item to before the current item in UMLDoc's
+	// object list.  This can be dropped when the deferred type
+	// resolution is generalized to also cover native ID resolution.
+	if (m_pSecondary == NULL)
+		return false;
+	m_SecondaryId = "";
+	maybeSignalObjectCreated();
+	return true;
+}
+
+QDomElement UMLObject::save( const QString &tag, QDomDocument & qDoc ) {
 	/*
 	  Call as the first action of saveToXMI() in child class:
 	  This creates the QDomElement with which to work.
 	*/
 	QDomElement qElement = qDoc.createElement(tag);
-
-	qElement.setAttribute( "xmi.id", m_nId );
-	if (!m_Name.isEmpty())
-		qElement.setAttribute( "name", m_Name );
+	qElement.setAttribute( "isSpecification", "false" );
+	if (m_BaseType != Uml::ot_Association && m_BaseType != Uml::ot_Role) {
+		qElement.setAttribute( "isLeaf", "false" );
+		qElement.setAttribute( "isRoot", "false" );
+		if (m_bAbstract)
+			qElement.setAttribute( "isAbstract", "true" );
+		else
+			qElement.setAttribute( "isAbstract", "false" );
+	}
+	qElement.setAttribute( "xmi.id", ID2STR(m_nId) );
+	qElement.setAttribute( "name", m_Name );
 	if (! m_Doc.isEmpty())
 		qElement.setAttribute( "comment", m_Doc );  //CHECK: uml13.dtd compliance
 #ifdef XMI_FLAT_PACKAGES
 	if (m_pUMLPackage)             //FIXME: uml13.dtd compliance
 		qElement.setAttribute( "package", m_pUMLPackage->getID() );
 #endif
-	switch (m_Scope) {
-		case Uml::Public:
-			qElement.setAttribute( "visibility", "public" );
-			break;
-		case Uml::Protected:
-			qElement.setAttribute( "visibility", "protected" );
-			break;
-		case Uml::Private:
-			qElement.setAttribute( "visibility", "private" );
-			break;
-	}
+        QString visibility = Umbrello::scopeToString(m_Scope, false);
+	qElement.setAttribute( "visibility", visibility);
 	if (m_pStereotype != NULL)
-		qElement.setAttribute( "stereotype", m_pStereotype->getID() );
-	if (m_bAbstract)
-		qElement.setAttribute( "isAbstract", "true" );
-	/* else
-		qElement.setAttribute( "isAbstract", "false" );
-	 *** isAbstract defaults to false if not set **********/
+		qElement.setAttribute( "stereotype", ID2STR(m_pStereotype->getID()) );
 	if (m_bStatic)
 		qElement.setAttribute( "ownerScope", "classifier" );
 	/* else
@@ -362,7 +465,7 @@ bool UMLObject::load( QDomElement& ) {
 	return true;
 }
 
-bool UMLObject::loadFromXMI( QDomElement & element, bool loadID /* =true */) {
+bool UMLObject::loadFromXMI( QDomElement & element) {
 	UMLDoc* umldoc = UMLApp::app()->getDocument();
 	if (umldoc == NULL) {
 		kdError() << "UMLObject::loadFromXMI: umldoc is NULL" << endl;
@@ -371,19 +474,19 @@ bool UMLObject::loadFromXMI( QDomElement & element, bool loadID /* =true */) {
 	// Read the name first so that if we encounter a problem, the error
 	// message can say the name.
 	m_Name = element.attribute( "name", "" );
-	if (loadID) {
-		QString id = element.attribute( "xmi.id", "" );
-		if (id.isEmpty() || id == "-1") {
+	QString id = element.attribute( "xmi.id", "" );
+	if (id.isEmpty() || id == "-1") {
+		if (m_BaseType == Uml::ot_Role) {
+			// Before version 1.4, Umbrello did not save the xmi.id
+			// of UMLRole objects.
+			m_nId = umldoc->getUniqueID();
+		} else {
 			kdError() << "UMLObject::loadFromXMI(" << m_Name
 				  << "): nonexistent or illegal xmi.id" << endl;
 			return false;
 		}
-		if (id.contains(QRegExp("\\D"))) {
-			m_AuxId = id;
-			m_nId = umldoc->getUniqueID();
-		} else {
-			m_nId = id.toInt();
-		}
+	} else {
+		m_nId = STR2ID(id);
 	}
 
 	if (element.hasAttribute("documentation"))  // for bkwd compat.
@@ -401,7 +504,7 @@ bool UMLObject::loadFromXMI( QDomElement & element, bool loadID /* =true */) {
 		else {
 			int nScope = scope.toInt();
 			if (nScope >= Uml::Public && nScope <= Uml::Protected)
-				m_Scope = (Scope)nScope;
+				m_Scope = (Uml::Scope)nScope;
 			else
 				kdError() << "UMLObject::loadFromXMI(" << m_Name
 					<< "): illegal scope" << endl;  // soft error
@@ -422,14 +525,14 @@ bool UMLObject::loadFromXMI( QDomElement & element, bool loadID /* =true */) {
 			// Old versions saved the stereotype name instead of the xmi.id.
 			setStereotype( stereo );
 		} else {
-			int stereoID = stereo.toInt();
-			m_pStereotype = umldoc->findStereotype(stereoID);
+			Uml::IDType stereoID = STR2ID(stereo);
+			m_pStereotype = umldoc->findStereotypeById(stereoID);
 			if (m_pStereotype)
 				m_pStereotype->incrRefCount();
 			else
 				kdError() << "UMLObject::loadFromXMI(" << m_Name << "): "
-					  << "UMLStereotype " << stereoID << " not found"
-					  << endl;
+					  << "UMLStereotype " << ID2STR(stereoID)
+					  << " not found" << endl;
 		}
 	}
 
@@ -452,9 +555,9 @@ bool UMLObject::loadFromXMI( QDomElement & element, bool loadID /* =true */) {
 	/**** Handle XMI_FLAT_PACKAGES and old files *************************/
 	QString pkg = element.attribute( "packageid", "-1" );
 	// Some interim versions used "packageid" so test for it.
-	int pkgId = -1;
+	Uml::IDType pkgId = Uml::id_None;
 	if (pkg != "-1") {
-		pkgId = pkg.toInt();
+		pkgId = STR2ID(pkg);
 	} else {
 		pkg = element.attribute( "package", "" );
 		if (! pkg.isEmpty()) {
@@ -462,39 +565,39 @@ bool UMLObject::loadFromXMI( QDomElement & element, bool loadID /* =true */) {
 				// Old versions saved the package name instead of the xmi.id.
 				setPackage( pkg );
 			else
-				pkgId = pkg.toInt();
+				pkgId = STR2ID(pkg);
 		}
 	}
-	if (pkgId != -1) {
-		UMLObject *pkgObj = umldoc->findUMLObject( pkgId );
+	if (pkgId != Uml::id_None) {
+		UMLObject *pkgObj = umldoc->findObjectById( pkgId );
 		if (pkgObj != NULL) {
 			m_pUMLPackage = dynamic_cast<UMLPackage *>(pkgObj);
 			if (m_pUMLPackage == NULL)  // soft error
 				kdError() << "UMLObject::loadFromXMI(" << m_Name
 					  << "): object of packageid "
-					  << pkgId << " is not a package" << endl;
+					  << ID2STR(pkgId) << " is not a package" << endl;
 		} else {  // soft error
 			kdError() << "UMLObject::loadFromXMI(" << m_Name
 				  << "): cannot resolve packageid "
-				  << pkgId << endl;
+				  << ID2STR(pkgId) << endl;
 		}
 	}
 	/**** End of XMI_FLAT_PACKAGES and old files handling ****************/
 
 	// If the name is not set, let's check whether the attributes are saved
 	// as child nodes.
-	// We only do this for the regular loadFromXMI call (when the loading
-	// of the xmi.id is done.)
-	if (loadID && m_Name.isEmpty()) {
+	if (m_Name.isEmpty()) {
 		QDomNode node = element.firstChild();
+		if (node.isComment())
+			node = node.nextSibling();
 		QDomElement elem = node.toElement();
 		while( !elem.isNull() ) {
 			QString tag = elem.tagName();
-			if (tagEq(tag, "name")) {
+			if (Uml::tagEq(tag, "name")) {
 				m_Name = elem.attribute("xmi.value", "");
 				if (m_Name.isEmpty())
 					m_Name = elem.text();
-			} else if (tagEq(tag, "visibility")) {
+			} else if (Uml::tagEq(tag, "visibility")) {
 				QString vis = elem.attribute("xmi.value", "");
 				if (vis.isEmpty())
 					vis = elem.text();
@@ -502,36 +605,39 @@ bool UMLObject::loadFromXMI( QDomElement & element, bool loadID /* =true */) {
 					m_Scope = Uml::Private;
 				else if (vis == "protected" || vis == "protected_vis")
 					m_Scope = Uml::Protected;
-			} else if (tagEq(tag, "isAbstract")) {
+			} else if (Uml::tagEq(tag, "isAbstract")) {
 				QString isAbstract = elem.attribute("xmi.value", "");
 				if (isAbstract.isEmpty())
 					isAbstract = elem.text();
 				m_bAbstract = (isAbstract == "true");
-			} else if (tagEq(tag, "ownerScope")) {
+			} else if (Uml::tagEq(tag, "ownerScope")) {
 				QString ownerScope = elem.attribute("xmi.value", "");
 				if (ownerScope.isEmpty())
 					ownerScope = elem.text();
 				m_bStatic = (ownerScope == "classifier");
 			}
 			node = node.nextSibling();
+			if (node.isComment())
+				node = node.nextSibling();
 			elem = node.toElement();
 		}
 	}
 
 	// Operations, attributes, enum literals, stereotypes, associations,
 	// and association role objects get added and signaled elsewhere.
-	if (m_BaseType != ot_Operation && m_BaseType != ot_Attribute &&
-	    m_BaseType != ot_EnumLiteral && m_BaseType != ot_Stereotype &&
-	    m_BaseType != ot_Association && m_BaseType != ot_UMLObject) {
+	if (m_BaseType != Uml::ot_Operation && m_BaseType != Uml::ot_Attribute &&
+	    m_BaseType != Uml::ot_EnumLiteral && m_BaseType != Uml::ot_EntityAttribute &&
+	    m_BaseType != Uml::ot_Stereotype &&
+	    m_BaseType != Uml::ot_Association && m_BaseType != Uml::ot_Role) {
 		if (m_bInPaste) {
 			m_pUMLPackage = NULL;  // forget any old parent
 			UMLListView *listView = UMLApp::app()->getListView();
 			UMLListViewItem *parentItem = (UMLListViewItem*)listView->currentItem();
 			if (parentItem) {
-				ListView_Type lvt = parentItem->getType();
-				if (lvt == lvt_Package ||
-				    lvt == lvt_Class ||
-				    lvt == lvt_Interface) {
+				Uml::ListView_Type lvt = parentItem->getType();
+				if (lvt == Uml::lvt_Package ||
+				    lvt == Uml::lvt_Class ||
+				    lvt == Uml::lvt_Interface) {
 					UMLObject *o = parentItem->getUMLObject();
 					m_pUMLPackage = static_cast<UMLPackage*>( o );
 				}
@@ -542,7 +648,6 @@ bool UMLObject::loadFromXMI( QDomElement & element, bool loadID /* =true */) {
 			m_pUMLPackage->addObject(this);
 		else
 			umldoc->addUMLObject(this);
-		umldoc->signalUMLObjectCreated(this);
 	}
 	return load(element);
 }
