@@ -906,6 +906,14 @@ public:
 		m_pOnWidget = w;
 	}
 
+	/**
+	 * Tests the given point against all widgets and returns the
+	 * widget for which the point is within its bounding rectangle.
+	 * Returns NULL if the point is not inside any widget.
+	 * Does not use or modify the m_pOnWidget member.
+	 */
+	UMLWidget *testOnWidget(QPoint p);
+
 	/**
 	 * Initialize and announce a newly created widget.
 	 * Auxiliary to contentsMouseReleaseEvent().
