@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -32,7 +37,7 @@ public:
 	 *	@param name		The name of the object.
 	 *	@param	id				The id of the object.
 	 */
-	UMLUseCase(const QString & name = "", int id = -1);
+	UMLUseCase(const QString & name = "", Uml::IDType id = Uml::id_None);
 
 	/**
 	 *	Standard deconstructor
